(* Declare.m, with changes, date Dec 13, 2005 *)

(* Comment added by Rolf Sulanke, December 13, 2005:
Today the search for Declare.m in The Mathematica Information center yields 
the old package starting with
"BeginPackage["PJ`Declare`"] (* preliminarly tested on 2.0 *)" 
The new URL for this original package is
http://library.wolfram.com/infocenter/MathSource/4574/
An adaption to Mathematica v. 4.1 and some complements are added by 
Rolf Sulanke, marked in corresponding comments below by R. S. The package 
has been applied in several notebooks of R. S. It has been very useful 
at some places and seems to be without errors.  
*)

(* Comment added by Rolf Sulanke, November 12, 1999:
The packages Declare.m, author Pekka Janhunen, has been downloaded from
mathsource, v.2.2, Enhancements/Algebraic 0202-149.
It uses the following packages belonging to the Mathematica distribution
R. Maeder und M. Buchholz:
/usr/local/mathematica/AddOns/StandardPackages/Algebra/ReIm.m 
R. Maeder:
/usr/local/mathematica/AddOns/ExtraPackages/ProgrammingInMathematica/ReIm.m 
(See also ReImDoc.txt). MathSource 2.2: Declaring Properties of Variables
/Enhancements/Algebraic/0202-149 
Declaring Properties of Variables Pekka Janhunen 
Allows you to declare properties for Mathematica variables and uses those
declarations 'intelligently'. For example, if you say Declare[x, Real], then
Sqrt[x^2] will simplify to Abs[x] and so forth. * Declare.m (6 Kb...
http://www.mathsource.com/Content22/Enhancements/Algebraic/0202-149, 1223
bytes 
MathSource 2.2: Complex Number Simplifications
... and enables these functions to make more complete use of known properties
of 
symbols and functions. For example, the input Im[z] ^= 0 can be used to
declare that z is real, after which Re[z] will evaluate to z. The function
RealValued is also included for declaring functions to be real-valued for
real... http://www.mathsource.com/Content22/Enhancements/Algebraic/0200-035,
1669 bytes 
Certain commands are added by Rolf Sulanke, see Elementary Functions, FFF 
below. The original downloaded file is saved under the name Declare.mold.
End of the comment. Next changes added Dec 9, 2002. *)

(* Recherche May 2004: Pekka Janhunen, Finn. Meteorological Institute,
Pekka.Janhunen@fmi.fi
http://www.geo.fmi.fi/~pjanhune *)

(* The original line "BeginPackage["PJ`Declare`"] (* preliminarly tested on 2.0 *)"
 has been replaced by the next one, R. S. Dec 13,2005 *)

BeginPackage["Declare`"] 

Declare::usage =
"Declare[a,Odd] or Declare[{x,y},Real] etc. Possible declarations are
Odd,Even,Integer,Real,Positive,Negative,NonPositive,NonNegative.
Declare[x,Real,{n,m},Integer] etc. are also possible. Declare does not
remove any previous properties, for example after (Declare[x,Integer];
Declare[x,Real]) IntegerQ[x] is still true. See also NewDeclare."

NewDeclare::usage =
"NewDeclare is the same as Declare except that it removes any previous
properties of the symbols using Clear. Only single symbols may be used
with NewDeclare."

NonPositive::usage = "NonPositive[x] is True if x >= 0 ."

RealQ::usage = "RealQ[x] is True if x is real."

Odd::usage = "Odd is a possible second argument for Declare."

Even::usage = "Even is a possible second argument for Declare."

Uncertain::usage = "Uncertain can be returned by Positive etc."

Begin["`Private`"]

Declare[x_List, attr_Symbol]:=
  (Declare[#,attr]& /@ x; Null)

Declare[a_,Odd]:=
  (OddQ[a]^= True; EvenQ[a]^= False; IntegerQ[a]^= True;
   RealQ[a]^= True; Null)

Declare[a_, Even]:=
  (OddQ[a]^= False; EvenQ[a]^= True; IntegerQ[a]^= True;
   RealQ[a]^= True; Null)
 
Declare[a_, Integer]:=
  (IntegerQ[a]^= True; RealQ[a]^= True;
   Null)

Declare[a_, Real]:=
  (RealQ[a]^= True; Null)

Declare[a_, Positive]:=
   (Positive[a]^= True; Negative[a]^= False;
    NonPositive[a]^= False; NonNegative[a]^= True;
    RealQ[a]^= True; Sign[a]^= 1; Null)

Declare[a_, Negative]:=
  (Negative[a]^= True; Positive[a]^= False;
   NonNegative[a]^= False; NonPositive[a]^= True;
   RealQ[a]^= True; Sign[a]^= -1; Null)

Declare[a_, NonNegative]:=
  (NonNegative[a]^= True; Negative[a]^= False;
   RealQ[a]^= True; Null)

Declare[a_, NonPositive]:=
  (NonPositive[a]^= True; Positive[a]^= False;
   RealQ[a]^= True; Null)

Declare[a_, attr_Symbol]:= (Message[Declare::undef,attr];Null)

Declare[args__]:= (((Declare @@ #&) /@ Partition[{args},2]);Null) /;
  EvenQ[Length[{args}]]

Declare::undef = "Undefined declaration `1`."

NewDeclare[x_Symbol, attr_Symbol]:= (Clear[x]; Declare[x,attr]; Null)

NewDeclare[x:{_Symbol..}, attr_Symbol]:=
  (Clear@@x; Declare[#,attr]& /@ x; Null)

NewDeclare[args__]:= (((NewDeclare @@ #&) /@ Partition[{args},2]);Null) /;
  EvenQ[Length[{args}]]

(* Rules for Plus, Times, Power *)

Unprotect[NonNegative]
NonNegative[x_ + y_]:=
  If[NonNegative[x],
     If[NonNegative[y],True,Uncertain,Uncertain],
     (*x negative*)If[NonPositive[y],False,Uncertain,Uncertain],
     If[NonPositive[x],If[Negative[y],False,Uncertain,Uncertain],
        Impossible,Uncertain] ]
 NonNegative[x_ y_]:=
   If[NonNegative[x],
      NonNegative[y],
      (*x negative*)
      If[NonPositive[y],True,False,
         If[NonNegative[y],Uncertain,Impossible,Uncertain] ],
      If[NonPositive[x],
         If[NonPositive[y],True,Uncertain,
            If[NonNegative[y],Uncertain,Impossible,Uncertain] ]] ]
NonNegative[_?NonNegative ^ _?RealQ]:= True
NonNegative[_?RealQ ^ _?EvenQ]:= True
NonNegative[_?Negative ^ _?OddQ]:= False
NonNegative[Abs[_]]:= True
NonNegative[x_]:= Uncertain /; !NumberQ[x]   (* a general rule *)
Protect[NonNegative]

(* Added by R.S., Jan 29 2008: *)
Unprotect[GreaterEqual]
x_ >= 0 := NonNegative[x]
Protect[GreaterEqual]

Unprotect[Negative]
Negative[x_ + y_]:=
  If[NonNegative[x],
     Negative[y],
     (*x negative*)If[NonPositive[y],True,Uncertain,Uncertain],
     If[NonPositive[x],If[Negative[y],True,Uncertain,Uncertain],
        Impossible,Uncertain] ]
 Negative[x_ y_]:=
   If[NonNegative[x],
      Negative[y],
      (*x negative*)
      If[NonPositive[y],False,True,
         If[NonNegative[y],Uncertain,Impossible,Uncertain] ],
      If[NonPositive[x],
         If[NonPositive[y],False,Uncertain,
            If[NonNegative[y],Uncertain,Impossible,Uncertain] ]] ]
Negative[_?NonNegative ^ _?RealQ]:= False
Negative[_?RealQ ^ _?EvenQ]:= False
Negative[_?Negative ^ _?OddQ]:= True
Negative[Abs[_]]:= False
Negative[x_]:= Uncertain /; !NumberQ[x]
Protect[Negative]

(* Added by R.S., Jan 29 2008: *)
Unprotect[Less]
x_ < 0 := Negative[x]
Protect[Less]

Unprotect[NonPositive]
NonPositive[x_]:= !Positive[x]
Protect[NonPositive]

(* Added by R.S., Jan 29 2008: *)
Unprotect[LessEqual]
x_ <= 0 := NonPositive[x]
Protect[LessEqual]

Unprotect[Positive]
Positive[x_ + y_]:=
  If[Positive[x],
     If[NonNegative[y],True,Uncertain,Uncertain],
     (*x<=0*)If[NonPositive[y],False,Uncertain,Uncertain],
     If[NonNegative[x],If[Positive[y],True,Uncertain,Uncertain],
        Impossible,Uncertain] ]
Positive[x_ y_]:=
  If[Positive[x],
     Positive[y],
     Uncertain,
     If[Negative[x],Negative[y],Uncertain,Uncertain] ]
Positive[_?Positive ^ _?RealQ]:= True
Positive[_?Negative ^ _?EvenQ]:= True
Positive[_?NonPositive ^ _?OddQ]:= False
Positive[x_]:= Uncertain /; !NumberQ[x]
Protect[Positive]

(* Added by R.S., Jan 29 2008: *)
Unprotect[Greater]
x_ > 0 := Positive[x]
Protect[Greater]

(* The following block leads to errors if one evaluates expressions
Abs[I*Sqrt[3]]; it results -I*Sqrt[3]; therefore it has been  commentated 
by R. S. 04.02.2003.*)

(* Unprotect[Abs]
Abs[x_?NonNegative]:= x;t
Abs[x_?Negative]:= -x;
Abs/: Abs[x_?RealQ]^n_?EvenQ:= x^n
Protect[Abs] *)

Unprotect[IntegerQ]
IntegerQ[x_ + y_]:= IntegerQ[x] && IntegerQ[y]
IntegerQ[x_ y_]:= IntegerQ[x] && IntegerQ[y]
IntegerQ[_?IntegerQ ^ n_?IntegerQ]:= True /; NonNegative[n]
IntegerQ[x_]:= Uncertain /; Head[x] =!= Integer
Protect[IntegerQ]

Unprotect[Floor,Ceiling,Round]
IntegerQ[Floor[_]]^= True
IntegerQ[Ceiling[_]]^= True
IntegerQ[Round[_]]^= True
Protect[Floor,Ceiling,Round]

(* Elementary Functions (introduced by R. S., Dec 5,2002)*)
(* see the line For...(**) in the next paragraph *)
FFF={Cos,Sin,Tan,Csc, Sec,Cot,ArcTan, ArcCot,Cosh, \
Sinh,Tanh,Csch,Sech,Coth,ArcSinh,ArcCsch, Exp}

(*Mathematica v. 4.1: RealQ is not implemented, Unprotect, Protect 
commented, R.S.*)

 (* Unprotect[RealQ] *)
RealQ[x_ + y_]:= RealQ[x] && RealQ[y] 
RealQ[x_ y_]:= RealQ[x] && RealQ[y]
RealQ[_Real] = True
RealQ[_Integer] = True
RealQ[_Rational] = True
RealQ[_Complex] = False
RealQ[Re[_]] = True
RealQ[Im[_]] = True
RealQ[Arg[_]] = True
RealQ[Abs[_]] = True
RealQ[_?NonNegative ^ _?RealQ] = True
RealQ[_?Negative ^ x_]:= False /; !IntegerQ[x]
RealQ[_?RealQ ^ _?IntegerQ] = True
For[j=1, j<18,RealQ[FFF[[j]][x_?RealQ]]:=True;j++]  (**)
RealQ[z_*Conjugate[z_]]=True (**)
RealQ[_] = Uncertain
(* Protect[RealQ] *)

(*Next block added by R.S., Jan 1, 2003:*)

Unprotect[Element]
Element[x_, Reals] := RealQ[x]
Protect[Element]

(*Next block added by R.S., Dec 9, 2002:*)

RealQ[A_List] :=
  Module[{},
    For[i = 1, i <= Length[A], i++,
      If[RealQ[A[[i]]] ==  False, Return[False]];
      If[RealQ[A[[i]]] ==  Uncertain, Return[Uncertain], Return[Uncertain]]];
    True]

(*Next two  block added by R.S., Dec 9, 2002:*)

RealQ[A_+B_]:= If[RealQ[A] && RealQ[B], True, Uncertain, Uncertain]


RealQ[A_*B_] :=
  Module[{}, If[(RealQ[A] && RealQ[B]) || A == 0 || B == 0, Return[True]];
     If[(RealQ[A] && Not[RealQ[B]]) || (RealQ[B] && Not[RealQ[A]]),
      Return[False]];
    Uncertain]

Unprotect[Re,Im]
Re[x_?RealQ] = x
Im[_?RealQ] = 0
Protect[Re,Im]

Unprotect[Power]
x_?(NumberQ[#]&&Negative[#]&) ^ n_?EvenQ:= (-x)^n
x_?(NumberQ[#]&&Negative[#]&) ^ n_?OddQ:= x (-x)^(n-1)
(x_?RealQ ^n_?EvenQ)^a_?RealQ:= Abs[x]^(n a)
Protect[Power]

Unprotect[EvenQ]
EvenQ[n_ + m_]:= If[EvenQ[n],EvenQ[m],OddQ[m],Uncertain]
EvenQ[n_ m_]:=
  
If[EvenQ[n],IntegerQ[m],EvenQ[m],If[IntegerQ[n],EvenQ[m],Uncertain,Uncertain]]
EvenQ[_?EvenQ ^ n_?IntegerQ]:= True /; Positive[n]
EvenQ[_?OddQ ^ n_?IntegerQ]:= False /; Positive[n]
EvenQ[x_]:= Uncertain /; Head[x] =!= Integer
Protect[EvenQ]

Unprotect[OddQ]
OddQ[n_ + m_]:= If[EvenQ[n],OddQ[m],EvenQ[m],Uncertain]
OddQ[n_ m_]:= If[OddQ[n],OddQ[m],False,Uncertain]
OddQ[_?EvenQ ^ n_?IntegerQ]:= False /; Positive[n]
OddQ[_?OddQ ^ n_?IntegerQ]:= True /; Positive[n]
OddQ[x_]:= Uncertain /; Head[x] =!= Integer
Protect[OddQ]

Unprotect[DirectedInfinity];
DirectedInfinity/: _?Positive DirectedInfinity[d_]:= DirectedInfinity[d]
DirectedInfinity/: _?Negative DirectedInfinity[d_]:= DirectedInfinity[-d]
Protect[DirectedInfinity]
Unprotect[E,Pi,Catalan,EulerGamma]
Declare[{E,Pi,Catalan,EulerGamma},Positive]
Protect[E,Pi,Catalan,EulerGamma]

(*Lines with (**) added by R.S., Dec 9,2002 *)
Uncertain/: Uncertain && True = Uncertain
Uncertain/: Uncertain || True = True    (**)
Uncertain/: Uncertain || False = Uncertain
Uncertain/: Uncertain && False = False      (**)
Uncertain/: False && Uncertain = False      (**)
Uncertain/: !Uncertain = Uncertain
Uncertain/: Uncertain && Uncertain = Uncertain
Uncertain/: Uncertain || Uncertain = Uncertain

Print[{Declare,NewDeclare,NonPositive,RealQ,Uncertain}]

End[]

EndPackage[]

Null





