(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 4.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    163577,       5762]*)
(*NotebookOutlinePosition[    165919,       5846]*)
(*  CellTagsIndexPosition[    165526,       5828]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Projective Symplectic Geometry", "Title",
  TextAlignment->Center],

Cell["by", "Subsubtitle",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontFamily->"Courier",
  FontSize->14,
  FontWeight->"Bold",
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]],

Cell["Rolf Sulanke", "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontSize->24],

Cell[TextData[{
  "January 11,  2006, revised March 18,2008\nCreated with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " v. 4.2"
}], "Text",
  Evaluatable->False,
  CellHorizontalScrolling->False,
  TextAlignment->Center,
  FontSize->14],

Cell[CellGroupData[{

Cell["Summary", "Subsubsection",
  TextAlignment->Left,
  FontFamily->"Courier"],

Cell[TextData[{
  "This notebook describes symplectic invariants for elementary \
configurations in the 3-dimensional real or complex projective space or the \
3-sphere.  Most of the basic constructs may also be applied in higher \
dimensions.  The theoretical background can be found in ",
  ButtonBox["[OSIII]",
    ButtonData:>"OSIII",
    ButtonStyle->"Hyperlink"],
  ", section 2.8.  The notebook contains examples, mentioned in this book, in \
greater detail; some calculations omitted in the book are carried out here. \
In particular, the classification of complex and real symmetric bilinear \
forms and the description of the corresponding quadrics is given. Some \
additional material, e. g. the symplectic othogonalization of a vector \
sequence,  is contained too. "
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Introduction", "Subsection"],

Cell[TextData[{
  "It is well known that ",
  StyleBox[" symplectic scalar products",
    FontColor->RGBColor[0, 0, 1]],
  ", i. e.  non degenerated skew symmetric bilinear forms, exist only in \
even-dimensional vector spaces. ",
  StyleBox["A vector space is called symplectic",
    FontColor->RGBColor[0, 0, 1]],
  ", if a symplectic scalar product  for its vectors is distinguished. The ",
  StyleBox["symplectic group",
    FontColor->RGBColor[0, 0, 1]],
  " is defined as the group of  linear transformations of the vector space \
preserving the symplectic scalar product;  it  acts transitively on the \
corresponding odd-dimensional projective space,  defining the ",
  StyleBox["projective symplectic geometry",
    FontColor->RGBColor[0, 0, 1]],
  " as the  theory of geometric properties invariant under this action, in \
the sense of  F. Klein's Erlanger Programm.  Since in dimension 2 the \
symplectic group coincides with the special linear group, the symplectic \
geometry of  a projective line coincides with its projective geometry. The \
first interesting case is the three-dimensional one,  the main subject of \
this notebook. Closely related to the projective geometry is the spherical \
geometry. One easily verifies, that the symplectic groups act transitively \
also on the odd-dimensional spheres, being double coverings of the projective \
spaces of the same dimension. The tools developed in this notebook can also \
be applied to explore the ",
  StyleBox["spherical symplectic geometries",
    FontColor->RGBColor[0, 0, 1]],
  ". \nIn the ",
  StyleBox["first section ",
    FontColor->RGBColor[1, 0, 0]],
  "basic concepts of symplectic linear algebra are presented. An algorithm \
called symplectic orthogonalization seems to be new: it constructs for a \
given sequence of vectors a symplectic  or optional an orthosymplectic \
sequence of vectors with the same span. This also gives a method to define \
adapted bases for subspaces. Rank and index of the scalar product restricted \
to the subspace are calculated, and the symplectic vector sequence consists \
of a basis of the defect subspace and a symplectic basis for a complementary \
symplectic subspace within the span. \nThe ",
  StyleBox["second section",
    FontColor->RGBColor[1, 0, 0]],
  " contains some considerations of symplectic transformations. In \
particular, we introduce a very simple class of these transformations, namely \
the  ",
  StyleBox["symplectic transvections",
    FontColor->RGBColor[0, 0, 1]],
  ", which generate the symplectic group.\n",
  StyleBox["Section 3",
    FontColor->RGBColor[1, 0, 0]],
  " is devoted to symplectic line geometry.  The absolute of projective \
symplectic geometry is the complex of the isotropic lines, called the ",
  StyleBox["absolute null system",
    FontColor->RGBColor[0, 0, 1]],
  ". The complement of the nullsystem is the set of symplectic lines, on \
which the restriction of the scalar product does not vanish. (Remember that \
the projective lines are the two-dimensional vector subspaces.) For pairs of \
symplectic lines there exists a symplectic invariant being similar to the \
distance of two lines in metric geometries. The value of this invariant is \
the function ",
  StyleBox["sym ",
    FontColor->RGBColor[0, 0, 1]],
  "defined and studied in this section.\nThe aim of ",
  StyleBox["the last two sections ",
    FontColor->RGBColor[1, 0, 0]],
  "is the classification of the ",
  StyleBox["quadrics",
    FontColor->RGBColor[0, 0, 1]],
  " in the 3-dimensional complex or real projective symplectic spaces. As in \
Euclidean or affine geometries one classifies the ",
  StyleBox["symmetric bilinear forms",
    FontColor->RGBColor[0, 0, 1]],
  " whose corresponding quadratic forms define the quadrics. They are \
equivariantly associated with a special class of endomorphisms, the ",
  StyleBox["skew symmetric operators",
    FontColor->RGBColor[0, 0, 1]],
  ", of the underlying vector space. These operators can be classified using \
their Jordan decompositions. In ",
  StyleBox["section 4",
    FontColor->RGBColor[1, 0, 0]],
  " this is done for complex symplectic spaces; for each class  normal forms \
of the operators and the bilinear forms are found.  ",
  StyleBox["Section 5",
    FontColor->RGBColor[1, 0, 0]],
  " describes the refinements necessary for real spaces. Also in this case \
normal forms are obtained with the help of which  one may discuss the shape \
of the quadrics.   \nThe ",
  StyleBox["appendix",
    FontColor->RGBColor[1, 0, 0]],
  " contains modules useful for any application of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to linear algebra and the corresponding geometries. They are contained in \
the ",
  StyleBox["package vectorcalc.m",
    FontColor->RGBColor[1, 0, 0]],
  ". The modules specific for symplectic linear algebra are collected in the \
",
  StyleBox["package symplecticgeo.m",
    FontColor->RGBColor[1, 0, 0]],
  ". Both packages are not needed for evaluating the present notebook; it is \
not recommended to import them, since then context or protection conflicts \
would arise with the equally named functions or modules created in this \
notebook. User who want to create their own notebooks or packages varying or \
continuing the considerations presented here may find it useful to import \
these packages as a starting point of their own new notebooks. \nThe ",
  StyleBox["References",
    FontColor->RGBColor[1, 0, 0]],
  " only contain a few items often used in this notebook; a more detailed \
bibliography can be found in [OSIII].  The author will be very thankful for \
any hints or comments, especially for critical ones. Please send all \
questions or comments to the ",
  ButtonBox["address",
    ButtonData:>"address",
    ButtonStyle->"Hyperlink"],
  " mentioned at the end of the notebook."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Keywords", "Subsection"],

Cell["\<\
annihilator, associated operator, Gram matrix, index of  a \
subspace, isotropic subspaces, lines, null system, orthosymplectic basis, \
polar, pole,  projective symplectic space, quadrics,  rank of a subspace, \
skew symmetric operator,  symmetric bilinear form, spherical symplectic \
geometry, sym, symplectic basis, symplectic orthogonalization, symplectic \
scalar product, symplectic transvection, symplectic vector space, \
\>", \
"Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Hints", "Subsection"],

Cell[TextData[{
  StyleBox["Initialization:",
    FontColor->RGBColor[1, 0, 0]],
  " Before starting interactive work with this notebook,  read the next \
section carefully and follow the instructions given there!"
}], "Text"],

Cell[TextData[{
  StyleBox["Style:",
    FontColor->RGBColor[1, 0, 0]],
  " Under Linux I had some problems with the default style sheets provided by \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  If you meet similar problems, e.g. if some cells appear empty because \
of missing fonts, try other style sheets of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", or the style sheet sustyle.nb, available from my homepage. (Download it \
to your working directory, open the notebook symplectic.nb, apply \
\"Format/Style Sheet/ Other\", and open sustyle.nb.) If this doesn't help, \
you may create your own style sheet modifying and saving an appropriate one.  \
"
}], "Text"],

Cell[TextData[{
  StyleBox["The subsection named 'Tests\"",
    FontColor->RGBColor[1, 0, 0]],
  " may be omitted; they only contain cells controlling the behavior of the \
newly defined objects."
}], "Text"],

Cell[TextData[{
  StyleBox["Hyperlinks",
    FontColor->RGBColor[1, 0, 0]],
  " to other places of this notebook are marked by white text on blue \
background, e.g.  ",
  ButtonBox["address",
    ButtonData:>"address",
    ButtonStyle->"Hyperlink"],
  ". A mouse click on such text brings you to the linked place, and by the \
menu button Find/Go Back you return to the subsection containing the link."
}], "Text"],

Cell[TextData[{
  StyleBox["Blue text",
    FontColor->RGBColor[1, 0, 0]],
  " marks places at which the marked concepts are defined or emphasized."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Copyright", "Subsection",
  CellTags->"copyrigt"],

Cell["\<\
This notebook, the linked notebooks, and the accompanying packages \
are  public. Authors who intend to publish a changed or completed version of \
them should do it under their own names, and under the condition that they \
cite the original notebook with the Internet address or other source where \
they got it. I am not responsible for errors or damages originated by the use \
of the procedures contained in my notebooks or packages; anybody who applies \
them should test carefully whether they are appropriate for his \
purposes.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[CellGroupData[{

Cell["A needed package", "Subsection"],

Cell[TextData[{
  "In the notebook we apply many constructs of  linear algebra, defined in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", or in our earlier developed packages. To make the notebook almost \
selfcontained we collected the constructs and describing usages not internal \
to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "  in the appendix. Some are commented in the main text. An exception is \
the  package Declare.m of  Pekka Janhunen, which we apply in some places in \
the present notebook. It is very useful for defining types of variables, as \
Real or Complex, and the transition between them in complex calculations. We \
use a renewed version available as a download from my ",
  ButtonBox["homepage",
    ButtonData:>"hp",
    ButtonStyle->"Hyperlink"],
  ", where one finds many packages and notebooks as well as the newest \
version of the present notebook.  \nBefore initializing the notebook ensure \
that the package\nDeclare.m\nis in your working directory. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Preparation: Path of the working directory", "Subsection"],

Cell["\<\
The operation of this subsection are necessary only at the first \
use of this notebook on your computer.  If this is done once, and if the the \
packages needed later, e.g. \"Declare.m\",  lie in the working directory,   \
you may start calling the initialization as described in the next subsection. \
\
\>", "Text"],

Cell[TextData[StyleBox["At the moment, DO NOT CONFIRM THE EVALUATION OF THE \
INITIALIZATION CELLS !",
  FontWeight->"Bold",
  FontColor->RGBColor[1, 0, 0]]], "Text"],

Cell["If the result of the following command ", "Text"],

Cell[BoxData[
    \(Directory[]\)], "Input",
  CellLabel->"In[75]:=",
  FontFamily->"Times"],

Cell["\<\
does not coincide with the working directory, e.g. \"C:\\Eigene \
Dateien\\mymath\\mathwork\", containing the needed packages, insert the Path \
of your working directory in the next command line (corresponding to your  \
operating system) and activate it first, without calling the \
initialization:\
\>", "Text"],

Cell[TextData[StyleBox["For Windows:",
  FontColor->RGBColor[1, 0, 0]]], "Text",
  FontSize->18],

Cell[BoxData[
    \(SetDirectory["\<C:\\Dokumente und Einstellungen\\rolf\\Eigene \
Dateien\\mymath\\mathwork\\symplecticDL\>"]\)], "Input",
  CellLabel->"In[1]:=",
  InitializationCell->True,
  FontFamily->"Times"],

Cell[TextData[StyleBox["For Linux:",
  FontColor->RGBColor[1, 0, 0]]], "Text",
  FontSize->18],

Cell[BoxData[
    \(SetDirectory["\<~/mymath/mathwork/symplecticDL\>"]\)], "Input",
  CellLabel->"In[1]:=",
  Evaluatable->False,
  FontFamily->"Times"],

Cell["\<\
Now give the cell corresponding to your  operating system the \
properties \"Cell Evaluatable\" and \"Initialization cell\" (Menu Cell/Cell \
properties), and inactivate these properties for the corresponding cells of \
the other operating system. If this is done, save the notebook. Next time you \
may start the notebook directly with the evaluation of the initialization, as \
follows:\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initialization", "Subsection"],

Cell["IMPORTANT!", "Text",
  FontSize->18,
  FontWeight->"Bold",
  FontColor->RGBColor[1, 0, 0]],

Cell["\<\
Before starting to work interactively with the notebook, first do \
the following two steps:\
\>", "Text"],

Cell["\<\
1. Activite Kernel/Evaluation/Evaluate Initialization from the \
menu.\
\>", "Text",
  Evaluatable->False,
  FontSize->16,
  FontWeight->"Bold",
  FontColor->RGBColor[1, 0, 0]],

Cell[TextData[StyleBox["2. Set the dimension:",
  FontSize->18,
  FontColor->RGBColor[1, 0, 0]]], "Text"],

Cell["\<\
In this notebook we shall  consider mainly the 3-dimensional \
symplectic projective space. The dimension dim of the vector space equals the \
dimension of the projective space +1,  therefore evaluate the next \
cell:\
\>", "Text"],

Cell[BoxData[
    \(dim\  = \ 4\)], "Input",
  CellLabel->"In[82]:="],

Cell[TextData[{
  StyleBox["Remark \n",
    FontSize->16,
    FontColor->RGBColor[0, 0, 1]],
  "The global constant ",
  StyleBox["dim",
    FontColor->RGBColor[0, 0, 1]],
  " appears in many functions as default argument of the dimension; without \
fixing it as a positive integer many error messages will be created, and they \
will not work. The order in which the two steps mentioned above are carried \
out is deciding: In case you fixed the dimension dim already before \
evaluating the initialization cells, it will be fixed for all the concepts as \
the default value of this variable.  Fixing it after the Initialization makes \
it possible to change the value of dim any time, if wanted."
}], "Text"],

Cell[BoxData[
    \(\(?dim\)\)], "Input",
  CellLabel->"In[99]:="],

Cell[BoxData[
    \(Context[dim]\)], "Input",
  CellLabel->"In[100]:="],

Cell[BoxData[
    \(dim\)], "Input",
  CellLabel->"In[101]:="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["1. Symplectic vector space. Corresponding projective space ", "Section"],

Cell["\<\
In this section we introduce the basic concepts of symplectic \
linear algebra: the scalar product, the Gram matrix, symplectic and \
orthosymplectic bases, polar spaces. Especially, in subsection 1.3,  we \
construct a procedure called symortho, being analogous to the \
orthogonalization procedure in Euclidean geometry.  The constructs of \
subsection 1.1 - 1.4 are valid for any even dimension, also if a default \
dimension is not defined. But for convenience, and for the tests and \
examples, it is recommended to set the default value dim.  For the Graphics3D \
objects considered in subsection 1.5 the default dimension dim of the \
underlying vector space must be set: dim = 4\
\>", "Text"],

Cell[CellGroupData[{

Cell["1.1. The symplectic scalar product ssp,  grammatrix, gram", "Subsection"],

Cell[CellGroupData[{

Cell["Definition of the symplectic scalar product ssp", "Subsubsection"],

Cell[BoxData[
    \(\(ssp::usage\  = "\<ssp[vx,vy] is the symplectic scalar product of two \
d-dimensional vectors; the dimension d= Length[vx] must be even.\>";\)\)], \
"Input",
  CellLabel->"In[2]:=",
  InitializationCell->True],

Cell[BoxData[
    \(ssp[vx_, vy_]\  := \ 
      Module[{\ n = Length[vx], d}, \[IndentingNewLine]If[
          Head[vx]\  \[NotEqual] \ List || \ Head[vy]\  \[NotEqual] \ List || 
            OddQ[n] || Length[vy] \[NotEqual] \ n, \ 
          Print["\<Incorrect arguments \>", vx, \ vy]; Return[], \ Null, 
          Print["\<Arguments correct?\>"]; Return[]]; \[IndentingNewLine]d = 
          n/2; \[IndentingNewLine]Sum[
          vx[\([i]\)]*vy[\([d + i]\)] - vx[\([d + i]\)]*vy[\([i]\)], {i, 1, 
            d}]]\)], "Input",
  CellLabel->"In[3]:=",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Properties of the symplectic scalar product ssp", "Subsubsection"],

Cell[BoxData[
    \(dim\  = \ 4\)], "Input",
  CellLabel->"In[103]:="],

Cell["A vector with components x[i], i = 1,..., dim, is defined by", "Text"],

Cell[BoxData[
    \(?? vec\)], "Input",
  CellLabel->"In[104]:="],

Cell[BoxData[
    \(vec[x]\)], "Input",
  CellLabel->"In[105]:="],

Cell[BoxData[
    \(\(\(Head[%]\)\(\ \)\)\)], "Input",
  CellLabel->"In[106]:="],

Cell["The scalar product is alternating:", "Text"],

Cell[BoxData[
    \(ssp[vec[x], \ vec[y]]\)], "Input",
  CellLabel->"In[107]:="],

Cell[BoxData[
    \(% \[Equal] \ \(-ssp[vec[y], \ vec[x]]\)\)], "Input",
  CellLabel->"In[108]:="],

Cell[BoxData[
    \(ssp[vec[x], vec[x]]\)], "Input",
  CellLabel->"In[109]:="],

Cell["it is bilinear:", "Text"],

Cell[BoxData[
    \(Simplify[
      ssp[vec[a], \ vec[b] + vec[c]] \[Equal] \ 
        ssp[vec[a], \ vec[b]]\  + \ ssp[vec[a], \ vec[c]]]\)], "Input",
  CellLabel->"In[110]:="],

Cell[BoxData[
    \(Simplify[
      ssp[vec[a], \ c*vec[b]] \[Equal] \ c*ssp[vec[a], \ vec[b]]]\)], "Input",\

  CellLabel->"In[111]:="],

Cell["In the next subsection we show that it is non degenerated.", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Definitions and properties of the grammatrix, gram", "Subsubsection"],

Cell["Remember the standard basis", "Text"],

Cell[BoxData[
    \(\(?stb\)\)], "Input",
  CellLabel->"In[92]:="],

Cell["\<\
The rows or columns of the Identitymatrix yield the standard \
basis\
\>", "Text"],

Cell[BoxData[
    \(IdentityMatrix[dim] // MatrixForm\)], "Input",
  CellLabel->"In[112]:="],

Cell[BoxData[
    \(?? stb\)], "Input",
  CellLabel->"In[113]:="],

Cell[BoxData[
    \(stb[4]\)], "Input",
  CellLabel->"In[114]:="],

Cell["\<\
We calculate Gram's matrix of the scalar products of a vector \
sequence, in particular of the standard basis:\
\>", "Text"],

Cell[BoxData[
    \(\(grammatrix::usage\  = \ "\<grammatrix[b_List] is the matrix of the \
scalar products of the vectors b[[i]] of List b with respect to the scalar \
product ssp.\>";\)\)], "Input",
  CellLabel->"In[4]:=",
  InitializationCell->True],

Cell[BoxData[
    \(grammatrix[b_List]\  := 
      Table[ssp[b[\([i]\)], \ b[\([j]\)]], {i, Length[b]}, {j, 
          Length[b]}]\)], "Input",
  CellLabel->"In[5]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(gram::usage\  = \ "\<gram[d:dim] is Gram's matrix of the scalar \
products of the standard basis in dimension d.\>";\)\)], "Input",
  CellLabel->"In[6]:=",
  InitializationCell->True],

Cell[BoxData[
    \(gram[d_:  dim] := \ 
      If[EvenQ[d] && \ d\  > \ 0, grammatrix[IdentityMatrix[d]], 
        Print["\<Incorrect argument!\>"]; \ Return[], \ 
        Print["\<Argument?\>"]; \ Return[]]\)], "Input",
  CellLabel->"In[7]:=",
  InitializationCell->True],

Cell[TextData[{
  "A basis of the underlying vector space is called a ",
  StyleBox["symplectic basis",
    FontColor->RGBColor[0, 0, 1]],
  ", if its grammatrix coincides with gram.\nThe determinant of the \
grammatrix vanishes iff the argument is linearely dependent, or  its span is \
isotropic:"
}], "Text"],

Cell[BoxData[
    \(grammatrix[{vec[x], \ vec[y], \ vec[z], 
        a*vec[x] + b*vec[y] + c*vec[z] + d*vec[w]}]\)], "Input",
  CellLabel->"In[115]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[116]:="],

Cell[BoxData[
    \(Det[%]\)], "Input",
  CellLabel->"In[117]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[118]:="],

Cell[BoxData[
    \(% \[Equal] \ 
      d^2*Simplify[
          Det[grammatrix[{vec[x], \ vec[y], \ vec[z], vec[w]}]]]\)], "Input",
  CellLabel->"In[119]:="],

Cell[TextData[{
  "The result illustrates Lemma 2.2.14 in  ",
  ButtonBox["[OSIII]",
    ButtonData:>"OSIII",
    ButtonStyle->"Hyperlink"],
  ".  Any three-dimensional subspace (Projectively: any plane) is isotropic, \
e.g."
}], "Text"],

Cell[BoxData[
    \(Det[grammatrix[{stb[1], stb[2], stb[3]}]]\)], "Input",
  CellLabel->"In[120]:="],

Cell["Take three random vectors", "Text"],

Cell[BoxData[
    \(a1 = randomv[]; a2 = 5*randomv[]; \ 
    a3 = 7*randomv[]; {a1, a2, a3} // MatrixForm\)], "Input",
  CellLabel->"In[121]:="],

Cell["\<\
By numerical reasons,  the determinant is not exactly zero, but \
only small:\
\>", "Text"],

Cell[BoxData[
    \(Det[grammatrix[{a1, a2, a3}]]\)], "Input",
  CellLabel->"In[122]:="],

Cell[BoxData[
    \(Chop[%]\)], "Input",
  CellLabel->"In[123]:="],

Cell["\<\
Of course, the vectors are linearely independent with probability \
1:\
\>", "Text"],

Cell[BoxData[
    \(Minors[{a1, a2, a3}, 3]\)], "Input",
  CellLabel->"In[124]:="],

Cell[BoxData[
    \(Clear[a1, a2, a3]\)], "Input",
  CellLabel->"In[125]:="],

Cell[BoxData[
    \(gram[] // MatrixForm\)], "Input",
  CellLabel->"In[126]:="],

Cell[BoxData[
    \(gram[] . gram[]\)], "Input",
  CellLabel->"In[127]:="],

Cell["The inverse of gram[] equals -gram[]:", "Text"],

Cell[BoxData[
    \(Inverse[gram[]] \[Equal] \ \(-gram[]\)\)], "Input",
  CellLabel->"In[128]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Tests ssp, grammatrix, gram", "Subsubsection"],

Cell[BoxData[
    \(ssp[, ]\)], "Input",
  CellLabel->"In[129]:="],

Cell[BoxData[
    \(ssp[\(,\)\(vec[x]\)]\)], "Input",
  CellLabel->"In[130]:="],

Cell[BoxData[
    \(ssp[\(vec[x]\)\(,\)\ ]\)], "Input",
  CellLabel->"In[131]:="],

Cell[BoxData[
    \(ssp[vec[x], \ randomv[]\ ]\)], "Input",
  CellLabel->"In[132]:="],

Cell[BoxData[
    \(seq\  = \ Table[3*randomv[6], {6}]\)], "Input",
  CellLabel->"In[133]:="],

Cell[BoxData[
    \(grammatrix[seq] // MatrixForm\)], "Input",
  CellLabel->"In[134]:="],

Cell[BoxData[
    \(Det[%]\)], "Input",
  CellLabel->"In[135]:="],

Cell[BoxData[
    \(Clear[seq]\)], "Input",
  CellLabel->"In[136]:="],

Cell[BoxData[
    \(dim\  = \ 6\)], "Input",
  CellLabel->"In[137]:="],

Cell[BoxData[
    \(gram[] // MatrixForm\)], "Input",
  CellLabel->"In[138]:="],

Cell["The volume spanned by any symplectic basis equals 1:", "Text"],

Cell[BoxData[
    \(Det[%]\)], "Input",
  CellLabel->"In[139]:="],

Cell[BoxData[
    \(dim\  = \ 4\)], "Input",
  CellLabel->"In[140]:="],

Cell[BoxData[
    \(gram[] // MatrixForm\)], "Input",
  CellLabel->"In[147]:="],

Cell["\<\
If the argument d of gram is not a positive even integer, gram[d] \
will not be evatuated:\
\>", "Text"],

Cell[BoxData[
    \(gram[5]\)], "Input",
  CellLabel->"In[148]:="],

Cell[BoxData[
    \(gram[Pi]\)], "Input",
  CellLabel->"In[149]:="],

Cell[BoxData[
    \(gram[\(-2\)]\)], "Input",
  CellLabel->"In[150]:="],

Cell[BoxData[
    \(gram[0]\)], "Input",
  CellLabel->"In[151]:="],

Cell[BoxData[
    \(gram[gram[]]\)], "Input",
  CellLabel->"In[152]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.2. Symplectic and orthosymplectic bases, subspaces", "Subsection"],

Cell[CellGroupData[{

Cell["Definitions", "Subsubsection"],

Cell[TextData[{
  "A basis of the vector space of dimension dim is called ",
  StyleBox["symplectic",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  ", if the matrix of the scalar products of its vectors coincides with \
gram[], in our case, with"
}], "Text"],

Cell[BoxData[
    \(gram[] // MatrixForm\)], "Input",
  CellLabel->"In[153]:="],

Cell["\<\
With our definition of the scalar product  the standard basis is \
symplectic; e.g.\
\>", "Text"],

Cell[BoxData[
    \(gram[8] // MatrixForm\)], "Input",
  CellLabel->"In[154]:="],

Cell[TextData[{
  StyleBox["SUBSPACES",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True}],
  "\nA subspace U is said to be ",
  StyleBox["symplectic",
    FontColor->RGBColor[0, 0, 1]],
  ", if the restriction of the scalarproduct ssp|UxU is non-degenerate, and \
i",
  StyleBox["sotropic",
    FontColor->RGBColor[0, 0, 1]],
  " otherwise;  U is ",
  StyleBox["totally isotropic",
    FontColor->RGBColor[0, 0, 1]],
  ", if this restriction is identically zero. Maximal totally isotropic \
subspaces are called ",
  StyleBox["Lagrangian",
    FontColor->RGBColor[0, 0, 1]],
  " subspaces. It can be shown, that the dimension of Lagrangian subspaces \
equals n = dim/2."
}], "Text"],

Cell[TextData[{
  "The span of the vectors {b[1],..., b[n]}, or {b[n+1],...,b[2n]},  of any \
symplectic basis {b[1],...,b[2n]} is ",
  StyleBox[" totally isotropic, indeed Lagrangian, ",
    FontColor->RGBColor[0, 0, 1]],
  "and the whole space is a direct sum of two totally isotropic subspaces. \
Vector spaces with a scalar product having this property are called ",
  StyleBox["neutral ",
    FontColor->RGBColor[0, 0, 1]],
  ". "
}], "Text"],

Cell[TextData[{
  "A vector sequence (b[i], i = 1,...,m) is called ",
  StyleBox["s",
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 1]],
  StyleBox["ymplectic of rank r and defect d = m-r",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[", ",
    FontColor->RGBColor[0, 0, 1]],
  "if the first r vectors are a symplectic basis of a symplectic subspace, \
and the last m-r vectors are a basis of the defect subspace of the span[b[i], \
i = 1,...,m]. Therefore one has r = 2s is  even, and the scalar products are"
}], "Text"],

Cell[BoxData[
    \(Table[ssp[b[i], b[j]], {i, r}, {j, r}]\  \[Equal] \ gram[r], \ 
    r = 2  s, \)], "Input",
  Evaluatable->False],

Cell[BoxData[
    \(ssp[b[i], b[h]]\  \[Equal] \ 0, \ i\  = \ 1,  ... , m; \ 
    h\  = \ r + 1,  ... , \(\(m\)\(.\)\)\)], "Input",
  Evaluatable->False],

Cell["\<\
By definition, a symplectic vector sequence is linearely \
independent. \
\>", "Text"],

Cell[TextData[{
  "Often one uses another type of bases, which we call  orthosymplectic; a \
basis (b[i], i = 1,..., 2k) of the 2k-dimensional symplectic vector space is \
",
  StyleBox["orthosymplectic",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[",",
    FontColor->RGBColor[0, 0, 1]],
  " if  the pairs (b[2i-1], b[2i]), i =1,...,k, span pairwise orthogonal \
2-dimensional symplectic subspaces. "
}], "Text"],

Cell["\<\
The following Module renumbers the vectors of a symplectic basis \
such that the result becomes an orthosymplectic basis:\
\>", "Text"],

Cell[BoxData[
    \(\(ortho::usage\  = \ "\<ortho[b] is a renumbering of the vectors of a \
symplectic basis b such that the result is an orthosymplectic basis. WARNING! \
ortho[b] yields false results if b is not a symplectic basis, without any \
error message!\>";\)\)], "Input",
  CellLabel->"In[8]:=",
  InitializationCell->True],

Cell[BoxData[
    \(ortho[c_]\  := \ 
      Module[{d, b}, \[IndentingNewLine]If[MatrixQ[c], d = Length[c], 
          Print["\<The argument is not a matrix!\>"]; Return[], 
          Print["\<The argument is not a matrix!\>"]; 
          Return[]]; \[IndentingNewLine]If[OddQ[d], 
          Print["\<Incorrect argument \>", \ c]; 
          Return[]]; \[IndentingNewLine]b = \ 
          nullmatrix[d]; \[IndentingNewLine]For[i = \ 1, 
          i \[LessEqual] \ d/2, \(i++\), \ 
          b[\([2*i]\)] = \ c[\([d/2\  + \ i]\)]; 
          b[\([2*i - 1]\)] = c[\([i]\)]]; \[IndentingNewLine]b]\)], "Input",
  CellLabel->"In[9]:=",
  InitializationCell->True],

Cell["The Gram matrix of the orthosymplectic bases is", "Text"],

Cell[BoxData[
    \(\(orthogram::usage = "\<orthogram[d], d even, is the Gram matrix of \
orthosymplectic bases in the d-dimensional symplectic vector space.\>";\)\)], \
"Input",
  CellLabel->"In[10]:=",
  InitializationCell->True],

Cell[BoxData[
    \(orthogram[d_:  dim]\  := \ 
      Module[{c\  = \ ortho[IdentityMatrix[d]]}, \ 
        Table[ssp[c[\([i]\)], c[\([j]\)]], {i, d}, {j, d}]]\)], "Input",
  CellLabel->"In[11]:=",
  InitializationCell->True],

Cell[TextData[StyleBox["Properties:",
  FontColor->RGBColor[0, 0, 1]]], "Text"],

Cell[BoxData[
    \(gram[] // MatrixForm\)], "Input",
  CellLabel->"In[155]:="],

Cell[BoxData[
    \(orthogram[] // MatrixForm\)], "Input",
  CellLabel->"In[156]:="],

Cell[BoxData[
    \(orthogram[8] // MatrixForm\)], "Input",
  CellLabel->"In[157]:="],

Cell[BoxData[
    \(\(-% . %\) // MatrixForm\)], "Input",
  CellLabel->"In[158]:="],

Cell["Thus, again we have", "Text"],

Cell[BoxData[
    \(Inverse[orthogram[10]] \[Equal] \ \(-orthogram[10]\)\)], "Input",
  CellLabel->"In[159]:="],

Cell[TextData[{
  "A vector sequence (b[i], i = 1,...,m) is called ",
  StyleBox["orthosymplectic of rank r and defect d = m-r",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"CompatibilityType"->0}],
  ", if the first r vectors are an orthosymplectic basis of a symplectic \
subspace, and the last m-r vectors are a basis of the ",
  StyleBox["defect subspace",
    FontColor->RGBColor[0, 0, 1]],
  " of the span  U=span[b[i], i = 1,...,m], which is defined to be the \
intersection of U with the subspace orthogonal to U. . Therefore one has r = \
2s is  even, and the scalar products are "
}], "Text"],

Cell[BoxData[
    \(Table[ssp[b[i], b[j]], {i, r}, {j, r}]\  \[Equal] \ orthogram[r], \ 
    r = 2  s, \)], "Input",
  Evaluatable->False],

Cell[BoxData[
    \(ssp[b[i], b[h]]\  \[Equal] \ 0, \ i\  = \ 1,  ... , m; \ 
    h\  = \ r + 1,  ... , \(\(m\)\(.\)\)\)], "Input",
  Evaluatable->False],

Cell["\<\
The following Module renumbers the vectors of an  orthosymplectic \
basis such that the result becomes a symplectic basis; sym is the operation \
inverse to ortho:\
\>", "Text"],

Cell[BoxData[
    \(\(symplectic::usage\  = \ "\<symplectic[b] is a renumbering of the \
vectors of an orthosymplectic basis b such that the result is a symplectic \
basis. WARNING! symplectic[b] yields false results if b is not an \
orthosymplectic basis, without any error message!\>";\)\)], "Input",
  CellLabel->"In[12]:=",
  InitializationCell->True],

Cell[BoxData[
    \(symplectic[c_]\  := \ 
      Module[{d, b}, \[IndentingNewLine]If[MatrixQ[c], d = Length[c], 
          Print["\<The argument is not a matrix!\>"]; Return[], 
          Print["\<The argument is not a matrix!\>"]; 
          Return[]]; \[IndentingNewLine]If[OddQ[d], 
          Print["\<Incorrect argument \>", \ c]; 
          Return[]]; \[IndentingNewLine]b = \ 
          nullmatrix[d]; \[IndentingNewLine]For[i = \ 1, 
          i \[LessEqual] \ d/2, \(i++\), \ b[\([i]\)] = \ c[\([2*i - 1]\)]; 
          b[\([d/2 + i]\)] = c[\([2*i]\)]]; \[IndentingNewLine]b]\)], "Input",\

  CellLabel->"In[13]:=",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Tests: ortho, symplectic", "Subsubsection"],

Cell["\<\
The rows (or columns) of the IdentityMatrix are the standard basis, \
being symplectic by the definition of ssp. ortho transforms it into an \
orthosymplectic basis, e.g:\
\>", "Text"],

Cell[BoxData[
    \(ortho[IdentityMatrix[4]]\)], "Input",
  CellLabel->"In[160]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[161]:="],

Cell[BoxData[
    \(ortho[gram[8]] // MatrixForm\)], "Input",
  CellLabel->"In[162]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[163]:="],

Cell[BoxData[
    \(dim\  = \ 6\)], "Input",
  CellLabel->"In[164]:="],

Cell[BoxData[
    \(gram[] // MatrixForm\)], "Input",
  CellLabel->"In[165]:="],

Cell[BoxData[
    \(pd = {stb[1], stb[2], stb[4], stb[5]}\)], "Input",
  CellLabel->"In[166]:="],

Cell[BoxData[
    \(pd // MatrixForm\)], "Input",
  CellLabel->"In[167]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[168]:="],

Cell[BoxData[
    \(ortho[pd]\)], "Input",
  CellLabel->"In[169]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[170]:="],

Cell["\<\
In the next subsection, in the context of the symplectic \
orthogonalization, we shall execute further tests of the operation \
ortho.\
\>", "Text"],

Cell[BoxData[
    \(ortho[gram[]] // MatrixForm\)], "Input",
  CellLabel->"In[192]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[172]:="],

Cell[BoxData[
    \(symplectic[%%]\)], "Input",
  CellLabel->"In[173]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[174]:="],

Cell[BoxData[
    \(ortho[gram[8]]\)], "Input",
  CellLabel->"In[175]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[176]:="],

Cell[BoxData[
    \(symplectic[%%]\)], "Input",
  CellLabel->"In[177]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[178]:="],

Cell["symplectic and ortho are  inverse to each other:", "Text"],

Cell[BoxData[
    \(symplectic[ortho[IdentityMatrix[6]]]\)], "Input",
  CellLabel->"In[179]:="],

Cell[BoxData[
    \(% \[Equal] \ IdentityMatrix[6]\)], "Input",
  CellLabel->"In[180]:="],

Cell[BoxData[
    \(ortho[symplectic[IdentityMatrix[6]]] // MatrixForm\)], "Input",
  CellLabel->"In[191]:="],

Cell[BoxData[
    \(% \[Equal] \ IdentityMatrix[6]\)], "Input",
  CellLabel->"In[182]:="],

Cell["\<\
The columnss of orthogram[6] form neither a symplectic nor an \
orthosymplectic basis: \
\>", "Text"],

Cell[BoxData[
    \(c\  = Transpose[orthogram[6]]\)], "Input",
  CellLabel->"In[183]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[184]:="],

Cell["\<\
Therefore neither the application of symplectic nor that of ortho \
give a symplectic  or orthosymplectic basis\
\>", "Text"],

Cell[BoxData[
    \(ortho[c] // MatrixForm\)], "Input",
  CellLabel->"In[185]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[186]:="],

Cell[BoxData[
    \(symplectic[c] // MatrixForm\)], "Input",
  CellLabel->"In[187]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[188]:="],

Cell[BoxData[
    \(Clear[a, b, c, d, pd]\)], "Input",
  CellLabel->"In[193]:="],

Cell[BoxData[
    \(dim\  = \ 4\)], "Input",
  CellLabel->"In[194]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.3. Symplectic orthogonalization: symortho", "Subsection"],

Cell["\<\
Given a vector sequence in a symplectic vector space one wants to \
know the dimension d of its span S, the rank r of the symplectic scalar \
product restricted to that span, and its defect f; clearly d = r+f.  The \
Module symortho defined in the next subsection may be considered as an \
analogue of the orthonormalization in metric vector spaces; it yields a \
symplectic or orthosymplectic vector sequence with the same span S and prints \
its rank and defect.\
\>", "Text"],

Cell[CellGroupData[{

Cell["symortho: Definition", "Subsubsection"],

Cell[BoxData[
    \(\(orthosymplectic::usage\  = "\<orthosymplectic is an option for the \
Module symortho; its default value is False. If set to True, symortho yields \
an orthosymplectic vector sequence, and a symplectic one else.\>";\)\)], \
"Input",
  CellLabel->"In[14]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(symortho::usage = \ "\<symortho[b]: for a given vector sequence b of \
a symplectic vector space with scalar product ssp the result symortho[b] is a \
symplectic vector sequence with the same span as that of b; setting the \
option orthosymplectic -> True, the resulting sequence is orthosymplectic. \
The rank and the defect of the span of b is calculated and printed, \
too.\>";\)\)], "Input",
  CellLabel->"In[15]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(Options[symortho] = {orthosymplectic\  \[Rule] \ False};\)\)], "Input",\

  CellLabel->"In[16]:=",
  InitializationCell->True],

Cell[BoxData[
    \(symortho[a_, opts___] := \ 
      Module[{b, bb, m, \ ea\ , iso = {}, \ c = {}, 
          or}, \[IndentingNewLine]or\  = \ \(orthosymplectic\  /. {opts}\) /. 
            Options[symortho]; \[IndentingNewLine]b = 
          noprops[a]; \[IndentingNewLine]If[b \[Equal] \ \ Null, \ 
          Return[]]; \[IndentingNewLine]\[IndentingNewLine]m = \ 
          Length[b]; \[IndentingNewLine]If[m \[Equal] 0, \ 
          Return[b]]; \[IndentingNewLine]If[OddQ[Length[b[\([1]\)]]], \ 
          Print["\<Incorrect dimension!\>"]; 
          Return[]]; \[IndentingNewLine]\[IndentingNewLine]Label[
          start]; \[IndentingNewLine]ea\  = \ 0; \[IndentingNewLine]b\  = \ 
          outzero[b, 10^\((\(-20\))\)]; \[IndentingNewLine]m = \ 
          Length[b]; \[IndentingNewLine]\[IndentingNewLine]If[m <= \ 1, \ 
          iso = Join[iso, b]; 
          Goto[fin]]; \[IndentingNewLine]\[IndentingNewLine]For[i = 2, 
          i \[LessEqual] \ m, \(i++\), \[IndentingNewLine]If[
            ssp[b[\([1]\)], b[\([i]\)]] \[NotEqual] \ 0, ea\  = \ i; 
            Goto[nnn], Null, 
            Print["\<Scalar products may not be evaluated!\>"]; 
            Return[]]]; \[IndentingNewLine]\[IndentingNewLine]If[
          ea \[Equal] \ 0, 
          For[i = 2, i \[LessEqual] m, \(i++\), \ 
            If[wedge[b[\([1]\)], b[\([i]\)]] \[Equal] \ nullmatrix[], \ 
              b[\([i]\)] = null[]]]; AppendTo[iso, b[\([1]\)]]; 
          b = Table[b[\([i]\)], {i, 2, m}]; 
          Goto[start]]; \[IndentingNewLine]\[IndentingNewLine]Label[
          nnn]; \[IndentingNewLine]AppendTo[c, b[\([1]\)]]; 
        AppendTo[c, 
          b[\([ea]\)]/ssp[b[\([1]\)], b[\([ea]\)]]]; \[IndentingNewLine]bb = 
          Join\ [Table[b[\([i]\)], {i, 2, ea - 1}], 
            Table[b[\([l]\)], {l, ea + 1, m}]]; \[IndentingNewLine]m = 
          Length[bb]; \[IndentingNewLine]For[i = 1, 
          i \[LessEqual] \ m, \(i++\), 
          b[\([i]\)]\  = \ 
            bb[\([i]\)]\  - 
              c[\([\(-2\)]\)]*ssp[bb[\([i]\)], c[\([\(-1\)]\)]]\  + 
              c[\([\(-1\)]\)]*
                ssp[bb[\([i]\)], 
                  c[\([\(-2\)]\)]]]; \[IndentingNewLine]bb\ \  = \ 
          Table[Chop[b[\([i]\)]], {i, m}]; \[IndentingNewLine]b\  = \ 
          noprops[bb]; \[IndentingNewLine]Goto[
          start]; \[IndentingNewLine]Label[fin]; \[IndentingNewLine]m = 
          Length[c]/2; 
        Print["\<The rank is \>", \ 2*m]; \[IndentingNewLine]ea = 
          Sum[ssp[c[\([2*i - 1]\)], c[\([2*i]\)]], {i, 
              m}]; \[IndentingNewLine]If[Chop[m - ea]\  \[NotEqual] \ 0, 
          Print["\<Scalar products may not be evaluated!\>"]; 
          Return[]]; \[IndentingNewLine]If[Length[iso] \[NotEqual] \ 0, 
          iso\  = \ 
            outzero[RowReduce[
                iso]]]; \[IndentingNewLine]Print["\<The defect is \>", \ 
          Length[iso]]; \[IndentingNewLine]ea = \ 
          Sum[Chop[ssp[c[\([i]\)], iso[\([j]\)]]], {i, 2*m}, {j, 
                Length[iso]}] + 
            Sum[Chop[ssp[iso[\([i]\)], iso[\([j]\)]]], {i, Length[iso]}, {j, 
                Length[iso]}]; \[IndentingNewLine]If[
          Chop[ea]\  \[NotEqual] \ 0, 
          Print["\<Scalar products may not be evaluated!\>"]; 
          Return[]]; \[IndentingNewLine]If[
          or \[Equal] False && \ \ c \[NotEqual] \ {}, 
          c = symplectic[c]]; \[IndentingNewLine]\[IndentingNewLine]c = 
          Join[c, iso]; \[IndentingNewLine]Print["\<The dimension is \>", \ 
          Length[c]]; \[IndentingNewLine]Return[c]]\)], "Input",
  CellLabel->"In[17]:=",
  InitializationCell->True],

Cell["\<\
Developing the Module symoortho we used the following construction \
completed with print output useful for test purpose only; if you want to \
apply this, activate the property  \"Evaluatable\" for the next cell and \
evaluate it:\
\>", "Text"],

Cell[BoxData[{
    \(\(Clear[symortho];\)\), "\[IndentingNewLine]", 
    \(symortho[a_, opts___] := \ 
      Module[{b, bb, m, \ ea\ , iso = {}, \ c = {}, 
          or}, \[IndentingNewLine]or\  = \ \(orthosymplectic\  /. {opts}\) /. 
            Options[symortho]; \[IndentingNewLine]b = 
          noprops[a]; \[IndentingNewLine]If[b \[Equal] \ \ Null, \ 
          Return[]]; \[IndentingNewLine]\[IndentingNewLine]m = \ 
          Length[b]; \[IndentingNewLine]If[m \[Equal] 0, \ 
          Return[b]]; \[IndentingNewLine]If[OddQ[Length[b[\([1]\)]]], \ 
          Print["\<Incorrect dimension!\>"]; 
          Return[]]; \[IndentingNewLine]\[IndentingNewLine]Label[
          start]; \[IndentingNewLine]ea\  = \ 0; \[IndentingNewLine]b\  = \ 
          outzero[b]; \[IndentingNewLine]m = \ 
          Length[b]; \[IndentingNewLine]\[IndentingNewLine]If[m <= \ 1, \ 
          iso = Join[iso, b]; 
          Goto[fin]]; \[IndentingNewLine]\[IndentingNewLine]Print["\<b =  \
\>", \ b]; \[IndentingNewLine]For[i = 2, 
          i \[LessEqual] \ m, \(i++\), \[IndentingNewLine]If[
            ssp[b[\([1]\)], b[\([i]\)]] \[NotEqual] \ 0, ea\  = \ i; 
            Goto[nnn], Null, 
            Print["\<Scalar products may not be evaluated!\>"]; 
            Return[]]]; \[IndentingNewLine]\[IndentingNewLine]If[
          ea \[Equal] \ 0, 
          For[i = 2, i \[LessEqual] m, \(i++\), \ 
            If[wedge[b[\([1]\)], b[\([i]\)]] \[Equal] \ nullmatrix[], \ 
              b[\([i]\)] = null[]]]; AppendTo[iso, b[\([1]\)]]; 
          b = Table[b[\([i]\)], {i, 2, m}]; 
          Goto[start]]; \[IndentingNewLine]\[IndentingNewLine]Label[
          nnn]; \[IndentingNewLine]AppendTo[c, b[\([1]\)]]; 
        AppendTo[c, 
          b[\([ea]\)]/ssp[b[\([1]\)], b[\([ea]\)]]]; \[IndentingNewLine]bb = 
          Join\ [Table[b[\([i]\)], {i, 2, ea - 1}], 
            Table[b[\([l]\)], {l, ea + 1, m}]]; \[IndentingNewLine]m = 
          Length[bb]; \[IndentingNewLine]For[i = 1, 
          i \[LessEqual] \ m, \(i++\), 
          b[\([i]\)]\  = \ 
            bb[\([i]\)]\  - 
              c[\([\(-2\)]\)]*ssp[bb[\([i]\)], c[\([\(-1\)]\)]]\  + 
              c[\([\(-1\)]\)]*
                ssp[bb[\([i]\)], 
                  c[\([\(-2\)]\)]]]; \[IndentingNewLine]bb\ \  = \ 
          Table[Chop[b[\([i]\)]], {i, m}]; \[IndentingNewLine]b\  = \ 
          noprops[bb]; \[IndentingNewLine]Goto[
          start]; \[IndentingNewLine]Label[fin]; \[IndentingNewLine]m = 
          Length[c]/2; 
        Print["\<The rank is \>", \ 2*m]; \[IndentingNewLine]ea = 
          Sum[ssp[c[\([2*i - 1]\)], c[\([2*i]\)]], {i, 
              m}]; \[IndentingNewLine]If[Chop[m - ea]\  \[NotEqual] \ 0, 
          Print["\<Scalar products may not be evaluated!\>"]; 
          Return[]]; \[IndentingNewLine]If[Length[iso] \[NotEqual] \ 0, 
          iso\  = \ 
            outzero[RowReduce[
                iso]]]; \[IndentingNewLine]Print["\<The defect is \>", \ 
          Length[iso]]; \[IndentingNewLine]ea = \ 
          Sum[Chop[ssp[c[\([i]\)], iso[\([j]\)]]], {i, 2*m}, {j, 
                Length[iso]}] + 
            Sum[Chop[ssp[iso[\([i]\)], iso[\([j]\)]]], {i, Length[iso]}, {j, 
                Length[iso]}]; \[IndentingNewLine]If[
          Chop[ea]\  \[NotEqual] \ 0, 
          Print["\<Scalar products may not be evaluated!\>"]; 
          Return[]]; \[IndentingNewLine]If[
          or \[Equal] False && \ \ c \[NotEqual] \ {}, 
          c = symplectic[c]]; \[IndentingNewLine]Print["\<c = \>", \ c]; 
        Print["\<iso = \>", \ iso]; \[IndentingNewLine]c = 
          Join[c, iso]; \[IndentingNewLine]Return[c]]\)}], "Input",
  CellLabel->"In[151]:=",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{

Cell["Tests: symortho", "Subsubsection"],

Cell[BoxData[
    \(dim\  = \ 4\)], "Input",
  CellLabel->"In[195]:="],

Cell[BoxData[
    \(a = {{1, 0, 0, 0}, {1, 0, 0, 2}, {5, 0, 0, 0}, {0, 0, 3, 0}}\)], "Input",\

  CellLabel->"In[196]:="],

Cell[BoxData[
    \(\(?noprops\)\)], "Input",
  CellLabel->"In[197]:="],

Cell[BoxData[
    \(b = noprops[a]\)], "Input",
  CellLabel->"In[198]:="],

Cell[BoxData[
    \(symortho[a]\)], "Input",
  CellLabel->"In[199]:="],

Cell[BoxData[
    \(a\  = \ {stb[1], Pi*stb[1], null[], 3*stb[3]}\)], "Input",
  CellLabel->"In[200]:="],

Cell[BoxData[
    \(symortho[a]\)], "Input",
  CellLabel->"In[201]:="],

Cell[BoxData[
    \(Clear[a, b]\)], "Input",
  CellLabel->"In[202]:="],

Cell[BoxData[
    \(symortho[{null[], stb[4], 3*stb[4]}]\)], "Input",
  CellLabel->"In[203]:="],

Cell[BoxData[
    \(symortho[{null[], null[], stb[1]}]\)], "Input",
  CellLabel->"In[204]:="],

Cell[BoxData[
    \(symortho[{null[], null[], null[]}]\)], "Input",
  CellLabel->"In[205]:="],

Cell[BoxData[
    \(symortho[{2*stb[3], null[], stb[1]}]\)], "Input",
  CellLabel->"In[206]:="],

Cell[BoxData[
    \(symortho[{2, null[], stb[1]}]\)], "Input",
  CellLabel->"In[207]:="],

Cell[BoxData[
    \(symortho[{stb[2], 3, stb[1]}]\)], "Input",
  CellLabel->"In[208]:="],

Cell[BoxData[
    \(symortho[{stb[1], stb[3], {2, 0, 1, 0}}]\)], "Input",
  CellLabel->"In[209]:="],

Cell[BoxData[
    \(symortho[{stb[1], stb[3], {2, 0, 1, 0, 5}}]\)], "Input",
  CellLabel->"In[210]:="],

Cell[BoxData[
    \(symortho[{stb[1], stb[3], {2, 0, 1, 1}}]\)], "Input",
  CellLabel->"In[211]:="],

Cell[BoxData[
    \(symortho[{stb[1], stb[3], {2, 0, 1, 1}, randomv[]}]\)], "Input",
  CellLabel->"In[212]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, dim}, {j, dim}]\)], "Input",
  CellLabel->"In[213]:="],

Cell[BoxData[
    \(% // TableForm\)], "Input",
  CellLabel->"In[214]:="],

Cell[BoxData[
    \(symortho[{stb[1], stb[3], {2, 0, 1, 1}, randomv[]}, \ 
      orthosymplectic\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[215]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, dim}, {j, dim}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[216]:="],

Cell[BoxData[
    \(symortho[{stb[1], stb[3], {2, 0, 1, 1}, vec[x]}]\)], "Input",
  CellLabel->"In[217]:="],

Cell["\<\
For the next arguments one may not expect any reasonable \
result:\
\>", "Text"],

Cell[BoxData[
    \(symortho[{vec[u], vec[x]}]\)], "Input",
  CellLabel->"In[218]:="],

Cell[BoxData[
    \(symortho[{stb[1], 
        stb[4], {1, 0, 0, 1}, {1, 0, 0, 2}, {1, 0, 0, 3}}]\)], "Input",
  CellLabel->"In[219]:="],

Cell[BoxData[
    \(symortho[{stb[1], stb[4], {1, 0, 0, 1}, {1, 0, 0, 2}, {1, 0, 0, 3}}, \ 
      orthosymplectic\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[220]:="],

Cell[BoxData[
    \(symortho[{stb[1], stb[2], 
        stb[4], {1, 0, 0, 1}, {1, 0, 0, 2}, {1, 0, 0, 3}}]\)], "Input",
  CellLabel->"In[221]:="],

Cell[BoxData[
    \(symortho[{stb[1], stb[2], 
        stb[4], {1, 0, 0, 1}, {1, 0, 0, 2}, {1, 0, 0, 3}}, 
      orthosymplectic\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[222]:="],

Cell[BoxData[
    \(symortho[{stb[2], 
        stb[4], {1, 0, 0, 1}, {1, 0, 0, 2}, {1, 0, 0, 3}}]\)], "Input",
  CellLabel->"In[223]:="],

Cell[BoxData[
    \(symortho[IdentityMatrix[4], 
      orthosymplectic\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[224]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, dim}, {j, dim}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[225]:="],

Cell[BoxData[
    \(symortho[IdentityMatrix[5]]\)], "Input",
  CellLabel->"In[226]:="],

Cell[BoxData[
    \(symortho[IdentityMatrix[2]]\)], "Input",
  CellLabel->"In[227]:="],

Cell[BoxData[
    \(symortho[IdentityMatrix[6]]\)], "Input",
  CellLabel->"In[228]:="],

Cell[BoxData[
    \(symortho[IdentityMatrix[6], \ 
      orthosymplectic\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[229]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[230]:="],

Cell[BoxData[
    \(symortho[gram[]]\)], "Input",
  CellLabel->"In[231]:="],

Cell[BoxData[
    \(symortho[gram[], \ orthosymplectic\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[232]:="],

Cell[BoxData[
    \(Table[ssp[%[\([i]\)], %[\([j]\)]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[233]:="],

Cell[BoxData[
    \(ar[n_, k_] := Table[randomv[n], {k}]\)], "Input",
  CellLabel->"In[234]:="],

Cell[BoxData[
    \(ar[3, 4]\)], "Input",
  CellLabel->"In[235]:="],

Cell[BoxData[
    \(symortho[ar[4, 3]]\)], "Input",
  CellLabel->"In[236]:="],

Cell[BoxData[
    \(Table[
        Chop[ssp[%[\([i]\)], %[\([j]\)]]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[237]:="],

Cell[BoxData[
    \(symortho[ar[3, 3]]\)], "Input",
  CellLabel->"In[238]:="],

Cell[BoxData[
    \(symortho[ar[8, 26], orthosymplectic\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[239]:="],

Cell[BoxData[
    \(Table[
        Chop[ssp[%[\([i]\)], %[\([j]\)]]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[240]:="],

Cell[BoxData[
    \(\(\(sa\)\(\ \)\(=\)\(ar[4, 5]\)\(\ \)\)\)], "Input",
  CellLabel->"In[241]:="],

Cell[BoxData[
    \(\(\(\ \)\(symortho[sa]\)\)\)], "Input",
  CellLabel->"In[242]:="],

Cell[BoxData[
    \(Table[
        Chop[ssp[%[\([i]\)], %[\([j]\)]]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[243]:="],

Cell[BoxData[
    \(\(\(\ \)\(symortho[sa, orthosymplectic\  \[Rule] \ True]\)\)\)], "Input",\

  CellLabel->"In[244]:="],

Cell[BoxData[
    \(Table[
        Chop[ssp[%[\([i]\)], %[\([j]\)]]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[245]:="],

Cell[BoxData[
    \(symortho[ar[8, 10], \ orthosymplectic\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[246]:="],

Cell[BoxData[
    \(Table[
        Chop[ssp[%[\([i]\)], %[\([j]\)]]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[247]:="],

Cell[BoxData[
    \(Clear[sa]\)], "Input",
  CellLabel->"In[248]:="],

Cell[BoxData[
    \(dim\  = \ 6\)], "Input",
  CellLabel->"In[249]:="],

Cell[BoxData[
    \(stt\  = {stb[1], stb[4], stb[1] + stb[2], 
        stb[3] + stb[4], \ {1, 1, 1, 1, 0, 0}, randomv[]}\)], "Input",
  CellLabel->"In[250]:="],

Cell[BoxData[
    \(stt\  = \ ar[6, 5]\)], "Input",
  CellLabel->"In[251]:="],

Cell[BoxData[
    \(ergtt\  = symortho[stt]\)], "Input",
  CellLabel->"In[252]:="],

Cell[BoxData[
    \(Table[
        Chop[ssp[%[\([i]\)], %[\([j]\)]]], {i, Length[%]}, {j, Length[%]}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[253]:="],

Cell[BoxData[
    \(rank[stt]\)], "Input",
  CellLabel->"In[254]:="],

Cell[BoxData[
    \(rank[Join[stt, ergtt]]\)], "Input",
  CellLabel->"In[255]:="],

Cell[BoxData[
    \(dim = 4\)], "Input",
  CellLabel->"In[256]:="],

Cell[BoxData[
    \(Clear[stt, ergtt, ar]\)], "Input",
  CellLabel->"In[257]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["1.4.  Polar spaces", "Subsection"],

Cell[TextData[{
  "A ",
  StyleBox["projective k-plane",
    FontColor->RGBColor[0, 0, 1]],
  " H is defined as the set of all projective points corresponding to the \
vectors of a (k+1)-dimensional vector subspace W. The polar subspace of H is \
the set of all projective points represented by the vectors orthogonal to W. \
If  (a[i]), i = 1,..., k+1, is a basis of W, the space orthogonal to W is the \
NullSpace of the covectors (gram[].a[i]), i = 1,..., k+1."
}], "Text"],

Cell[BoxData[
    \(\(?NullSpace\)\)], "Input",
  CellLabel->"In[258]:="],

Cell[BoxData[
    \(vec[x] . gram[] . vec[y]\  \[Equal] \ ssp[vec[x], vec[y]]\)], "Input",
  CellLabel->"In[259]:="],

Cell["\<\
One remarks that vec[] is considered as a row- or a column vector \
depending on the place at which it appears in a matrix product.\
\>", "Text"],

Cell[BoxData[
    \(NullSpace[{gram[] . stb[1], gram[] . \((stb[2] + stb[3])\)}]\)], "Input",\

  CellLabel->"In[260]:="],

Cell[BoxData[
    \(ssp[%[\([1]\)], stb[2] + stb[3]]\)], "Input",
  CellLabel->"In[261]:="],

Cell[BoxData[
    \(ssp[%%[\([2]\)], stb[2] + stb[3]]\)], "Input",
  CellLabel->"In[262]:="],

Cell[BoxData[
    \(NullSpace[{gram[] . stb[1], gram[] . randomv[]}]\)], "Input",
  CellLabel->"In[263]:="],

Cell[BoxData[
    \(ssp[stb[1], %[\([1]\)]]\)], "Input",
  CellLabel->"In[264]:="],

Cell[BoxData[
    \(ssp[stb[1], %%[\([2]\)]]\)], "Input",
  CellLabel->"In[265]:="],

Cell[BoxData[
    \(Map[\((gram[] . #)\) &, {stb[1], \ stb[3]}]\)], "Input",
  CellLabel->"In[266]:="],

Cell[BoxData[
    \(gram[] . stb[3]\)], "Input",
  CellLabel->"In[267]:="],

Cell["\<\
Here is a Module for a basis of the vector space, underlying the \
polar space:\
\>", "Text"],

Cell[BoxData[
    \(\(polarspace::usage\  = \ "\<Given a List m of vectors, polarspace[m] \
is a basis of the vector subspace orthogonal to the span of m.\>";\)\)], \
"Input",
  CellLabel->"In[18]:=",
  InitializationCell->True],

Cell[BoxData[
    \(polarspace[m_List] := \ 
      Module[{b}, b\  = \ Map[\((gram[] . #)\) &, m]; \ 
        Chop[NullSpace[b]]]\)], "Input",
  CellLabel->"In[19]:=",
  InitializationCell->True],

Cell[BoxData[
    \(polarspace[{stb[2], stb[4]}]\)], "Input",
  CellLabel->"In[268]:="],

Cell[BoxData[
    \(a2\  = \ randomv[]; \ a3\  = \ randomv[]; 
    polarspace[{stb[2], a2, a3}]\)], "Input",
  CellLabel->"In[269]:="],

Cell[BoxData[
    \(b = \ %[\([1]\)]\)], "Input",
  CellLabel->"In[270]:="],

Cell[BoxData[
    \(ssp[b, a2]\)], "Input",
  CellLabel->"In[271]:="],

Cell[BoxData[
    \(ssp[b, a3]\)], "Input",
  CellLabel->"In[272]:="],

Cell[BoxData[
    \(Clear[a2, a3, b]\)], "Input",
  CellLabel->"In[273]:="],

Cell[BoxData[
    \(a2\  = \ randomv[]; \ a3\  = \ randomv[]; 
    polarspace[{a2, a3}]\)], "Input",
  CellLabel->"In[274]:="],

Cell[BoxData[
    \(b = %\)], "Input",
  CellLabel->"In[275]:="],

Cell[BoxData[
    \(ssp[b[\([1]\)], a2]\)], "Input",
  CellLabel->"In[276]:="],

Cell[BoxData[
    \(ssp[b[\([2]\)], a2]\)], "Input",
  CellLabel->"In[277]:="],

Cell[BoxData[
    \(ssp[b[\([2]\)], a3]\)], "Input",
  CellLabel->"In[278]:="],

Cell[BoxData[
    \(ssp[b[\([1]\)], a3]\)], "Input",
  CellLabel->"In[279]:="],

Cell["\<\
 Clearly, if the dimension of the subspace generated by the vectors \
of a_List is d, the dimension of the orthogonal vector space is dim -d.\
\>", \
"Text"],

Cell[BoxData[
    \(rank[b]\)], "Input",
  CellLabel->"In[280]:="],

Cell[BoxData[
    \(Clear[a2, a3, b]\)], "Input",
  CellLabel->"In[281]:="],

Cell["\<\
Up to unavoidable numerical errors the Module yields the wanted \
result. \
\>", "Text"],

Cell[BoxData[
    \(polarspace[{a*stb[1] + b*stb[2]}]\)], "Input",
  CellLabel->"In[282]:="],

Cell[BoxData[
    \(\(\(ssp[%[\([1]\)], a*stb[1] + b*stb[2]]\)\(\ \)\)\)], "Input",
  CellLabel->"In[283]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
1.5.  Projective points, spherical points, and the corresponding \
Graphics3D objects\
\>", "Subsection"],

Cell[CellGroupData[{

Cell["Projective points", "Subsubsection"],

Cell["\<\
The projective point corresponding to a vector, in the affine model \
consisting in the complement of the \"infinite\" (or \"absolute\") hyperplane \
x[dim] = 0, is an point of the (dim-1)-dimensional affine space given by \
\
\>", "Text"],

Cell[BoxData[
    \(\(projpt::usage\  = \ "\<projpt[vx] is the point in the affine model of \
the projective space corresponding to the vector vx; if the last component of \
vx equals 0, projpt returns ComplexInfinity.\>";\)\)], "Input",
  CellLabel->"In[20]:=",
  InitializationCell->True],

Cell[BoxData[
    \(projpt[vx_]\  := \ 
      Module[{dd\  = \ Length[vx], \ v\  = \ vx . vx}, \[IndentingNewLine]If[
          Chop[v]\  \[Equal] \ 0, \ 
          Return["\<nopt\>"]]; \[IndentingNewLine]If[
          Chop[vx[\([dd]\)]] \[Equal] \ 0, \ 
          Return[ComplexInfinity]]; \[IndentingNewLine]Take[vx/vx[\([dd]\)], 
          dd - 1]]\)], "Input",
  CellLabel->"In[21]:=",
  InitializationCell->True],

Cell["Only the points of the complement of the projective plane", "Text"],

Cell[BoxData[
    \(stb[4] . vec[x] \[Equal] \ 0\)], "Input",
  CellLabel->"In[284]:="],

Cell["\<\
are represented by points of the affine 3-space.  Obviously, projpt \
is homogeneous; for u \[NotEqual] 0 one has\
\>", "Text"],

Cell[BoxData[
    \(projpt[vec[x]*u]\)], "Input",
  CellLabel->"In[285]:="],

Cell[BoxData[
    \(projpt[vec[x]*u] \[Equal] \ projpt[vec[x]]\)], "Input",
  CellLabel->"In[286]:="],

Cell[BoxData[
    \(projpt[stb[4]]\)], "Input",
  CellLabel->"In[287]:="],

Cell[BoxData[
    \(projpt[stb[1]]\)], "Input",
  CellLabel->"In[288]:="],

Cell["We may choose a distinguished  representing vector:", "Text"],

Cell[BoxData[
    \(\(reprprojpt::usage\  = \ "\<vx = reprprojpt[px] is a vector with last \
component 1 representing the projective point px = projpt[vx].\>";\)\)], \
"Input",
  CellLabel->"In[22]:=",
  InitializationCell->True],

Cell[BoxData[
    \(reprprojpt[a_List]\  := Join[a, {1}]\)], "Input",
  CellLabel->"In[23]:=",
  InitializationCell->True],

Cell[BoxData[
    \(reprprojpt[{2, 0, 9}]\)], "Input",
  CellLabel->"In[289]:="],

Cell[BoxData[
    \(projpt[%]\)], "Input",
  CellLabel->"In[290]:="],

Cell["\<\
In the definition of projpt the dimension dim has been replaced by \
Length[vx]; the definition works in any dimension.\
\>", "Text"],

Cell[BoxData[
    \(dim\  \[Equal] \ Length[vec[x]]\)], "Input",
  CellLabel->"In[291]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Tests", "Subsubsection"],

Cell[BoxData[
    \(projpt[null[]]\)], "Input",
  CellLabel->"In[292]:="],

Cell[BoxData[
    \(projpt[stb[3]]\)], "Input",
  CellLabel->"In[293]:="],

Cell[BoxData[
    \(projpt[stb[4]]\)], "Input",
  CellLabel->"In[294]:="],

Cell[BoxData[
    \(\(?randomv\)\)], "Input",
  CellLabel->"In[295]:="],

Cell[BoxData[
    \(projpt[randomv[4]]\)], "Input",
  CellLabel->"In[296]:="],

Cell[BoxData[
    \(projpt[randomv[6]]\)], "Input",
  CellLabel->"In[297]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Spherical points", "Subsubsection"],

Cell["\<\
The spherical  point corresponding to a vector is the point whose \
radius vector is the normed vector:\
\>", "Text"],

Cell[BoxData[
    \(\(?dotnorme\)\)], "Input",
  CellLabel->"In[298]:="],

Cell["Only the null vector may not be normed: ", "Text"],

Cell[BoxData[
    \(dotnorme[null[]]\)], "Input",
  CellLabel->"In[299]:="],

Cell[BoxData[
    \(dotnorme[randomv[4]]\)], "Input",
  CellLabel->"In[300]:="],

Cell[BoxData[
    \(% . %\)], "Input",
  CellLabel->"In[301]:="],

Cell["\<\
To visualize the  points of the 3-sphere, which is a subset of the \
4-space, use stereographical projection:\
\>", "Text"],

Cell[BoxData[
    \(\(?sterproj\)\)], "Input",
  CellLabel->"In[302]:="],

Cell[BoxData[
    \(sterproj[stb[3]]\)], "Input",
  CellLabel->"In[303]:="],

Cell[BoxData[
    \(sterproj[stb[4]]\)], "Input",
  CellLabel->"In[304]:="],

Cell[BoxData[
    \(sterproj[\(-stb[4]\)]\)], "Input",
  CellLabel->"In[305]:="],

Cell[BoxData[
    \(randomv[]\)], "Input",
  CellLabel->"In[306]:="],

Cell[BoxData[
    \(sterproj[dotnorme[%]]\)], "Input",
  CellLabel->"In[307]:="],

Cell["Because", "Text"],

Cell[BoxData[
    \(projpt[vec[x]]\  \[Equal] \ projpt[\(-vec[x]\)]\)], "Input",
  CellLabel->"In[308]:="],

Cell["\<\
the 3-sphere is a twofold covering of the projective 3-space:\
\>", \
"Text"],

Cell[BoxData[
    \(va\  = \ randomv[3]; \ 
    Solve[{vec[x] . vec[x] \[Equal] \ 1, projpt[vec[x]] \[Equal] \ va}, 
      vec[x]]\)], "Input",
  CellLabel->"In[309]:="],

Cell[BoxData[
    \(v1\  = \ vec[x] /. %[\([1]\)]\)], "Input",
  CellLabel->"In[310]:="],

Cell[BoxData[
    \(v2\  = \ vec[x] /. %%[\([2]\)]\)], "Input",
  CellLabel->"In[311]:="],

Cell[BoxData[
    \(v1 + v2\)], "Input",
  CellLabel->"In[312]:="],

Cell[BoxData[
    \(v1 . v1\)], "Input",
  CellLabel->"In[313]:="],

Cell[BoxData[
    \(Clear[va, v1, v2]\)], "Input",
  CellLabel->"In[314]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["The corresponding Graphics3D objects", "Subsubsection"],

Cell["For the corresponding Graphics3D object set", "Text"],

Cell[BoxData[
    \(\(projpt3D::usage\  = \ "\<projpt3D[a] is the Graphics primitive Point \
of the corresponding projective point corresponding to the 4-dimensional \
vector a.\>";\)\)], "Input",
  CellLabel->"In[24]:=",
  InitializationCell->True],

Cell[BoxData[
    \(projpt3D[a_]\  := \ Point[projpt[a]]\)], "Input",
  CellLabel->"In[25]:=",
  InitializationCell->True],

Cell["Example", "Text"],

Cell[BoxData[
    \(\(pptlist\  = \ Table[10*randomv[], \ {300}];\)\)], "Input",
  CellLabel->"In[315]:="],

Cell[BoxData[
    \(\(Map[projpt, pptlist];\)\)], "Input",
  CellLabel->"In[316]:="],

Cell[BoxData[
    \(\(Map[projpt3D, pptlist];\)\)], "Input",
  CellLabel->"In[317]:="],

Cell[BoxData[
    \(Show[Graphics3D[{PointSize[0.02], %}], \ 
      Axes\  \[Rule] \ True]\)], "Input",
  CellLabel->"In[318]:="],

Cell["\<\
Of course, the resulting points are not equally distributed; use \
the option PlotRange -> All making this fact yet more obvious. 
Similarly for the spherical points under stereographic projection:\
\>", \
"Text"],

Cell[BoxData[
    \(spherpt3D[a_]\  := \ Point[sterproj[dotnorme[a]]]\)], "Input",
  CellLabel->"In[26]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(spherpt3D::usage\  = \ "\<If a is a vector of dimension 4, then \
spherpt3D[a] is the graphics3D primitive Point of the stereographical \
projection sterproj of the normed vector a.\>";\)\)], "Input",
  CellLabel->"In[27]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(Map[spherpt3D, pptlist];\)\)], "Input",
  CellLabel->"In[319]:="],

Cell[BoxData[
    \(Show[Graphics3D[{PointSize[0.03], %}]]\)], "Input",
  CellLabel->"In[320]:="]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2. The symplectic group, transvections", "Section"],

Cell["\<\
The transvections defined in this section should better be called \
\"symplectic transvections\": it will be proved that the symplectic scalar \
product remains invariant und their action.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Definitions", "Subsubsection"],

Cell[TextData[{
  "For a symplectic vector space the group of all linear transformations \
leaving the scalar product ssp invariant is named the ",
  StyleBox["symplectic group",
    FontColor->RGBColor[0, 0, 1]],
  " ."
}], "Text"],

Cell[TextData[{
  StyleBox["Transvections",
    FontColor->RGBColor[0, 0, 1]],
  " are very simple linear transformations of the following shape: "
}], "Text"],

Cell[BoxData[
    \(\(\(F[vec[x]]\  = \ vec[x]\  + \ vec[b]*f[vec[x]]\)\(,\)\)\)], "Input",
  Evaluatable->False],

Cell[TextData[{
  "where vec[b] is constant,  and f is a linear function with values in the \
field K fulfilling f[vec[b]] = 0.  Clearly,  if f \[NotEqual] 0, the set of \
the fixed points of F is the subspace of codimension 1 defined as the ",
  "annihilator",
  StyleBox[" ",
    FontColor->RGBColor[0, 0, 1]],
  " of f by f[vec[x]] = 0; this property  is characteristic for transvections \
F\[NotEqual] identity.  The projectivities defined by linear transvections \
are called transvections, too. In the projective model of the affine space \
the translations are those transvections whose hyperplane of fixed points is \
the absolute, i.e. the infinite hyperplane.\nIt is well known that any \
transformation with determinant 1 is a product of finitely many transvections \
see e.g. section 2.1.7 in  [",
  ButtonBox["OSIII",
    ButtonData:>"OSIII",
    ButtonStyle->"Hyperlink"],
  "].  Especially, any symplectic transformation is such a product. More \
precisely: ",
  StyleBox["Any symplectic transformation may be represented as a product of \
finitely many",
    FontSlant->"Italic"],
  " ",
  StyleBox["symplectic transvections",
    FontColor->RGBColor[0, 0, 1]],
  ", i.e. transvections leaving the symplectic scalar product invariant  (for \
a proof see e.g. section 2.8 of  [OSIII]). "
}], "Text"],

Cell["\<\
The  following Module yields a general symplectic \
transvection:\
\>", "Text"],

Cell[BoxData[
    \(\(transvection::usage\  = \ "\<transvection[b,c] is the symplectic \
transvection corresponding to the vector b and the scalar c.\>";\)\)], "Input",\

  CellLabel->"In[28]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(transvection[b_, c_]\)[x_]\  := \ 
      Module[{bb\  = Length[b], \ xx\  = \ Length[x]}, \[IndentingNewLine]If[
          bb\  \[NotEqual] \ xx || \ OddQ[bb], \ 
          Print["\<Incorrect arguments!\>"]; 
          Return[]]; \[IndentingNewLine]x\  + \ b*c*ssp[b, x]]\)], "Input",
  CellLabel->"In[29]:=",
  InitializationCell->True],

Cell["Fix the dimension", "Text"],

Cell[BoxData[
    \(dim\  = \ 4\)], "Input",
  CellLabel->"In[118]:="],

Cell[BoxData[
    \(\(transvection[vec[b], c]\)[vec[x]]\)], "Input",
  CellLabel->"In[119]:="],

Cell[BoxData[
    \(Solve[% \[Equal] \ vec[x], vec[x]]\)], "Input",
  CellLabel->"In[120]:="],

Cell[BoxData[
    \(svx = \((vec[x] /. %[\([1]\)])\)*b[3]\)], "Input",
  CellLabel->"In[121]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[122]:="],

Cell["vec[b] is the pole of the fixed points hyperplane svx:", "Text"],

Cell[BoxData[
    \(ssp[vec[b], %]\)], "Input",
  CellLabel->"In[123]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[124]:="],

Cell["\<\
The matrix of the transvection with respect to the symplectic basis \
is\
\>", "Text"],

Cell[BoxData[
    \(\(transv::usage\  = \ "\<transv[b,c] is the  matrix of \
transvection[b,c].\>";\)\)], "Input",
  CellLabel->"In[30]:=",
  InitializationCell->True],

Cell[BoxData[
    \(transv[b_, c_]\  := \ 
      Transpose[Table[\(transvection[b, c]\)[stb[i]], {i, dim}]]\)], "Input",
  CellLabel->"In[31]:=",
  InitializationCell->True],

Cell[BoxData[
    \(transv[vec[b], c]\)], "Input",
  CellLabel->"In[135]:="],

Cell[BoxData[
    \(% // MatrixForm\)], "Input",
  CellLabel->"In[128]:="],

Cell[BoxData[
    \(transv[vec[b], c] . vec[x] \[Equal] \ \(transvection[vec[b], c]\)[
        vec[x]]\)], "Input",
  CellLabel->"In[129]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[130]:="],

Cell[BoxData[
    \(Det[transv[vec[b], c]]\)], "Input",
  CellLabel->"In[131]:="],

Cell["The inverse of transvection[b,c] is the transvection[b,-c]:", "Text"],

Cell[BoxData[
    \(Inverse[transv[vec[b], c]] \[Equal] \ 
      transv[vec[b], \ \(-c\)]\)], "Input",
  CellLabel->"In[132]:="],

Cell["\<\
The product of transvections is not a transvection, in general. The \
set of all transvections with the same hyperplane of fixed points is an \
abelian group:\
\>", "Text"],

Cell[BoxData[
    \(Simplify[
      transv[vec[b], c1] . transv[vec[b], c2] \[Equal] \ 
        transv[vec[b], c1 + c2]]\)], "Input",
  CellLabel->"In[133]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Test: transvection", "Subsubsection"],

Cell[BoxData[
    \(\(transvection[stb[3], 2]\)[{1, 2, 3}]\)], "Input",
  CellLabel->"In[136]:="],

Cell[BoxData[
    \(\(transvection[stb[3, 3], 2]\)[{1, 2, 3}]\)], "Input",
  CellLabel->"In[137]:="],

Cell[BoxData[
    \(stb[3, 3]\)], "Input",
  CellLabel->"In[138]:="],

Cell[BoxData[
    \(\(transvection[stb[3], c]\)[vec[x]]\)], "Input",
  CellLabel->"In[139]:="],

Cell[BoxData[
    \(tb[x_] := \(transvection[vec[b], c]\)[vec[x]]\)], "Input",
  CellLabel->"In[140]:="],

Cell[BoxData[
    \(tb[x]\)], "Input",
  CellLabel->"In[141]:="],

Cell[BoxData[
    \(ssp[tb[x], tb[y]]\  \[Equal] \ ssp[vec[x], vec[y]]\)], "Input",
  CellLabel->"In[142]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[143]:="],

Cell["\<\
It follows, as stated: Any symplectic transvection is a symplectic \
transformation. \
\>", "Text"],

Cell[BoxData[
    \(Clear[tb]\)], "Input",
  CellLabel->"In[144]:="],

Cell["For b = stb[4] the set of fixed points  is found by:", "Text"],

Cell[BoxData[
    \(Solve[\(transvection[stb[4], \ c]\)[vec[x]] \[Equal] \ vec[x], \ 
      vec[x]]\)], "Input",
  CellLabel->"In[145]:="],

Cell["It is, as stated, the polar plane of stb[4]", "Text"],

Cell[BoxData[
    \(Solve[ssp[stb[4], vec[x]] \[Equal] \ 0, vec[x]]\)], "Input",
  CellLabel->"In[146]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Example of a symplectic transformation not being a \
transvection:\
\>", "Subsubsection"],

Cell[BoxData[
    \(Clear[a, b, c]; \ 
    g[a_] := \ {{1, \(-1\), 0, 0}, {0, 1, 0, 0}, {0, 1, 1, 0}, {1, a, 1, 
          1}}\)], "Input",
  CellLabel->"In[147]:="],

Cell[BoxData[
    \(g[a] // MatrixForm\)], "Input",
  CellLabel->"In[148]:="],

Cell[BoxData[
    \(Det[%]\)], "Input",
  CellLabel->"In[149]:="],

Cell[BoxData[
    \(Transpose[g[a]] . gram[] . g[a]\)], "Input",
  CellLabel->"In[150]:="],

Cell[BoxData[
    \(% \[Equal] \ gram[]\)], "Input",
  CellLabel->"In[151]:="],

Cell[BoxData[
    \(h[a_]\  := \ g[a] - IdentityMatrix[4]\)], "Input",
  CellLabel->"In[152]:="],

Cell[BoxData[
    \(h[a] // MatrixForm\)], "Input",
  CellLabel->"In[153]:="],

Cell[BoxData[
    \(\(?rank\)\)], "Input",
  CellLabel->"In[154]:="],

Cell[BoxData[
    \(rank[h[a]]\)], "Input",
  CellLabel->"In[155]:="],

Cell["\<\
The rank of transv[vec[b],c]-IdentityMatrix[4] is 1 for all \
transvections with vec[b] \[NotEqual] null[]:\
\>", "Text"],

Cell[BoxData[
    \(rank[transv[vec[b], c] - IdentityMatrix[4]]\)], "Input",
  CellLabel->"In[156]:="],

Cell["Thus, g may not be a transvection.", "Text"],

Cell[BoxData[
    \(Clear[g, h, a, b]\)], "Input",
  CellLabel->"In[157]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Example: Affine translations are not symplectic, in general", \
"Subsubsection"],

Cell["\<\
In the affine modell, a translation of a point a with coordinates \
vec[x,3] by a vector vec[a,3] is defined as\
\>", "Text"],

Cell[BoxData[
    \(Clear[x, a]\)], "Input",
  CellLabel->"In[116]:="],

Cell[BoxData[
    \(apt[x_]\  := \ {x[1], x[2], x[3], 1}\)], "Input",
  CellLabel->"In[117]:="],

Cell[BoxData[
    \(avt[x_]\  := \ {x[1], x[2], x[3], 0}\)], "Input",
  CellLabel->"In[118]:="],

Cell[BoxData[
    \(transl[x_, a_]\  := \ apt[x]\  + \ avt[a]\)], "Input",
  CellLabel->"In[119]:="],

Cell[BoxData[
    \(apt[y]\  + \ avt[b]\)], "Input",
  CellLabel->"In[120]:="],

Cell[BoxData[
    \(transl[x, a]\)], "Input",
  CellLabel->"In[121]:="],

Cell[BoxData[
    \(ssp[apt[x], apt[y]] - ssp[transl[x, a], transl[y, a]]\)], "Input",
  CellLabel->"In[122]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[123]:="]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3. Lines. The line invariant ", "Section"],

Cell["\<\
In this section we introduce the basic invariant sym of pairs of \
symplectic lines, and show some of its properties. \
\>", "Text"],

Cell[CellGroupData[{

Cell["3.1.  Symplectic and  isotropic lines", "Subsection"],

Cell[CellGroupData[{

Cell["Definitions", "Subsubsection"],

Cell["\<\
A projective line is generated by the projective points of a \
2-dimensional vector subspace. If x and y are distinct points of the line, \
represented by the linearely independent vectors vec[x], vec[y], then the \
linear hull of these vectors span the mentioned subspace, and any point of \
the line is obtained by its parameter representation\
\>", "Text"],

Cell[BoxData[
    \(\(parrepr[x_, y_]\)[t_]\  := \ x*t\  + y*\((1 - t)\)\)], "Input",
  CellLabel->"In[236]:="],

Cell[BoxData[
    \(a\  = randomv[]; b\  = 2*randomv[]; \ 
    ParametricPlot3D[
      projpt[\(parrepr[a, b]\)[t]], \ {t, \(-2\), 2}]\)], "Input",
  CellLabel->"In[241]:="],

Cell[BoxData[
    \(ssp[a, b]\)], "Input",
  CellLabel->"In[238]:="],

Cell[BoxData[
    \(Clear[a, b]\)], "Input",
  CellLabel->"In[244]:="],

Cell[TextData[{
  "A line is called ",
  StyleBox["symplectic",
    FontColor->RGBColor[0, 0, 1]],
  ", if the generating vector subspace is symplectic, and ",
  StyleBox["isotropic",
    FontColor->RGBColor[0, 0, 1]],
  ", if this space is isotropic. The line h= x\[Or]y is isotropic iff \
ssp[vec[x], vec[y]] =0. The set of isotropic lines is a 3-dimensional \
submanifold in the 4-dimensional Grassmann manifold of all lines, called \
the",
  StyleBox[" linear line complex ",
    FontColor->RGBColor[0, 0, 1]],
  "of the nullsystem defined by the symplectic scalar product; it is the ",
  StyleBox["absolute",
    FontColor->RGBColor[0, 0, 1]],
  " of 3-dimensional symplectic projective geometry. Clearly, a random line, \
defined connecting two random points, is symplectic with probability 1.  One \
shows easily: "
}], "Text"],

Cell[TextData[{
  StyleBox["Theorem", "Text",
    FontWeight->"Bold"],
  StyleBox[".  ", "Text"],
  StyleBox["The symplectic group acts transitively on the set of isotropic \
lines and on the set of symplectic lines.", "Text",
    FontSlant->"Italic"]
}], "Text"],

Cell["\<\
To get a geometric idea of this linear line complex consider a \
hyperplane H of the N-dimensional projective space. The pole h is a point of \
H, and the isotropic lines belonging to H are exactly the lines of the line \
bundle through h. \
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Pairs of lines", "Subsubsection"],

Cell["\<\
The symplectical group acts elementwise on the set of all \
non-ordered pairs {A, B} of lines: g{A, B} = {gA, gB}, g a symplectic \
transformation. We consider the pairs {A,B} with A\[NotEqual] B only, and \
identify the lines with the corresponding 2-dimensional vector subspaces. \
Since the symplectic action on the vector space is linear, one has for A = a1\
\[Or]a2 : gA = g(a1)\[Or]g(a2). We denote the homogeneous coordinates of a \
point x by vec[x]; let B = b1\[Or]b2 and consider the rank\
\>", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(rank[{vec[a1], vec[a2], \ vec[b1], \ vec[b2]}]\)\)\)], "Input",\

  CellLabel->"In[215]:="],

Cell["\<\
This is the rank of points in general position; also for random \
points one has\
\>", "Text"],

Cell[BoxData[
    \(a1\  = \ randomv[]; a2\  = \ randomv[]; b1\  = \ randomv[]; 
    b2\  = \ randomv[];\)], "Input",
  CellLabel->"In[216]:="],

Cell[BoxData[
    \(rank[{a1, \ a2, \ b1, \ b2}]\)], "Input",
  CellLabel->"In[217]:="],

Cell[BoxData[
    \(Clear[a1, a2, b1, b2]\)], "Input",
  CellLabel->"In[218]:="],

Cell[TextData[{
  "Clearly, the rank is the dimension of the vector subspace spanned by the \
vectors, representing the points, or the 2-dimensional subspaces A,B. \
Therefore we can call it the ",
  StyleBox["dimension of the line pair {A,B}.",
    FontColor->RGBColor[0, 0, 1]],
  " The rank is invariant under arbitrary linear ( or semilinear) \
transformations. Its possible values are:\ndimension =2 in case A = B,\t\
dimension =3 in case A\[Or]B is a plane (coplanar lines), ",
  "dimension",
  " = 4 in case A\[Or]B is a 3-dimensional projective subspace ."
}], "Text"],

Cell[TextData[{
  "Leaving the trivial case dimension = 2 aside we consider first coplanar \
pairs. There exist 3 possibilities, corresponding to the number k = 0,1,2 of \
isotropic lines of the pair. One easily shows that the symplectic group acts \
transitively on the set of coplanar pairs of lines of the same type, see \
corollary 2.8.8 in [",
  ButtonBox["OSIII",
    ButtonData:>"OSIII",
    ButtonStyle->"Hyperlink"],
  "].  Thus we have  only the discrete invariants dimension = 3 and k. A \
continuous invariant appears only in the case dimension = 4, which we \
consider in the next section "
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.2. The symplectic invariant of a pair of symplectic lines", \
"Subsection"],

Cell["\<\
In this section we continue the consideration of line pairs. We \
first assume that the lines A = x\[Or]y, B = u\[Or]v are symplectic, and \
define \
\>", "Text"],

Cell[BoxData[
    \(Clear[x, y, u, v, a, b, c, d]\)], "Input",
  CellLabel->"In[175]:="],

Cell[BoxData[
    \(\(sym::usage\  = \ "\<sym[x, y, u, v] yields a symplectic invariant of \
two projective symplectic lines spanned by the vectors (x,y) and (u,v) \
respectively; for isotropic lines it is Infinity or Indeterminate.\>";\)\)], \
"Input",
  CellLabel->"In[32]:=",
  InitializationCell->True],

Cell[BoxData[
    \(sym[x_, y_, u_, 
        v_]\  := \((ssp[x, u]*ssp[y, v] - ssp[x, v]*ssp[y, u])\)/\((ssp[x, y]*
            ssp[u, v])\)\)], "Input",
  CellLabel->"In[33]:=",
  InitializationCell->True],

Cell["\<\
Abusing the notation,  in this definition we identified the points \
with their representing vectors. Indeed, one easily proves that sym does not \
change if the representing vectors of the lines are changed: \
\>", "Text"],

Cell[BoxData[
    \(Simplify[
      sym[vec[x]*a + vec[y]*b, vec[x]*c + vec[y]*d, \ vec[u], 
          vec[v]] \[Equal] \ 
        sym[vec[x], vec[y], \ vec[u], vec[v]]]\)], "Input",
  CellLabel->"In[245]:="],

Cell["\<\
Therefore we may consider sym(A,B) := sym[x, y, u, v] for  A = x\
\[Or]y, B = u\[Or]v  as well defined. The function sym is symmetric: sym(A,B) \
= sym(B,A):\
\>", "Text"],

Cell[BoxData[
    \(Simplify[
      sym[vec[x], vec[y], \ vec[u], vec[v]] \[Equal] \ 
        sym[vec[u], vec[v], \ vec[x], vec[y]]]\)], "Input",
  CellLabel->"In[246]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["3.3. Geometric Interpretation", "Subsection"],

Cell[TextData[{
  "For a geometric interpretation  of the invariant sym we consider a fixed \
symplectic line ",
  StyleBox["A",
    FontWeight->"Bold"],
  ", and a point ",
  StyleBox["b",
    FontWeight->"Bold"],
  " belonging neither to ",
  StyleBox["A",
    FontWeight->"Bold"],
  " nor to its polar line ",
  Cell[BoxData[
      FormBox[
        StyleBox[\(A\^ \[UpTee] \),
          FontWeight->"Bold"], TraditionalForm]]],
  ".  We want to find all symplectic lines ",
  StyleBox["B",
    FontWeight->"Bold"],
  " through ",
  StyleBox["b",
    FontWeight->"Bold"],
  ", for which sym[",
  StyleBox["A,B",
    FontWeight->"Bold"],
  "] = c has a fixed value c. "
}], "Text"],

Cell[CellGroupData[{

Cell["Frame adaption, pole, polare", "Subsubsection"],

Cell[TextData[{
  " We remember that the value sym[",
  StyleBox["A,C",
    FontWeight->"Bold"],
  "] =1 for any symplectic line ",
  StyleBox["C",
    FontWeight->"Bold"],
  " intersectiing ",
  StyleBox["A",
    FontWeight->"Bold"],
  ", and sym[",
  StyleBox["A,C",
    FontWeight->"Bold"],
  "] = 0 for any ",
  StyleBox["C",
    FontWeight->"Bold"],
  " intersecting the polar line ",
  Cell[BoxData[
      \(TraditionalForm\`\(A\^ \[UpTee] \)\)],
    FontWeight->"Bold"],
  " (see exercise 2.8.4 in [OSIII]), and the example below. "
}], "Text"],

Cell["\<\
In this subsection we shall use the fixed symplectic base (stb[i]), \
and consider the affine space complementary to the plane x[4] = 0. The \
projective point [stb[4]] is the origin of the affine coordinate system, and \
its axes are spanned by {stb[4], stb[i]}, i = 1,2,3, or, if one wants to \
avoid the infinite points [stb[i]],  they may be spanned by stb[4] and the \
unit points stb[4] + stb[i] of the x[i]- axes. Remark, that from the \
symplectic point of view, the axes are not equivalent; only the x[2]-axis is \
symplectic, the two others are isotropic. For convenience we define the \
affine coordinates of an affine point  as follows\
\>", "Text"],

Cell[BoxData[
    \({x, y, z}\  = {x[1], x[2], x[3]}; \ x[4]\  = \ 1\)], "Input",
  Evaluatable->False],

Cell["The x-axis and the z-axis are isotropic:", "Text"],

Cell[BoxData[
    \(ssp[{0, 0, 0, 1}, stb[1] + stb[4]]\)], "Input",
  CellLabel->"In[247]:="],

Cell[BoxData[
    \(ssp[{0, 0, 0, 1}, stb[3] + stb[4]]\)], "Input",
  CellLabel->"In[248]:="],

Cell["but the y-axis is symplectic:", "Text"],

Cell[BoxData[
    \(ssp[{0, 0, 0, 1}, stb[2] + stb[4]]\)], "Input",
  CellLabel->"In[249]:="],

Cell[TextData[{
  " The polar of the unit point ",
  StyleBox["e3",
    FontWeight->"Bold"],
  " = [stb[3]+stb[4]] of the z-axiis is the plane ",
  StyleBox["E3",
    FontWeight->"Bold"],
  " defined by x + y = 0 containing the z-axis"
}], "Text"],

Cell[BoxData[
    \(Solve[ssp[{0, 0, 1, 1}, vec[x]] \[Equal] \ 0, \ vec[x]]\)], "Input",
  CellLabel->"In[250]:="],

Cell[BoxData[
    \(polarspace[{{0, 0, 1, 1}}]\)], "Input",
  CellLabel->"In[251]:="],

Cell[TextData[{
  StyleBox["E3",
    FontWeight->"Bold"],
  "  is the plane spanned by the vectors {0,0,1}, {-1,1,0},and the point ",
  StyleBox["e3",
    FontWeight->"Bold"],
  " or any other point of the z-axis."
}], "Text"],

Cell[TextData[{
  "The isotropic lines through ",
  StyleBox["e3",
    FontWeight->"Bold"],
  " are just the lines connecting ",
  StyleBox["e3 ",
    FontWeight->"Bold"],
  "\[Element] ",
  StyleBox["E3",
    FontWeight->"Bold"],
  "  with another point of ",
  StyleBox["E3",
    FontWeight->"Bold"],
  ".  Especially,  the z-axes is isotropic, as has been remarked before."
}], "Text"],

Cell[TextData[{
  "Without restriction of generality we may assume that a fixed non-isotropic \
line denoted by ",
  StyleBox["A",
    FontWeight->"Bold"],
  "  is the y-axis, spanned by the origin and its unit point:"
}], "Text"],

Cell[BoxData[
    \(ssp[{0, 0, 0, 1}, {0, 1, 0, 1}]\)], "Input",
  CellLabel->"In[252]:="],

Cell[TextData[{
  "The polar of this line is the  symplectic line ",
  Cell[BoxData[
      FormBox[
        StyleBox[\(A\^ \[UpTee] \),
          FontWeight->"Bold"], TraditionalForm]]],
  " = [stb[1], stb[3]] in the infinite plane"
}], "Text"],

Cell[BoxData[
    \(polarspace[{{0, 0, 0, 1}, {0, 1, 0, 1}}]\)], "Input",
  CellLabel->"In[253]:="],

Cell[BoxData[
    \(ssp[stb[1], stb[3]]\)], "Input",
  CellLabel->"In[254]:="],

Cell["\<\
In the affine geometry it corresponds to all vectors parallel to \
the x,z-plane.\
\>", "Text"],

Cell[TextData[{
  "An arbitrary point neither belonging to the y-axis ",
  StyleBox["A",
    FontWeight->"Bold"],
  " nor to its polar ",
  Cell[BoxData[
      \(TraditionalForm\`\(A\^ \[UpTee] \)\)],
    FontWeight->"Bold"],
  " can be taken to be the unit point ",
  StyleBox["b",
    FontWeight->"Bold"],
  " = ",
  StyleBox["e3",
    FontWeight->"Bold"],
  " of the z-axis (adapt the frame!). Consider a line ",
  StyleBox["C ",
    FontWeight->"Bold"],
  "connecting ",
  StyleBox["b",
    FontWeight->"Bold"],
  " with an arbitrary point ",
  StyleBox["y",
    FontWeight->"Bold"],
  " = [{0, y, 0, 1}] of the y-axis. This line is symplectic for all y\
\[NotEqual]0:"
}], "Text"],

Cell[BoxData[
    \(ssp[{0, 0, 1, 1}, {0, y, 0, 1}]\)], "Input",
  CellLabel->"In[255]:="],

Cell["Therefore one has", "Text"],

Cell[BoxData[
    \(sym[stb[4], \ stb[2], \ {0, 0, 1, 1}, {0, y, 0, 1}]\)], "Input",
  CellLabel->"In[256]:="],

Cell[TextData[{
  " Of course,  in such general formulas ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " gives the generic result only; the exceptional cases of the parameters, \
here y = 0,  are neglected. For y = 0 the result is wrong, since then ",
  StyleBox["C",
    FontWeight->"Bold"],
  " is the z-axis being isotrop:"
}], "Text"],

Cell[BoxData[
    \(sym[stb[4], \ stb[2], \ {0, 0, 1, 1}, {0, 0, 0, 1}]\)], "Input",
  CellLabel->"In[257]:="],

Cell[TextData[{
  "The exceptional role of the origin comes from the fact that it is the pole \
of the x, z-plane ",
  Cell[BoxData[
      FormBox[
        StyleBox[\(A\^ \[UpTee] \),
          FontWeight->"Bold"], TraditionalForm]]],
  "\[Or] ",
  StyleBox["b ",
    FontWeight->"Bold"],
  ":"
}], "Text"],

Cell[BoxData[
    \(polarspace[{stb[3] + stb[4], \ stb[1], stb[3]}]\)], "Input",
  CellLabel->"In[259]:="],

Cell[TextData[{
  "A line through",
  StyleBox[" b",
    FontWeight->"Bold"],
  " intersects the polar ",
  Cell[BoxData[
      \(TraditionalForm\`\(A\^ \[UpTee] \)\)],
    FontWeight->"Bold"],
  " iff it is parallel to the x,z-plane, such a line is spanned by ",
  StyleBox["b",
    FontWeight->"Bold"],
  " and a vector{x,0,z,0}. Indeed,  one has "
}], "Text"],

Cell[BoxData[
    \(sym[stb[4], \ stb[2], \ {0, 0, 1, 1}, {x, 0, z, 0}]\)], "Input",
  CellLabel->"In[260]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["===>>>The general case", "Subsubsection"],

Cell[TextData[{
  "Now we will find all the points ",
  StyleBox["x",
    FontWeight->"Bold"],
  " = [{x,y,z,w}] of the projective space for which the line ",
  StyleBox["C",
    FontWeight->"Bold"],
  " =",
  StyleBox[" b",
    FontWeight->"Bold"],
  " \[Or] ",
  StyleBox["x ",
    FontWeight->"Bold"],
  StyleBox["paired with the y-axis ",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox["A",
    FontWeight->"Bold",
    FontVariations->{"CompatibilityType"->0}],
  StyleBox[" gives the invariant sym the constant value c \[NotEqual] 0:",
    FontVariations->{"CompatibilityType"->0}]
}], "Text"],

Cell[BoxData[
    \(Clear[c]; 
    Solve[sym[stb[4], \ stb[2], \ {0, 0, 1, 1}, {x, y, z, w}] \[Equal] \ 
        c, {x, y, z, w}]\)], "Input",
  CellLabel->"In[83]:="],

Cell[TextData[{
  "The solution is the plane containing the z-axis defined by c x + (c-1)y = \
0.  For c = 1 we get the y,z-plane x= 0, and this corresponds to the case \
discussed above. ",
  StyleBox["A",
    FontWeight->"Bold"],
  " \[And] ",
  StyleBox["C",
    FontWeight->"Bold"],
  " \[NotEqual] \[EmptySet].  For arbitrary c the plane is spanned by the \
z-axis and its intersection with the x,y-plane, being the line"
}], "Text"],

Cell[BoxData[
    \(lin[t_, c_]\  := \ {\((1\  - \ c)\)*t, \ c*t, \ 0, \ 1}\)], "Input",
  CellLabel->"In[84]:="],

Cell["through the origin; indeed", "Text"],

Cell[BoxData[
    \(sym[stb[4], \ stb[2], \ {0, 0, 1, 1}, lin[t, c]]\)], "Input",
  CellLabel->"In[85]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[86]:="],

Cell["Thus we obtained the following ", "Text"],

Cell[TextData[{
  StyleBox["Geometric interpretation:",
    FontSize->16,
    FontColor->RGBColor[1, 0, 0]],
  " \n",
  StyleBox["Let ",
    FontSlant->"Italic"],
  StyleBox["A",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[" be a symplectic line, and be given a point ",
    FontSlant->"Italic"],
  StyleBox["b",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[" neither belonging to ",
    FontSlant->"Italic"],
  StyleBox["A",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[" nor to its polar ",
    FontSlant->"Italic"],
  Cell[BoxData[
      FormBox[
        StyleBox[\(A\^ \[UpTee] \),
          FontWeight->"Bold"], TraditionalForm]]],
  StyleBox[" .  Let ",
    FontSlant->"Italic"],
  StyleBox["o",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[" denote the intersection ",
    FontSlant->"Italic"],
  StyleBox["o",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[" = ",
    FontSlant->"Italic"],
  StyleBox["A ",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox["\[And]",
    FontSlant->"Italic"],
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(\(\(b\)\(\ \)\)\^ \[UpTee] \)\)\)],
    FontWeight->"Bold"],
  "= ",
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              FormBox[
                StyleBox[\(A\^ \[UpTee] \),
                  FontWeight->"Bold"],
                "TraditionalForm"], "\[Or]", " ", 
              StyleBox["b",
                FontWeight->"Bold"]}], ")"}], "\[UpTee]"], 
        TraditionalForm]]],
  ".  Then the line ",
  StyleBox["  ",
    FontSlant->"Italic"],
  StyleBox["o",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[" \[Or] ",
    FontSlant->"Italic"],
  StyleBox["b ",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  " is isotropic. ",
  StyleBox["The invariant sym of the line pair (",
    FontSlant->"Italic"],
  StyleBox["A",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[", ",
    FontSlant->"Italic"],
  StyleBox["b ",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox["\[Or] ",
    FontSlant->"Italic"],
  StyleBox["x",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[") is constant for all points ",
    FontSlant->"Italic"],
  StyleBox["x \[NotEqual] b ",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[" of a  plane of the plane bundle through  ",
    FontSlant->"Italic"],
  StyleBox["o",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[" \[Or] ",
    FontSlant->"Italic"],
  StyleBox["b",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox[", for which ",
    FontSlant->"Italic"],
  StyleBox["b ",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox["\[Or] ",
    FontSlant->"Italic"],
  StyleBox["x",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  StyleBox["  is symplectic.",
    FontSlant->"Italic"]
}], "Text"],

Cell[TextData[{
  "For arbitrary c the line lin[t,c] defined above is the intersection of a \
plane containing the z-axis with the x,y-plane. For t = 0 we have ",
  StyleBox["o",
    FontWeight->"Bold"],
  " =[lin[0, c]] for all c and for t = 1 we get the point ",
  StyleBox["p",
    FontWeight->"Bold"],
  "[c] belonging to the line x+y = 1, z = 0, of the x,y-plane. Representing \
this point by "
}], "Text"],

Cell[BoxData[
    \(pt[c_]\  := \ lin[1, c]\)], "Input",
  CellLabel->"In[89]:="],

Cell[TextData[{
  "we obtain with (",
  StyleBox["A,  C",
    FontWeight->"Bold"],
  "), ",
  StyleBox["C ",
    FontWeight->"Bold"],
  "= ",
  StyleBox["b ",
    FontWeight->"Bold"],
  "\[Or] ",
  StyleBox["p",
    FontWeight->"Bold"],
  "[c], a symplectic line with sym[",
  StyleBox["A,C",
    FontWeight->"Bold"],
  "] = c: "
}], "Text"],

Cell[BoxData[
    \(sym[stb[4], \ stb[2], \ {0, 0, 1, 1}, pt[c]]\)], "Input",
  CellLabel->"In[90]:="],

Cell[BoxData[
    \(ssp[{0, 0, 1, 1}, pt[c]]\)], "Input",
  CellLabel->"In[91]:="],

Cell[TextData[{
  "If c runs, then line[t,c] runs over all lines in the x,y-plane through the \
origin ",
  StyleBox["o",
    FontWeight->"Bold"],
  " with exception of the line defined by x+y = 0, which is parallel to x+y = \
1. But this line is the intersection of the x,y-plane with the polar  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\(b\)\(\ \)\)\^ \[UpTee] \)\)],
    FontWeight->"Bold"],
  " of ",
  StyleBox["b",
    FontWeight->"Bold"],
  ", and the lines ",
  StyleBox["b",
    FontWeight->"Bold"],
  " \[Or] ",
  StyleBox["x",
    FontWeight->"Bold"],
  " are isotropic for ",
  StyleBox["x",
    FontWeight->"Bold"],
  " \[Element]  ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\(b\)\(\ \)\)\^ \[UpTee] \)\)],
    FontWeight->"Bold"],
  " . The poles of the planes of the considered bundle are the points"
}], "Text"],

Cell[BoxData[
    \(polarspace[{stb[4], \ stb[3], \ lin[1, c]}]\)], "Input",
  CellLabel->"In[92]:="],

Cell[TextData[{
  "The poles run over the z-axis; for c \[ShortRightArrow] 1 we obtain its \
infinite point of the z-axis: [",
  Cell[BoxData[
      \({0, 0, c\/\(\(-1\) + c\), 1}\)]],
  "] =[{0, 0, c, -1+c}] \[ShortRightArrow] [{0, 0, 1, 0}]. Remember that in \
this case the plane of the bundle is the y,z-plane, and the lines ",
  StyleBox["A",
    FontWeight->"Bold"],
  ", ",
  StyleBox["C",
    FontWeight->"Bold"],
  " intersect."
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["3.4. The invariant sym and the scalar product for 2-vectors", \
"Subsection"],

Cell["\<\
The invariant sym may be defined in the framework of tensor \
algebra.  It is well known that a scalar product on a vector space may be \
lifted to scalar products of the tensor spaces over this vector space. \
\>", \
"Text"],

Cell["\<\
In particular, the scalar product induces a symmetric bilinear form \
in the space of 2-vectors, defined by\
\>", "Text"],

Cell[BoxData[
    \(linessp[x_, y_, u_, v_]\  := \ 
      ssp[x, u]*ssp[y, v] - ssp[x, v]*ssp[y, u]\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(linessp[vec[x], vec[y], vec[x], vec[y]]\)], "Input",
  CellLabel->"In[94]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[95]:="],

Cell[BoxData[
    \(Table[linessp[stb[2], stb[4], \ stb[2], stb[i]], {i, 4}]\)], "Input",
  CellLabel->"In[96]:="],

Cell[BoxData[
    \(linessp[vec[x], vec[y], vec[u], 
        vec[v]]/\((ssp[vec[x], vec[y]]*ssp[vec[u], vec[v]])\)\)], "Input",
  CellLabel->"In[97]:="],

Cell["The invariant sym  coincides with the normed scalar product:", "Text"],

Cell[BoxData[
    \(% \[Equal] \ sym[vec[x], vec[y], vec[u], vec[v]]\)], "Input",
  CellLabel->"In[98]:="],

Cell[TextData[{
  StyleBox["Remark",
    FontVariations->{"Underline"->True}],
  ". It can be proved that two pairs {A,B}, {C,D} of  lines of ",
  "dimension",
  " = 4  in the 3-dimensional symplectic  projective space each of which \
contains at least one isotropic line are symplectically congruent if either \
only one or two of the lines in each of the pairs are isotropic, see theorems \
2.8.15, 2.8.16 in [",
  ButtonBox["OSIII",
    ButtonData:>"OSIII",
    ButtonStyle->"Hyperlink"],
  "]. "
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
Example of a line pair  in the 5-dimensional projective symplectic \
space spanning an isotropic projective 3-space\
\>", "Subsubsection"],

Cell[BoxData[
    \(dim\  = \ 6\)], "Input",
  CellLabel->"In[104]:="],

Cell[BoxData[
    \(sym[stb[1], stb[4], stb[1]\  + stb[2], stb[3]\  + stb[4]]\)], "Input",
  CellLabel->"In[105]:="],

Cell[BoxData[
    \(symortho[{stb[1], stb[4], stb[1]\  + stb[2], 
        stb[3]\  + stb[4]}]\)], "Input",
  CellLabel->"In[106]:="],

Cell["\<\
 It follows that the lines are symplectic. The span is the space \
spanned by {stb[i]}, i = 1,...,4. We have\
\>", "Text"],

Cell[BoxData[
    \(Table[ssp[stb[i], stb[j]], \ {i, 4}, {j, 4}] // MatrixForm\)], "Input",
  CellLabel->"In[107]:="],

Cell["\<\
Clearly, in the 3-dimensional projective space such an example may \
not exist, since the only 4-dimensional vector subspace which exists is the \
space itself,  being symplectic.\
\>", "Text"],

Cell[BoxData[
    \(dim\  = \ 4\)], "Input",
  CellLabel->"In[108]:="]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["4. Associated operators and symmetric  bilinear forms", "Section"],

Cell[TextData[{
  "The aim of this and the following section is the classification of the \
quadrics in the 3-dimensional projective symplectic space.  As in other \
linear geometries one classifies the symmetric bilinear forms and applies the \
results to describe the symplectic geometry of the quadrics. To any symmetric \
 bilinear form corresponds a so-called ",
  StyleBox["skew symmetric associated operator ",
    FontColor->RGBColor[0, 0, 1]],
  "of the underlying 4-dimensional symplectic vector space, and this \
correspondence is equivariant and bijective: The associated operators are  \
symplectically equivalent iff the bilinear forms have this property. Now it \
can be proved that two complex skew symmetric operators are symplectically \
equivalent iff they have the same Jordan normal form. Therefore the \
symplectic classification is not as simple as the Euclidean. For the details \
of the theory see I. M. Jaglom ",
  ButtonBox["[Jag1]",
    ButtonData:>"Jag1",
    ButtonStyle->"Hyperlink"],
  ", A. I. Maltzev",
  ButtonBox[" [Mal]",
    ButtonData:>"Mal",
    ButtonStyle->"Hyperlink"],
  ", and ",
  ButtonBox["[OSIII]",
    ButtonData:>"OSIII",
    ButtonStyle->"Hyperlink"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell["4.1. Definitions. First Examples", "Subsection"],

Cell[CellGroupData[{

Cell["Associated bilinear forms of an endomorphism c", "Subsubsection"],

Cell[TextData[{
  "Let as before gram[] denote gram's matrix of the scalar products of the \
basis vectors. The matrix of the ",
  StyleBox["associated bilinear form ",
    FontColor->RGBColor[0, 0, 1]],
  "is"
}], "Text"],

Cell[BoxData[
    \(bilinear[c_] := \ Transpose[c] . gram[]\)], "Input",
  CellLabel->"In[34]:=",
  InitializationCell->True],

Cell[BoxData[
    \(bilinear[matrix[c]] // MatrixForm\)], "Input",
  CellLabel->"In[84]:="],

Cell["\<\
To any linear operator c  the associated bilinear form  bilinear[c] \
fulfils\
\>", "Text"],

Cell[BoxData[
    \(vec[x] . bilinear[matrix[c]] . vec[y]\  == \ 
      ssp[matrix[c] . vec[x], vec[y]]\)], "Input",
  CellLabel->"In[85]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[86]:="],

Cell[TextData[{
  "Conversely, if the matrix b of a bilinear form is given, one obtaines the \
",
  StyleBox["associated operator ",
    FontColor->RGBColor[0, 0, 1]],
  "by"
}], "Text"],

Cell[BoxData[
    \(op[b_]\  := \ Transpose[b . Inverse[gram[]]]\)], "Input",
  CellLabel->"In[35]:=",
  InitializationCell->True],

Cell["\<\
In case of symmetric bilinear forms and symplectic scalar products \
this coincides with\
\>", "Text"],

Cell[BoxData[
    \(operator[b_] := \ gram[] . b\)], "Input",
  CellLabel->"In[36]:=",
  InitializationCell->True],

Cell["since", "Text"],

Cell[BoxData[
    \(Transpose[Inverse[gram[]]] \[Equal] \ gram[]\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \(\((operator[blf[s]] // 
          MatrixForm)\) \[Equal] \ \((Simplify[op[blf[s]]] // 
          MatrixForm)\)\)], "Input",
  CellLabel->"In[90]:="],

Cell["The operator associated to a symmetric bilinear form:", "Text"],

Cell[BoxData[
    \(opa\  = operator[blf[s]]\)], "Input",
  CellLabel->"In[91]:="],

Cell[BoxData[
    \(% // MatrixForm\)], "Input",
  CellLabel->"In[92]:="],

Cell["\<\
is skew symmetric with respect to the symplectic scalar product, \
i.e. it fulfills\
\>", "Text"],

Cell[BoxData[
    \(Simplify[
      ssp[opa . vec[x], 
          vec[y]]\  \[Equal] \ \(-ssp[vec[x], opa . vec[y]]\)]\)], "Input",
  CellLabel->"In[93]:="],

Cell["Equivalently, we obtain the null matrix by", "Text"],

Cell[BoxData[
    \(Transpose[opa] . gram[]\  + \ gram[] . opa\)], "Input",
  CellLabel->"In[94]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
The general matrix of a skew symmetric endomorphism of the \
4-dimensional symplectic vector space\
\>", "Subsubsection"],

Cell["\<\
The general skew symmetric operator (for dim = 4) depends on 10 \
parameters. We organize them in 3 lists, one of length 4, the others of \
length 3, corresponding to the 2x2-blocks of  the general skew matrix opa \
above:\
\>", "Text"],

Cell[BoxData[
    \(Clear[
      a]; \ {{matrix[a, 2] // MatrixForm, {{b[1], b[2]}, {b[2], b[3]}} // 
            MatrixForm}, {{{c[1], c[2]}, {c[2], c[3]}} // 
            MatrixForm, \(-Transpose[matrix[a, 2]]\) // MatrixForm}} // 
      MatrixForm\)], "Input",
  CellLabel->"In[95]:="],

Cell["This leads to the definition", "Text"],

Cell[BoxData[
    \(Clear[a, b, c]; \ 
    skewop[a_, b_, 
        c_] := \ {{a[\([1]\)], a[\([2]\)], b[\([1]\)], 
          b[\([2]\)]}, {a[\([3]\)], a[\([4]\)], b[\([2]\)], 
          b[\([3]\)]}, \[IndentingNewLine]{c[\([1]\)], 
          c[\([2]\)], \(-a[\([1]\)]\), \(-a[\([3]\)]\)}, {c[\([2]\)], 
          c[\([3]\)], \(-a[\([2]\)]\), \(-a[\([4]\)]\)}}\)], "Input",
  CellLabel->"In[37]:=",
  InitializationCell->True],

Cell["Set", "Text"],

Cell[BoxData[
    \(abc\  = skewop[vec[a], vec[b, 3], \ vec[c, 3]]\)], "Input",
  CellLabel->"In[97]:=",
  CellTags->"skewop"],

Cell[BoxData[
    \(% // MatrixForm\)], "Input",
  CellLabel->"In[98]:="],

Cell["\<\
Obviously, this is a general skew symmetric operator  in the \
4-space:\
\>", "Text"],

Cell[BoxData[
    \(Transpose[abc] . gram[]\  + gram[] . abc\)], "Input",
  CellLabel->"In[99]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Random skew symmetric operators", "Subsubsection"],

Cell["\<\
Using the definition skewop of a general skew symmetric operator \
now we define a random skew symmetric matrix of order 4:\
\>", "Text"],

Cell[BoxData[
    \(randomabc := \ skewop[randomv[], randomv[3], randomv[3]]\)], "Input",
  CellLabel->"In[38]:=",
  InitializationCell->True],

Cell[BoxData[
    \(randomabc // MatrixForm\)], "Input",
  CellLabel->"In[101]:="],

Cell[BoxData[
    \(Eigenvalues[%]\)], "Input",
  CellLabel->"In[102]:="],

Cell[BoxData[
    \(Chop[Table[Eigenvalues[randomabc], \ {10}]]\)], "Input",
  CellLabel->"In[103]:="],

Cell["\<\
In the following we want to solve the question wether there exist  \
Eigenvalues being neither real nor purely imaginary. To this aim we need the \
package\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<Declare`\>"]\)], "Input",
  CellLabel->"In[86]:=",
  InitializationCell->True],

Cell[BoxData[
    \(Names["\<Declare`*\>"]\)], "Input",
  CellLabel->"In[87]:="],

Cell[BoxData[
    \(\(?RealQ\)\)], "Input",
  CellLabel->"In[94]:="],

Cell[BoxData[
    \(Context[RealQ]\)], "Input",
  CellLabel->"In[92]:="],

Cell[BoxData[
    \(\(?Declare\)\)], "Input",
  CellLabel->"In[95]:="],

Cell[BoxData[
    \(\(?NewDeclare\)\)], "Input",
  CellLabel->"In[96]:="],

Cell[BoxData[
    \(?? Positive\)], "Input",
  CellLabel->"In[100]:="],

Cell["\<\
The properties of some system functions like \"Positive\" are \
changed resp. completed if the Cobtext \"Declare`\" is loaded.\
\>", "Text"],

Cell[BoxData[
    \(Context["\<Positive\>"]\)], "Input",
  CellLabel->"In[101]:="],

Cell[BoxData[
    \(Chop[Eigensystem[randomabc]]\)], "Input",
  CellLabel->"In[103]:="],

Cell["\<\
Question: Do there exist real skew symmetric matrices whose \
Eigenvalues are neither real nor purely imaginary?
The answer ist \"Yes\"; the probability that randomabc is such a matrix, is \
very small, nearly 68/10000: \
\>", "Text"],

Cell[BoxData[
    \(Clear[lj]; 
    For[lj = {}; i = 1, 
      i < 10000, \(i++\), \((\ rrm[i]\  = randomabc; \ 
        vvm[i] = 
          Chop[Eigenvalues[rrm[i]]]; \ \[IndentingNewLine] (*\ \(Print[
              vvm[i]];\)\ *) \[IndentingNewLine]If[\((Not[
                  RealQ[\(vvm[i]\)[\([1]\)]]] && 
                Not[RealQ[I*\(vvm[i]\)[\([1]\)]]])\) && \((Not[
                  RealQ[\(vvm[i]\)[\([3]\)]]] && 
                Not[RealQ[I*\(vvm[i]\)[\([3]\)]]])\), 
          Print[vvm[i]]; \[IndentingNewLine]AppendTo[lj, i]])\)]; 
    Print[lj]\)], "Input",
  CellLabel->"In[110]:="],

Cell[BoxData[
    \(Length[lj]\)], "Input",
  CellLabel->"In[111]:="],

Cell["\<\
We will find  an example where only one of the Eigenvalues pairs \
would be not real; we evaluate the next cells several times.\
\>", "Text"],

Cell[BoxData[
    \(Clear[lj, rrm, vvm, rabc]\)], "Input",
  CellLabel->"In[104]:="],

Cell[BoxData[
    \(For[lj = {}; i = 1, 
      i < 100, \(i++\), \((\ rrm[i]\  = randomabc; \ 
        vvm[i] = Chop[Eigenvalues[rrm[i]]]; \ 
        If[\((RealQ[\(vvm[i]\)[\([1]\)]] && 
                Not[RealQ[\(vvm[i]\)[\([3]\)]]])\) || \ \((RealQ[\(vvm[
                      i]\)[\([3]\)]] && Not[RealQ[\(vvm[i]\)[\([1]\)]]])\), 
          Print[vvm[i]]; AppendTo[lj, i]])\)]; Print[lj]; 
    Length[lj]\)], "Input",
  CellLabel->"In[107]:="],

Cell["\<\
It results: In nearly 27% of the cases have the required \
property.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Two purely imaginary Eigenvalues \[ImaginaryI]\[Alpha] \
\[ImaginaryI]a, \[ImaginaryI]b\
\>", "Subsubsection"],

Cell["\<\
Using randomabc we did not find any example where all the \
Eigenvalues would be purely imaginary; we evaluated the next cells several \
times.\
\>", "Text"],

Cell[BoxData[
    \(Clear[lj, rrm, vvm, rabc]\)], "Input",
  CellLabel->"In[183]:="],

Cell[BoxData[
    \(For[lj = {}; i = 1, 
      i < 100000, \(i++\), \((\ rrm[i]\  = randomabc; \ 
        vvm[i] = Chop[Eigenvalues[rrm[i]]]; \ 
        If[RealQ[I*\(vvm[i]\)[\([1]\)]] && RealQ[I*\(vvm[i]\)[\([3]\)]], 
          Print[vvm[i]]; AppendTo[lj, i]])\)]; Print[lj]\)], "Input",
  CellLabel->"In[184]:="],

Cell["\<\
Of course, over the complex numbers there exist such \
examples:\
\>", "Text"],

Cell[BoxData[
    \(pi\  = \ diag[I, I, \(-I\), \(-I\)]\)], "Input",
  CellLabel->"In[171]:="],

Cell[BoxData[
    \(ssp[pi . vec[x], vec[y]] + ssp[vec[x], pi . vec[y]]\)], "Input",
  CellLabel->"In[172]:="],

Cell["\<\
The corresponding symmetric bilinear form has complex coefficients:\
\
\>", "Text"],

Cell[BoxData[
    \(bilinear[pi] // MatrixForm\)], "Input",
  CellLabel->"In[173]:="],

Cell["More generally", "Text"],

Cell[BoxData[
    \(bilinear[diag[c1, c2, \(-c1\), \(-c2\)]] // MatrixForm\)], "Input",
  CellLabel->"In[174]:="],

Cell["\<\
REAL  SKEW SYMMETRIC OPERATORS WITH F0UR  PURELY IMMAGINARY \
EIGENVALUES EXIST:
Indeed, consider, for a,b real: \
\>", "Text"],

Cell[BoxData[
    \(rj1imop[a_, 
        b_]\  := \ {{0, 0, a, 0}, {0, 0, 0, b}, {\(-a\), 0, 0, 
          0}, {0, \(-b\), 0, 0}}\)], "Input",
  CellLabel->"In[205]:="],

Cell[BoxData[
    \(rj1imop[a, b] // MatrixForm\)], "Input",
  CellLabel->"In[206]:="],

Cell[BoxData[
    \(Eigensystem[%]\)], "Input",
  CellLabel->"In[207]:="],

Cell[BoxData[
    \(operator[diag[a, b, c, d]]\)], "Input",
  CellLabel->"In[208]:="],

Cell[BoxData[
    \(operator[diag[a, b, c, d]] // MatrixForm\)], "Input",
  CellLabel->"In[209]:="],

Cell[BoxData[
    \(Eigensystem[%%]\)], "Input",
  CellLabel->"In[210]:="],

Cell["\<\
Two purely imaginary Eigenvalues appear as specialiation sign(a )= \
sign(c), sign(b)=sign(d) of the general diagonal case; therefore the \
probability of this event is 0 if one considers random skew symmetric \
operators.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Tori", "Subsubsection"],

Cell["\<\
The non-degenerate quadrics whose associated operators have two \
purely imaginary Eigenvalue pairs are tori:\
\>", "Text"],

Cell[BoxData[
    \(bt[a_, b_]\  := bilinear[rj1imop[a, b]]\)], "Input",
  CellLabel->"In[191]:="],

Cell[BoxData[
    \(Simplify[vec[x] . bt[a, b] . vec[x]]\)], "Input",
  CellLabel->"In[192]:="],

Cell["Obviously, there exist real solutions of ", "Text"],

Cell[BoxData[
    \(%\  == \ 0\)], "Input",
  CellLabel->"In[193]:="],

Cell[TextData[{
  "iff a/b < 0 (non degenerate case) or a = 0 or b = 0 (degenerate cases). In \
the non degerate case the corresponding quadric is a torus: Without loss of \
generality we may assume a = 1, b  = -",
  Cell[BoxData[
      \(TraditionalForm\`c\^2\)]],
  ". The solution has the parameter representation"
}], "Text"],

Cell[BoxData[
    \(tor[s_, t_, c_]\  := \ {c*Cos[t], \ Cos[s], \ c*\ Sin[t], \ 
        Sin[s]}\)], "Input",
  CellLabel->"In[194]:="],

Cell["\<\
with 0 \[LessEqual] s, t < Pi, and this is a torus in the spherical \
image  \
\>", "Text"],

Cell[BoxData[
    \(tor[s, t, c] . bt[1, \(-c^2\)] . tor[s, t, c]\)], "Input",
  CellLabel->"In[195]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[196]:="],

Cell[BoxData[
    \(\(?sterproj\)\)], "Input",
  CellLabel->"In[197]:="],

Cell[BoxData[
    \(tor[s, t, c] . tor[s, t, c]\)], "Input",
  CellLabel->"In[198]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[199]:="],

Cell[BoxData[
    \(torus[s_, t_, c_]\  := \ tor[s, t, c]/Sqrt[1 + c^2]\)], "Input",
  CellLabel->"In[200]:=",
  CellTags->"torus"],

Cell[BoxData[
    \(Simplify[torus[s, t, c] . torus[s, t, c]]\)], "Input",
  CellLabel->"In[201]:="],

Cell[BoxData[
    \(ParametricPlot3D[
      sterproj[torus[s, t, 0.5]], {s, 0, 2*Pi}, {t, 0, 2*Pi}, \ 
      PlotRange\  \[Rule] \ All, PlotPoints\  \[Rule] \ 60]\)], "Input",
  CellLabel->"In[202]:="],

Cell[BoxData[
    \(tor1\  = \ %\)], "Input",
  CellLabel->"In[203]:="],

Cell["\<\
The projective torus is twofold covered by the spherical, since \
\
\>", "Text"],

Cell[BoxData[
    \(torus[s, t, c]\  + \ torus[s + Pi, t + Pi, c]\)], "Input",
  CellLabel->"In[204]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["A more general example", "Subsubsection"],

Cell["\<\
Non-degenerate quadrics whose associated operators have two purely \
imaginary Eigenvalue pairs are tori:\
\>", "Text"],

Cell[BoxData[
    \(genbt[a_, b_, c_, d_]\  := 
      bilinear[operator[diag[a, b, c, d]]]\)], "Input",
  CellLabel->"In[211]:="],

Cell[BoxData[
    \(Simplify[vec[x] . genbt[a, b, c, d] . vec[x]]\)], "Input",
  CellLabel->"In[212]:="],

Cell["Obviously, there exist real solutions of ", "Text"],

Cell[BoxData[
    \(%\  == \ 0\)], "Input",
  CellLabel->"In[213]:="],

Cell["\<\
iff at least some of the coefficients have distinct signs. In the \
non degerate case the corresponding quadric is a torus: Without loss of \
generality we may assume a = 1, c>0, b<0, d<0,.e.g.  \
\>", "Text"],

Cell[BoxData[
    \(Simplify[vec[x] . genbt[1, \(-1\), 2, \(-1\)] . vec[x]]\)], "Input",
  CellLabel->"In[214]:="],

Cell[BoxData[
    \(Solve[% \[Equal] 0, vec[x]]\)], "Input",
  CellLabel->"In[215]:="],

Cell["The solution has the parameter representation", "Text"],

Cell[BoxData[
    \(tora[s_, t_]\  := \ {Cos[t], \ Cos[s], \ Sin[t]/Sqrt[2], \ 
        Sin[s]}\)], "Input",
  CellLabel->"In[216]:="],

Cell["\<\
with 0 \[LessEqual] s, t < Pi, and this is a torus in the spherical \
image  \
\>", "Text"],

Cell[BoxData[
    \(tora[s, t] . genbt[1, \(-1\), 2, \(-1\)] . tora[s, t]\)], "Input",
  CellLabel->"In[217]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[218]:="],

Cell[BoxData[
    \(\(?sterproj\)\)], "Input",
  CellLabel->"In[219]:="],

Cell[BoxData[
    \(tora[s, t] . tora[s, t]\)], "Input",
  CellLabel->"In[221]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[222]:="],

Cell[BoxData[
    \(toru[s_, t_]\  := \ tora[s, t]/Sqrt[\((7 + Cos[2*t])\)/4]\)], "Input",
  CellLabel->"In[223]:=",
  CellTags->"torus"],

Cell[BoxData[
    \(Simplify[toru[s, t] . toru[s, t]]\)], "Input",
  CellLabel->"In[224]:="],

Cell[BoxData[
    \(ParametricPlot3D[sterproj[toru[s, t]], {s, 0, 2*Pi}, {t, 0, 2*Pi}, \ 
      PlotRange\  \[Rule] \ All, PlotPoints\  \[Rule] \ 60]\)], "Input",
  CellLabel->"In[225]:="],

Cell[BoxData[
    \(tor1\  = \ %\)], "Input",
  CellLabel->"In[226]:="],

Cell["\<\
The projective torus is twofold covered by the spherical, since \
\
\>", "Text"],

Cell[BoxData[
    \(toru[s, t]\  + \ toru[s + Pi, t + Pi]\)], "Input",
  CellLabel->"In[228]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[229]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Proof of the equivariance of b \[RightTeeArrow] op[b]\
\>", \
"Subsubsection"],

Cell[TextData[{
  "In this subsection we want to show that the correspondence b \
\[RightTeeArrow] op[b] is equivariant: op(gb) = gop(b) for any symplectic \
transformation g.  This is a nice example for symbolic calculations with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Simply evaluate this subsection.\nThe Matrix of the bilinear form g b is \
"
}], "Text"],

Cell[BoxData[
    \(F[g_, b_]\  := \ Transpose[Inverse[g]] . b . Inverse[g]\)], "Input",
  CellLabel->"In[230]:="],

Cell["\<\
We may assume that g is a symplectic transvection, since these \
generate the symplectic group. We calculate\
\>", "Text"],

Cell[BoxData[
    \(F[transv[vec[tr], cr], \ blf[b]]\)], "Input",
  CellLabel->"In[231]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[232]:="],

Cell[BoxData[
    \(\(trgb\  = \ %;\)\)], "Input",
  CellLabel->"In[233]:="],

Cell["The operator gop[p] has the matrix", "Text"],

Cell[BoxData[
    \(mgopb\  = \ 
      transv[vec[tr], cr] . op[blf[b]] . 
        Inverse[transv[vec[tr], cr]]\)], "Input",
  CellLabel->"In[234]:="],

Cell[BoxData[
    \(%\  \[Equal] \ op[trgb]\)], "Input",
  CellLabel->"In[235]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[236]:="],

Cell[BoxData[
    \(Clear[F, tr, cr, b]\)], "Input",
  CellLabel->"In[237]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
4.2. Bilinear forms with diagonizable skew symmetric operators and \
the corresponding quadrics\
\>", "Subsection"],

Cell[TextData[{
  "In section 2.8.5  of ",
  ButtonBox["[OSIII]",
    ButtonData:>"OSIII",
    ButtonStyle->"Hyperlink"],
  "  skew symmetric endomorphismss and the corresponding associated bilinear \
forms of the 4-dimensional complex and real symplectic vector spaces are \
classified; normal forms of their matrices with respect to symplectic bases \
are given.  In this and the following subsection we discuss these forms and \
apply them to the classification of the corresponding real quadrics (see \
[OSIII], Examples 2.8.8, 2.8.9)."
}], "Text"],

Cell[CellGroupData[{

Cell["Symplectically diagonal bilinear forms", "Subsubsection"],

Cell[TextData[{
  "In this subsection we consider bilinear forms for which exists a \
symplectic basis in which its matrix is diagonal; we call them ",
  StyleBox["symplectically diagonal",
    FontColor->RGBColor[0, 0, 1]],
  ". We show that their associated operators are diagonizable and calculate \
the Eigenvalues of these operators in terms of the coefficients of the \
symplectically bilinear form.  The results yield a criterion for the \
symplectic equivalence of two  symplectically diagonazal bilinear forms. \n\
For shortness, we write for a diagonal matrix"
}], "Text"],

Cell[BoxData[
    \(diag[a1_, a2_, a3_, a4_]\  := \ 
      DiagonalMatrix[{a1, a2, a3, a4}]\)], "Input",
  CellLabel->"In[39]:=",
  InitializationCell->True],

Cell["\<\
We interpret diag as the matrix of a symmetric bilinear form and \
calculate the matrix of the  associated operator\
\>", "Text"],

Cell[BoxData[
    \(opd\  = operator[diag[a1, a2, a3, a4]]\)], "Input",
  CellLabel->"In[112]:="],

Cell[BoxData[
    \(% // MatrixForm\)], "Input",
  CellLabel->"In[113]:="],

Cell[BoxData[
    \(Eigensystem[opd]\)], "Input",
  CellLabel->"In[129]:="],

Cell[BoxData[
    \(ebas\  = %[\([2]\)]\)], "Input",
  CellLabel->"In[130]:="],

Cell[BoxData[
    \(Table[ssp[ebas[\([i]\)], \ ebas[\([j]\)]], \ {i, 4}, {j, 4}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[131]:="],

Cell["We define", "Text"],

Cell[BoxData[
    \(eebas\  = \ ebas; \ 
    eebas[\([2]\)]\  = \ 
      ebas[\([2]\)]/\((\(2\ \[ImaginaryI]\ \@a3\)\/\@a1)\); \ 
    eebas[\([4]\)]\  = \ 
      ebas[\([4]\)]/\((\(2\ \[ImaginaryI]\ \@a4\)\/\@a2)\);\)], "Input",
  CellLabel->"In[132]:="],

Cell[BoxData[
    \(eebas // MatrixForm\)], "Input",
  CellLabel->"In[133]:="],

Cell[BoxData[
    \(Table[ssp[eebas[\([i]\)], \ eebas[\([j]\)]], \ {i, 4}, {j, 4}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[134]:="],

Cell[BoxData[
    \(opd . Transpose[eebas] // MatrixForm\)], "Input",
  CellLabel->"In[136]:="],

Cell["\<\
It follows:
In a complex symplectic vector space the associated operator of a \
symplectically diagonal bilinear form is diagonizable; there exist symplectic \
orthogonal (or symplectic) bases of Eigenvectors of the operator. For real \
symplectically diagonal forms the Eigenvalues of the associated operators are \
either real or purely imaginary complex numbers. \
\>", "Text"],

Cell[BoxData[
    \(eebass\  = \ symplectic[eebas]\)], "Input",
  CellLabel->"In[120]:="],

Cell[BoxData[
    \(Table[ssp[eebass[\([i]\)], \ eebass[\([j]\)]], \ {i, 4}, {j, 4}] // 
      MatrixForm\)], "Input",
  CellLabel->"In[137]:="],

Cell[BoxData[
    \(opd . Transpose[eebass] // MatrixForm\)], "Input",
  CellLabel->"In[138]:="],

Cell[BoxData[
    \(JordanDecomposition[operator[diag[a1, a2, a3, a4]]]\)], "Input",
  CellLabel->"In[139]:="],

Cell[BoxData[
    \(js\  = \ %[\([1]\)]\)], "Input",
  CellLabel->"In[140]:="],

Cell[BoxData[
    \(%%[\([2]\)] // MatrixForm\)], "Input",
  CellLabel->"In[141]:="],

Cell[BoxData[
    \(Inverse[js] . opd . js\)], "Input",
  CellLabel->"In[142]:="],

Cell[BoxData[
    \(Transpose[js] . gram[] . js\)], "Input",
  CellLabel->"In[143]:="],

Cell[BoxData[
    \(Det[js]\)], "Input",
  CellLabel->"In[144]:="],

Cell["Obviously, js is not symplectic (it needs not to be).", "Text"],

Cell[TextData[{
  "It follows: \n",
  StyleBox["Corollary",
    FontColor->RGBColor[0, 0, 1]],
  ": Two non degenerate symplectically diagonal bilinear forms \
diag[a1,a2,a3,a4], diag[b1,b2,b3,b4] are symplectically equivalent iff their \
diagonal elements aj, bk under appropriate  numeration fulfil a1a3 = b1b3 and \
a2a4 = b2b4.\nExercise: Give a direct proof for this statement."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Example: The isotropic sphere", "Subsubsection"],

Cell[BoxData[
    \(diag[1, 1, 1, 1] // MatrixForm\)], "Input",
  CellLabel->"In[145]:="],

Cell[BoxData[
    \(op11\  = operator[diag[1, 1, 1, 1]]\)], "Input",
  CellLabel->"In[146]:="],

Cell[BoxData[
    \(op11 // MatrixForm\)], "Input",
  CellLabel->"In[147]:="],

Cell[BoxData[
    \(Eigensystem[op11]\)], "Input",
  CellLabel->"In[148]:="],

Cell[BoxData[
    \(JordanDecomposition[op11]\)], "Input",
  CellLabel->"In[149]:="],

Cell[BoxData[
    \(%[\([2]\)] // MatrixForm\)], "Input",
  CellLabel->"In[150]:="],

Cell[BoxData[
    \(ssp[op11 . vec[x], vec[x]] \[Equal] \ 0\)], "Input",
  CellLabel->"In[151]:="],

Cell["\<\
 This is the equation of the isotropic (or imaginary) sphere; its \
only real solution is the nopoint (the zero vector).\
\>", "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.3. Complex classification", "Subsection"],

Cell["\<\
We start with [OSIII], Examples 2.8.8; the hints (Case xxx) give \
the corresponding cases discussed there.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Diagonizable operators, (Cases A.1, B.1)", "Subsubsection"],

Cell[TextData[{
  "We consider the cases A.1, B.1 of Example 2.5.8. If a skew symmetric \
operator is diagonazable it must have the normal form (see  ",
  ButtonBox["skewop",
    ButtonData:>"skewop",
    ButtonStyle->"Hyperlink"],
  " )."
}], "Text"],

Cell[BoxData[
    \(nfdiag[x_, y_]\  := \ diag[x, y, \(-x\), \(-y\)]\)], "Input",
  CellLabel->"In[152]:=",
  InitializationCell->True],

Cell[BoxData[
    \(nfdiag[x, y]\)], "Input",
  CellLabel->"In[153]:="],

Cell["The Eigensystem is", "Text"],

Cell[BoxData[
    \(Eigensystem[nfdiag[x, y]]\)], "Input",
  CellLabel->"In[154]:="],

Cell["\<\
The Eigenvectors are the vertices of the underlying symplectic \
frame. The corresponding bilinear form has the matrix\
\>", "Text"],

Cell[BoxData[
    \(bnf4[a_, b_]\  := bilinear[nfdiag[a, b]]\)], "Input",
  CellLabel->"In[155]:=",
  InitializationCell->True],

Cell[BoxData[
    \(bnf4[a, b] // MatrixForm\)], "Input",
  CellLabel->"In[156]:="],

Cell["\<\
The equation of the corresponding quadric in homogeneous \
coordinates with respect to a symplectic basis is\
\>", "Text"],

Cell[BoxData[
    \(Simplify[vec[x] . bnf4[a, b] . vec[x]\  \[Equal] \ 0]\)], "Input",
  CellLabel->"In[157]:=",
  CellTags->"eq4.2.1"],

Cell["\<\
One easily finds a symplectic transformation showing that this \
quadric is symplectically diagonal. If the bilinear form is not zero, we may \
assume b\[NotEqual] 0 and norm the equation of the quadric such that b= 1: \
the quadric   depends on one complex parameter c = a/b:  \
\>", "Text"],

Cell[BoxData[
    \(q4diag[c_]\  := c*\ x[1]\ x[3] + x[2]\ x[4] == 0\)], "Input",
  CellLabel->"In[158]:=",
  InitializationCell->True],

Cell[BoxData[
    \(q4diag[c]\)], "Input",
  CellLabel->"In[159]:="],

Cell["\<\
In the degenerate case c = 0 the  solution is the union of the \
planes x[2] = 0 and x[4] = 0 which intersect in the symplectic line stb[1]\
\[Or]stb[3]. For c\[NotEqual] 0 the solution containes the vertices and the \
isotropic edges of the basis tetraeder, but not its symplectic edges.\
\>", \
"Text",
  CellTags->"diag"]
}, Closed]],

Cell[CellGroupData[{

Cell["Non diagonizable Operators, (Cases A.2, B.4)", "Subsubsection"],

Cell["The canonical matrix shape of  an operator of such type is", "Text"],

Cell[BoxData[
    \(mqj2[
        a_]\  := \ {{a, 0, 0, 0}, {1, a, 0, 0}, {0, 0, \(-a\), \(-1\)}, {0, 
          0, 0, \(-a\)}}\)], "Input",
  CellLabel->"In[160]:=",
  InitializationCell->True],

Cell[BoxData[
    \(mqj2[a] // MatrixForm\)], "Input",
  CellLabel->"In[161]:="],

Cell["It degenerates for a = 0:", "Text"],

Cell[BoxData[
    \(Det[mqj2[a]]\)], "Input",
  CellLabel->"In[162]:="],

Cell[BoxData[
    \(Eigensystem[mqj2[a]]\)], "Input",
  CellLabel->"In[163]:="],

Cell[BoxData[
    \(bqj2[a_]\  := \ bilinear[mqj2[a]]\)], "Input",
  CellLabel->"In[164]:=",
  InitializationCell->True],

Cell[BoxData[
    \(bqj2[a] // MatrixForm\)], "Input",
  CellLabel->"In[165]:="],

Cell[BoxData[
    \(Simplify[vec[x] . bqj2[a] . vec[x]] \[Equal] \ 0\)], "Input",
  CellLabel->"In[166]:=",
  InitializationCell->True,
  CellTags->"j2.1"],

Cell["\<\
The solution in the non degenerate case a \[NotEqual] 0 contains \
the vertices but not all the isotropic edges of the basis tetraeder; for the \
point of the edge stb[1]\[Or]stb[4] : {t,0,0,s}  the quadratic form evaluates \
to 2*s*t \[NotEqual] 0. 
In the degenerate case a = 0 the solution is the union of the planes x[1] = \
0, x[4] = 0 intersecting in the isotropic line stb[2]\[Or]stb[3]. \
\>", \
"Text"],

Cell["\<\
Remark. In [OSIII], under example 8, B.4 we considered the \
following matrix:\
\>", "Text"],

Cell[BoxData[
    \(mj20\  = \ {null[], stb[4], stb[1], \ null[]}\)], "Input",
  CellLabel->"In[167]:="],

Cell[BoxData[
    \(bilinear[mj20] // MatrixForm\)], "Input",
  CellLabel->"In[168]:="],

Cell[BoxData[
    \(vec[x] . % . vec[x] \[Equal] 0\)], "Input",
  CellLabel->"In[169]:="],

Cell["\<\
Of course, the result is essentially the same: a pair of planes \
intersecting in the  isotropic edge  stb[2]\[Or]stb[3].\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
The operator is not diagonizable and has an Eigenvalue a  of \
multiplicity 2 (Cases B.2, B.3)\
\>", "Subsubsection"],

Cell["\<\
 In this case the canonical form of the operator  can be written as\
\
\>", "Text"],

Cell[BoxData[
    \(opj20[a_]\  := \ {a*stb[1], null[], \(-a\)*stb[3], \ stb[2]}\)], "Input",\

  CellLabel->"In[170]:="],

Cell[BoxData[
    \(opj20[a] // MatrixForm\)], "Input",
  CellLabel->"In[171]:="],

Cell[BoxData[
    \(JordanDecomposition[opj20[a]]\)], "Input",
  CellLabel->"In[172]:="],

Cell[BoxData[
    \(%[\([2]\)] // MatrixForm\)], "Input",
  CellLabel->"In[173]:="],

Cell[BoxData[
    \(Eigenvalues[opj20[a]]\)], "Input",
  CellLabel->"In[174]:="],

Cell[BoxData[
    \(bilinear[opj20[a]] // MatrixForm\)], "Input",
  CellLabel->"In[175]:="],

Cell[BoxData[
    \(vec[x] . bilinear[opj20[a]] . vec[x]\  \[Equal] \ 0\)], "Input",
  CellLabel->"In[176]:=",
  CellTags->"opj2"],

Cell["\<\
In case a \[NotEqual]  0 the solution is a cone (cf. [OSIII], \
example 1.9.4) with vertex at the origin [stb[4]] consisting of all lines \
joining the origin with the ellipse in the infinite plane x[4] = 0 defined by \
the last equation. For a = 0 the solution is the double counting plane x[2] = \
0.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
The Jordan normal form of the operator is a single Jordan cell of \
order 4 with Eigenvalue 0 (Case B.5)\
\>", "Subsubsection",
  CellTags->"B5"],

Cell["\<\
A skew symmetric operator having the required canonical form \
is\
\>", "Text"],

Cell[BoxData[
    \(qj40\  = \ {stb[2], \ null[], \ \(-stb[1]\), \ \(-stb[3]\)}\)], "Input",\

  CellLabel->"In[265]:="],

Cell[BoxData[
    \(% // MatrixForm\)], "Input",
  CellLabel->"In[266]:="],

Cell[BoxData[
    \(JordanDecomposition[qj40]\)], "Input",
  CellLabel->"In[267]:="],

Cell[BoxData[
    \(%[\([2]\)] // MatrixForm\)], "Input",
  CellLabel->"In[268]:="],

Cell[BoxData[
    \(Solve[qj40 . vec[x] \[Equal] \ null[], \ vec[x]]\)], "Input",
  CellLabel->"In[269]:="],

Cell[BoxData[
    \(bilinear[qj40] // MatrixForm\)], "Input",
  CellLabel->"In[270]:="],

Cell[BoxData[
    \(vec[x] . % . vec[x] \[Equal] \ 0\)], "Input",
  CellLabel->"In[271]:="],

Cell[TextData[{
  "From the affine point of view, the quadric is equivalent to those of the \
foregoing subsection, see ",
  ButtonBox["opj2",
    ButtonData:>"opj2",
    ButtonStyle->"Hyperlink"],
  " , especially for a = -1.  Symplectically, the quadrics differ by their \
position within the canonical frame. In each of the cases there is exactly \
one face of the basic tetraeder containing two generators of the cone:\ncase \
B.4, a = -1: x[2] = 0 contains the x[1]- and the x[3]-axes, both are \
isotropic generators of the cone.\nCase B.5, : x[1] = 0 contains the x[2]- \
and the x[3]-axes, the first is a symplectic, the second an isotropic \
generator. Therefore the quadric being affinely equivalent ( indeed,   even \
euclidean, if the coordinates are interpreted as euclidean coordinates), are \
not symplectically equivalent."
}], "Text"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
5. Real classification of symmetric bilinear forms and the \
corresponding quadrics\
\>", "Section"],

Cell[CellGroupData[{

Cell["\<\
5.1. Real classification of bilinear forms with diagonizable skew \
symmetric operators\
\>", "Subsection"],

Cell[TextData[{
  "We remember that with a also  -a is an Eigenvalue of a skew symmetric \
operator, and discuss the real symmetric bilinear forms following the \
considerations in example 9 of section  2.8 in [OSIII]; the hints (Case xxx) \
give the corresponding cases discussed there. Clearly,  such real bilinear \
forms with complex Eigenvalues exist (see the next subsections). For real \
operators complex Eigenvalues always appear in pairs, together with their \
conjugates. Thus we have to distinguish the cases:\n1. a, b real Eigenvalues\n\
2. x=a  real, y = I*b purely imaginary\n3. x = I*a, y = I*b  all Eigenvalues \
purely imaginary,\n4. The Eigenvalues are  a, ",
  Cell[BoxData[
      \(TraditionalForm\`a\&_\)]],
  ", -a, -",
  Cell[BoxData[
      \(TraditionalForm\`a\&_\)]],
  ", a neither real nor purely imaginary Eigenvalue."
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
Case 1:Diagonizable operators with  real Eigenvalues a,b \
\[NotEqual] 0 (Cases A.1, B.1)\
\>", "Subsubsection"],

Cell["\<\
Since the Eigenvalues appear in pairs a, -a we may assume a \
\[GreaterEqual]  0, b>0. The normal form of case 1 is a special case of  the \
complex case. Consider the equation  \
\>", "Text"],

Cell[BoxData[
    \(q4diag[c]\)], "Input",
  CellLabel->"In[186]:="],

Cell["\<\
for real c. In case c\[NotEqual] 0 the corresponding bilinear form \
has index 2 and the quadric is a torus, see example 1.9.2 in [OSIII].  
We set x = x[1], y= x[3],  z = x[2]  and  obtain for  the affine points  with \
x[4] = 1 of the solution the hyperbolic paraboloid defined as the graph of \
the function\
\>", "Text"],

Cell[BoxData[
    \(q4c1[x_, y_, c_]\  := \ \(-c\)*x*y\)], "Input",
  CellLabel->"In[187]:="],

Cell[BoxData[
    \(Plot3D[q4c1[x, y, 2. ], {x, \(-1\), 1}, \ {y, \(-1\), 1}, \ 
      BoxRatios\  \[Rule] \ {1, 1, 2}]\)], "Input",
  CellLabel->"In[188]:="],

Cell[BoxData[
    \(For[i\  = \ 1, \ i < 10, \(i++\)\ , 
      Plot3D[q4c1[x, y,  .5\  + \ i/10], {x, \(-1\), 1}, \ {y, \(-1\), 1}, \ 
        BoxRatios\  \[Rule] \ {1, 1,  .5\  + \ i/10}]]\)], "Input",
  CellLabel->"In[189]:="],

Cell[TextData[{
  "The parameter c>0  defines the shape of the surface with respect  to the \
symplectic frame, defined by the Eigenvectors of the associated skew \
symmetric operator.  For the degenerate case c = 0 and the position with \
respect to the symplectic basis tetraeder again the statement ",
  ButtonBox["diag",
    ButtonData:>"diag",
    ButtonStyle->"Hyperlink"],
  " is valid."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Case 2: Diagonizable operators with Eigenvalues a\[ImaginaryI], b, \
a, b real (Case D.1.2)\
\>", "Subsubsection"],

Cell["\<\
 In  case 2. we may assume   a\[NotEqual]0. We introduce the matrix \
\
\>", "Text"],

Cell[BoxData[
    \(m4x2[a_, 
        b_]\  := \ {{0, 0, a, 0}, {0, b, 0, 0}, {\(-a\), 0, 0, 0}, {0, 0, 
          0, \(-b\)}}\)], "Input",
  CellLabel->"In[190]:="],

Cell[BoxData[
    \(m4x2[a, b] // MatrixForm\)], "Input",
  CellLabel->"In[191]:="],

Cell[BoxData[
    \(Eigensystem[m4x2[a, b]]\)], "Input",
  CellLabel->"In[192]:="],

Cell[BoxData[
    \(bilinear[m4x2[a, b]] // MatrixForm\)], "Input",
  CellLabel->"In[193]:="],

Cell[BoxData[
    \(vec[x] . % . vec[x]\  \[Equal] \ 0\)], "Input",
  CellLabel->"In[194]:="],

Cell["\<\
Obviously, the sign Sign[a] defines  the index of the bilinear \
form, which is an invariant; it equals  1  in case a>0, and 3 in case a<0.  \
This is an example for a pair of non symplectically equivalent real bilinear \
forms which are complex symplectically equivalent.  To find the corresponding \
quadric we may assume a > 0, multiplying the last equation by -1, if \
necessary.  In the non-degenerate case we norm the equation such that  b = \
\[PlusMinus]1.  It is a projective ellipsoid, appearing in the affine modell \
as the elliptical paraboloid (x[4] = 1), \
\>", "Text"],

Cell[BoxData[
    \(2*x[2]\  \[Equal] \ \(-a\)*\((x[1]^2\  + x[3]^2)\)\)], "Input",
  CellLabel->"In[195]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Case 3: Diagonizable operators with  Eigenvalues a\[ImaginaryI], b\
\[ImaginaryI], a,b \[NotEqual] 0 (Case D.1.1)\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(\(m4x3[a_, 
        b_]\)\(:=\)\({{0, 0, a, 0}, {0, 0, 0, b}, {\(-a\), 0, 0, 
          0}, {0, \(-b\), 0, 0}}\)\(\ \)\)\)], "Input",
  CellLabel->"In[196]:="],

Cell[BoxData[
    \(m4x3[a, b] // MatrixForm\)], "Input",
  CellLabel->"In[197]:="],

Cell[BoxData[
    \(Eigensystem[m4x3[a, b]]\)], "Input",
  CellLabel->"In[198]:="],

Cell[BoxData[
    \(bilinear[m4x3[a, b]]\)], "Input",
  CellLabel->"In[199]:="],

Cell[BoxData[
    \(MatrixForm[%]\)], "Input",
  CellLabel->"In[200]:="],

Cell[BoxData[
    \(vec[x] . % . vec[x] \[Equal] \ 0\)], "Input",
  CellLabel->"In[201]:="],

Cell[TextData[{
  "The index of the corresponding real symmetric bilinear form may be 0, 2, \
or 4. In the definite cases index = 0, 4 the quadric has no real points; in \
case index = 2  the quadric has been  already discussed in section 4.1; it is \
a ",
  ButtonBox["torus",
    ButtonData:>"torus",
    ButtonStyle->"Hyperlink"]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Case 4:  Eigenvalues c, ",
  Cell[BoxData[
      \(TraditionalForm\`c\&_\)]],
  ", -c -",
  Cell[BoxData[
      \(TraditionalForm\`c\&_\)]],
  ", c neither real nor purely imaginary (Case C)"
}], "Subsubsection"],

Cell["\<\
Since under this assumption the four Eigenvalues are distinct, the \
operator must be diagonizable. Let c = a + b\[ImaginaryI], a, b \[NotEqual] \
0, real.\
\>", "Text"],

Cell[BoxData[
    \(m4x4[a_, 
        b_] := \ {{a, b, 0, 0}, {\(-b\), a, 0, 0}, {0, 0, \(-a\), b}, {0, 
          0, \(-b\), \(-a\)}}\)], "Input",
  CellLabel->"In[202]:="],

Cell[BoxData[
    \(m4x4[a, b] // MatrixForm\)], "Input",
  CellLabel->"In[203]:="],

Cell[BoxData[
    \(Eigenvalues[m4x4[a, b]]\)], "Input",
  CellLabel->"In[204]:="],

Cell[BoxData[
    \(Eigensystem[m4x4[a, b]]\)], "Input",
  CellLabel->"In[205]:="],

Cell[BoxData[
    \(bilinear[m4x4[a, b]] // MatrixForm\)], "Input",
  CellLabel->"In[206]:="],

Cell[BoxData[
    \(vec[x] . bilinear[m4x4[a, b]] . vec[x] \[Equal] \ 0\)], "Input",
  CellLabel->"In[207]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[208]:="],

Cell["\<\
Dividing this equation by 2a and considering the affine part of the \
solution with x[4] = 1 only, we obtain the equation of the quadric in the \
shape (with a = x[4] = 1, x[1] = x, x[2] = z, x[3] = y)\
\>", "Text"],

Cell[BoxData[
    \(z*\((1 + b*y)\) - x*\((b - y)\) \[Equal] \ 0\)], "Input",
  CellLabel->"In[209]:="],

Cell["\<\
We plot the solution z = fs[x_,y_,b_] for some values of the \
parameter b\
\>", "Text"],

Cell[BoxData[
    \(fs[x_, y_, b_] := \ x*\((b - y)\)/\((1 + b*y)\)\)], "Input",
  CellLabel->"In[210]:="],

Cell[BoxData[
    \(Plot3D[fs[x, y, 1], {x, \ \(-5\), 5}, {y, \(- .8\), 5}, 
      PlotRange\  \[Rule] \ All, \ BoxRatios\  \[Rule] {1, 1, 6}]\)], "Input",\

  CellLabel->"In[211]:="],

Cell["\<\
As in case 1, the surfaces are tori; they are affinely equivalent \
to hyperbolic paraboloids. In particular, for the limit b \[ShortRightArrow] \
0 we obtain a special case  of  the surfaces discussed under case 1 : z = \
-x*y.\
\>", "Text"],

Cell[BoxData[
    \(Plot3D[fs[x, y, 0], {x, \ \(-5\), 5}, {y, \(-5\), 5}, 
      PlotRange\  \[Rule] \ All, \ BoxRatios\  \[Rule] {1, 1, 3}]\)], "Input",\

  CellLabel->"In[212]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["5.2. J2-quadrics", "Subsection"],

Cell["\<\
We consider non degenerate symmetrc bilinear forms whose associated \
operators have Jordan normal forms with a Jordan cell of dimension 2.  In \
dimension 4 the only possibility is that the Jordan normal form consists of \
two Jordan cells of order 2 with Eigenvalues a,  -a, where a\[NotEqual]0  is \
arbitrary for complex symplectic vector spaces. For a real bilinear form the \
eigenvalue a must be either real or purely imaginary.
We remark that the canonical forms of  non degenerate skew symmetric \
endomorphisms in dimension 4 can not contain Jordan cells of dimension 3 or \
4, see  corollary 2.8.19, example 2.8.8, and proposition 2.8.22.\
\>", "Text"],

Cell[CellGroupData[{

Cell["The real case, a real (Case A.2 in Example 2.8.8)", "Subsubsection"],

Cell[BoxData[
    \(mqj2[
        a_]\  := \ {{a, 0, 0, 0}, {1, a, 0, 0}, {0, 0, \(-a\), \(-1\)}, {0, 
          0, 0, \(-a\)}}\)], "Input",
  CellLabel->"In[213]:="],

Cell[BoxData[
    \(mqj2[a] // MatrixForm\)], "Input",
  CellLabel->"In[214]:="],

Cell[BoxData[
    \(Eigensystem[mqj2[a]]\)], "Input",
  CellLabel->"In[215]:="],

Cell[BoxData[
    \(bqj2[a_]\  := \ bilinear[mqj2[a]]\)], "Input",
  CellLabel->"In[216]:="],

Cell[BoxData[
    \(bqj2[a] // MatrixForm\)], "Input",
  CellLabel->"In[217]:="],

Cell[BoxData[
    \(Det[%]\)], "Input",
  CellLabel->"In[218]:="],

Cell[BoxData[
    \(Simplify[vec[x] . %% . vec[x]] \[Equal] \ 0\)], "Input",
  CellLabel->"In[219]:=",
  CellTags->"j2.1"],

Cell["\<\
The solution is again a torus;  the affine part  x[4] = 1 is the \
hyperbolic paraboloid: With x[1] = x, x[2] = y, x[3] = z the solution is \
y=fsj2[x,z,a] with\
\>", "Text"],

Cell[BoxData[
    \(fsj2[x_, z_, a_]\  := \ \(-x\)*\((1 + a*z)\)/a\)], "Input",
  CellLabel->"In[220]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
The imaginary case, eigenvalue \[ImaginaryI]a, a real (Case \
D.2.1)\
\>", "Subsubsection"],

Cell["\<\
Define the operator with matrix (with respect to a symplectic \
basis)\
\>", "Text"],

Cell[BoxData[
    \(miij2[
        a_]\  := \ {{0, a, 1, 0}, {\(-a\), 0, 0, 1}, {0, 0, 0, a}, {0, 
          0, \(-a\), \ 0}}\)], "Input",
  CellLabel->"In[221]:="],

Cell[BoxData[
    \(miij2[a] // MatrixForm\)], "Input",
  CellLabel->"In[222]:="],

Cell["It is skew symmetric:", "Text"],

Cell[BoxData[
    \(Transpose[miij2[a]] . gram[]\  + \ gram[] . miij2[a]\)], "Input",
  CellLabel->"In[223]:="],

Cell[BoxData[
    \(Eigensystem[miij2[a]]\)], "Input",
  CellLabel->"In[224]:="],

Cell[BoxData[
    \(JordanDecomposition[miij2[a]]\)], "Input",
  CellLabel->"In[225]:="],

Cell[BoxData[
    \(%[\([2]\)] // MatrixForm\)], "Input",
  CellLabel->"In[226]:="],

Cell[BoxData[
    \(bilinear[miij2[a]]\)], "Input",
  CellLabel->"In[227]:="],

Cell[BoxData[
    \(% // MatrixForm\)], "Input",
  CellLabel->"In[228]:="],

Cell["The equation of the corresponding quadric is", "Text"],

Cell[BoxData[
    \(Simplify[vec[x] . %% . vec[x]] \[Equal] \ 0\)], "Input",
  CellLabel->"In[229]:="],

Cell[BoxData[
    \(Det[%%]\)], "Input",
  CellLabel->"In[230]:="],

Cell["\<\
Analogeously to the case of real Eigenvalues one easily shows that \
the resulting quadric in the affine model is a hyperbolic paraboloid.\
\>", \
"Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Jordan decomposition of a dilated operator", "Subsubsection"],

Cell["\<\
Clearly, the Jordan structure of an operator  does not change if \
the operator is multiplied by a constant parameter:\
\>", "Text"],

Cell[BoxData[
    \(bil[t_, a_] := \ t*bilinear[mqj2[a]]\)], "Input",
  CellLabel->"In[231]:="],

Cell[BoxData[
    \(bil[t, a] // MatrixForm\)], "Input",
  CellLabel->"In[232]:="],

Cell[BoxData[
    \(oper[t_, a_] := \ operator[bil[t, a]]\)], "Input",
  CellLabel->"In[233]:="],

Cell[BoxData[
    \(oper[t, a] // MatrixForm\)], "Input",
  CellLabel->"In[234]:="],

Cell[BoxData[
    \(Eigenvalues[oper[t, a]]\)], "Input",
  CellLabel->"In[235]:="],

Cell[BoxData[
    \(Eigensystem[oper[t, a]]\)], "Input",
  CellLabel->"In[236]:="],

Cell[BoxData[
    \(JordanDecomposition[oper[t, a]]\)], "Input",
  CellLabel->"In[237]:="],

Cell[BoxData[
    \(%[\([2]\)] // MatrixForm\)], "Input",
  CellLabel->"In[238]:="],

Cell[BoxData[
    \(%% /. \ t \[Rule] 1/a\)], "Input",
  CellLabel->"In[239]:="],

Cell[BoxData[
    \(vec[x] . %[\([2]\)] . vec[x]\)], "Input",
  CellLabel->"In[240]:="],

Cell[BoxData[
    \(Simplify[%]\)], "Input",
  CellLabel->"In[241]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["5.3. Degenerated bilinear forms", "Subsection"],

Cell["\<\
In this subsection we treat the degenerated symmetric bilinear \
forms and the corresponding associated operators. Some can be obtained as \
limits  or special cases of non degenerated forms, and some are new.\
\>", \
"Text"],

Cell[CellGroupData[{

Cell["\<\
Diagonizable associated operators \[NotEqual] 0 (Case B.1)\
\>", \
"Subsubsection"],

Cell[BoxData[
    \(mm01[a_]\  := \ diag[a, 0, \(-a\), 0]\)], "Input",
  CellLabel->"In[242]:="],

Cell[BoxData[
    \(bdeg1[a_]\  := bilinear[mm01[a]]\)], "Input",
  CellLabel->"In[243]:="],

Cell[BoxData[
    \(bdeg1[a] // MatrixForm\)], "Input",
  CellLabel->"In[244]:="],

Cell[BoxData[
    \(vec[x] . % . vec[x]\  \[Equal] \ 0\)], "Input",
  CellLabel->"In[245]:="],

Cell[TextData[{
  "Special case of equation ",
  ButtonBox["4.2.1",
    ButtonData:>"eq4.2.1",
    ButtonStyle->"Hyperlink"],
  " with b = 0. The solution is the pair of planes x[1] = 0, x[3] = 0 with \
symplectic intersection stb[2]\[Or]stb[4]. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Real Eigenvalues a, -a, and a Jordan 2-cell with Eigenvalue 0 \
(Cases B.2, B.3)\
\>", "Subsubsection"],

Cell[BoxData[
    \(j2d[a_]\  := \ {{a, 0, 0, 0}, \ 
        null[], \ {0, 0, \(-a\), 0}, {0, 1, 0, 0}}\)], "Input",
  CellLabel->"In[246]:="],

Cell[BoxData[
    \(j2d[a] // MatrixForm\)], "Input",
  CellLabel->"In[247]:="],

Cell[BoxData[
    \(JordanDecomposition[j2d[a]]\)], "Input",
  CellLabel->"In[248]:="],

Cell[BoxData[
    \(%[\([2]\)] // MatrixForm\)], "Input",
  CellLabel->"In[249]:="],

Cell[BoxData[
    \(bilinear[j2d[a]] // MatrixForm\)], "Input",
  CellLabel->"In[250]:="],

Cell[BoxData[
    \(vec[x] . % . vec[x]\  \[Equal] \ 0\)], "Input",
  CellLabel->"In[251]:="],

Cell["\<\
The affine part of the solution is a cone, and in case a = 0 the \
\"double counting\" plane x[2] = 0. \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Eigenvalues \[PlusMinus]\[ImaginaryI] a, a real, and a Jordan \
2-cell with Eigenvalue 0 (Case D.2.2)\
\>", "Subsubsection"],

Cell[BoxData[
    \(jd22[a_]\  := \ {{0, 0, a, 0}, \ null[], \ {\(-a\), 0, 0, 0}, 
        stb[2]}\)], "Input",
  CellLabel->"In[252]:="],

Cell[BoxData[
    \(jd22[a] // MatrixForm\)], "Input",
  CellLabel->"In[253]:="],

Cell[BoxData[
    \(JordanDecomposition[jd22[a]]\)], "Input",
  CellLabel->"In[254]:="],

Cell[BoxData[
    \(%[\([2]\)] // MatrixForm\)], "Input",
  CellLabel->"In[255]:="],

Cell[BoxData[
    \(bilinear[jd22[a]] // MatrixForm\)], "Input",
  CellLabel->"In[256]:="],

Cell[BoxData[
    \(vec[x] . % . vec[x] \[Equal] \ 0\)], "Input",
  CellLabel->"In[257]:="],

Cell["\<\
The quadric is a cone with another position to the symplectic frame \
as in the foregoing case.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Two Jordan 2-cells with Eigenvalues 0 (Case B.4)", "Subsubsection"],

Cell[BoxData[
    \(mm00 = {{0, 0, 0, 0}, {1, 0, 0, 0}, {0, 0, 0, \(-1\)}, {0, 0, 0, 
          0}}\)], "Input",
  CellLabel->"In[258]:="],

Cell[BoxData[
    \(mm00 // MatrixForm\)], "Input",
  CellLabel->"In[259]:="],

Cell[BoxData[
    \(Eigensystem[mm00]\)], "Input",
  CellLabel->"In[260]:="],

Cell[BoxData[
    \(JordanDecomposition[mm00]\)], "Input",
  CellLabel->"In[261]:="],

Cell[BoxData[
    \(%[\([2]\)] // MatrixForm\)], "Input",
  CellLabel->"In[262]:="],

Cell[BoxData[
    \(bilinear[mm00] // MatrixForm\)], "Input",
  CellLabel->"In[263]:="],

Cell[BoxData[
    \(vec[x] . % . vec[x]\  \[Equal] \ 0\)], "Input",
  CellLabel->"In[264]:="],

Cell[TextData[{
  "Special case of equation ",
  ButtonBox["j2.1",
    ButtonData:>"j2.1",
    ButtonStyle->"Hyperlink"],
  " with a = 0. The solution is the pair of planes x[1] =  0, x[4] = 0  with \
isotropic intersection stb[2]\[Or]stb[3]. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
A Jordan 4-cell with Eigenvalue 0 (Case B.5)\
\>", "Subsubsection"],

Cell[TextData[{
  "Here one has the same result as in the complex classification, see ",
  ButtonBox["B.5",
    ButtonData:>"B5",
    ButtonStyle->"Hyperlink"],
  " ."
}], "Text"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Appendix.  Some tools of linear algebra",
  FontSize->18]], "Section",
  FontSize->16],

Cell[CellGroupData[{

Cell["\<\
A.2.1  List of the usages for all newly implemented general \
constructions used or developed in this notebook\
\>", "Subsection"],

Cell["\<\
Appendix A2 contains constructs often used in linear algebra. \
Generally, their usages and  definitions can be seen as usual. e.g.\
\>", \
"Text"],

Cell[BoxData[
    \(\(?vec\)\)], "Input",
  CellLabel->"In[84]:="],

Cell[BoxData[
    \(?? vec\)], "Input",
  CellLabel->"In[85]:="],

Cell[BoxData[
    \(vec[x]\)], "Input",
  CellLabel->"In[86]:="],

Cell[TextData[{
  "As a rule, the symbols of the new created objects start with a small \
character, in difference to those coming from ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". "
}], "Text"],

Cell["\<\
Evaluating the next cell one obtaines a list of usages for concepts \
concerning vectors, see A.2.2\
\>", "Text"],

Cell[BoxData[
    \(\({\(?"\<stb\>"\), \(?"\<vec\>"\), \(?"\<randomv\>"\), \ \
\(?"\<null\>"\), \ \(?"\<basis\>"\), \ \(?"\<outzero\>"\), \(?"\<wedge\>"\), \
\(?"\<noprops\>"\), \ \(?"\<dotnorme\>"\), \ \(?"\<sterproj\>"\), \ \
\(?"\<invsterproj\>"\), \(?"\<projpt\>"\), \ \(?"\<reprprojpt\>"\), \ \
\(?"\<projpt3D\>"\), \ \(?"\<spherpt3D\>"\)};\)\)], "Input",
  CellLabel->"In[86]:="],

Cell["\<\
Evaluating the next cell one obtaines a list of usages for concepts \
concerning matrices, see A.2.3\
\>", "Text"],

Cell[BoxData[
    \(\({\(?"\<matrix\>"\), \ \(?"\<nullmatrix\>"\), \ \(?"\<blf\>"\), \ \(?"\
\<rank\>"\), \ \(?"\<randommatrix\>"\)};\)\)], "Input",
  CellLabel->"In[88]:="],

Cell["\<\
The next cell yields objects specific for symplectic linear \
algebra; they are defined in the corresponding sections of the notebook, and \
contained in the package symplecticgeo.m:\
\>", "Text"],

Cell[BoxData[
    \(\({\(?"\<ssp\>"\), \ \(?"\<grammatrix\>"\), \ \(?"\<ortho\>"\), \ \
\.08?"\<orthogram\>", \ \(?"\<sym\>"\), \ \(?"\<symortho\>"\), \
\(?"\<orthosymplectic\>"\), \(?"\<polarspace\>"\), \ \(?"\<transvection\>"\), \
\ \(?"\<transv\>"\), \(?"\<sym\>"\)};\)\)], "Input",
  CellLabel->"In[87]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["A.2.2 Vector calculus", "Subsection",
  FontFamily->"Times New Roman"],

Cell[CellGroupData[{

Cell["Some vectors", "Subsubsection"],

Cell[BoxData[
    \(dim::usage\  = \ "\<dim is the dimension of the vector space under \
consideration; it must be set within the Global Context.\>"\)], "Input",
  CellLabel->"In[46]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(stb::usage = "\<stb[i, d:dim] is the i-th standard basis vector, 1 <= \
i <= d.\>";\)\)], "Input",
  CellLabel->"In[47]:=",
  InitializationCell->True],

Cell[BoxData[
    \(stb[i_Integer, d_:  dim] := \(IdentityMatrix[d]\)[\([i]\)]\)], "Input",
  CellLabel->"In[48]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(vec::usage\  = "\<vec[x, n:dim] is the n-vector with components x[i].\
\>";\)\)], "Input",
  CellLabel->"In[49]:=",
  InitializationCell->True],

Cell[BoxData[
    \(vec[x_, \ n_:  dim]\  := \ Array[x, \ \ n]\)], "Input",
  CellLabel->"In[50]:=",
  InitializationCell->True,
  FontFamily->"Times New Roman",
  FontSize->15],

Cell[BoxData[
    \(\(null::usage\  = \ "\<null[d:dim] is the zero vector of length \
d.\>";\)\)], "Input",
  CellLabel->"In[51]:=",
  InitializationCell->True],

Cell[BoxData[
    \(null[d_:  dim]\  := Table[0, {d}]\)], "Input",
  CellLabel->"In[52]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(dotnorme::usage = "\<dotnorme[b] yields the unit vector with respect \
to the Dot
product in direction of b, b!=o.\>";\)\)], "Input",
  CellLabel->"In[53]:=",
  InitializationCell->True],

Cell[BoxData[
    \(dotnorme[b_] := b/Sqrt[b . b]\)], "Input",
  CellLabel->"In[54]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(sterproj::usage = "\<sterproj is the stereographical projection of \
the unit 
n-sphere from the north pole stb[n+1] onto the euclidean (equatorial) \
n-space. 
WARNING: The result is correct only for arguments of norm 1 !\>";\)\)], \
"Input",
  CellLabel->"In[55]:=",
  InitializationCell->True],

Cell[BoxData[
    \(sterproj[v_List] := 
      Module[{m = Length[v], l, w}, 
        l = null[m]; \[IndentingNewLine]w = v[\([m]\)]; \[IndentingNewLine]If[
          smoothing[w - 1] \[Equal] 0, 
          For[i = 1, i < m, \(i++\), l[\([i]\)] = Infinity], 
          l = Simplify[stb[m, m] + \((v - stb[m, m])\)/\((1 - w)\)], 
          l = Simplify[
              stb[m, m] + \((v - stb[m, m])\)/\((1 - 
                      w)\)]]; \[IndentingNewLine]l = 
          Take[l, m - 1]; \[IndentingNewLine]l]\)], "Input",
  CellLabel->"In[56]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(invsterproj::usage = "\<invsterproj is the inverse of the general \
euclidean 
stereographic projection sterproj.\>";\)\)], "Input",
  CellLabel->"In[57]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(invsterproj[y_List] := 
      Module[{n = Length[y] + 1, x = Append[y, 0], u = y . y}, 
        x*2/\((1 + u)\) + stb[n, n]*\((u - 1)\)/\((u + 1)\)]\)\(\n\)
    \)\)], "Input",
  CellLabel->"In[58]:=",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Random vectors", "Subsubsection"],

Cell[BoxData[
    \(\(randomv::usage = "\<randomv[n:dim] is a random n-vector with values \
in the  positive unit n-cube.\>";\)\)], "Input",
  CellLabel->"In[59]:=",
  InitializationCell->True],

Cell[BoxData[
    \(randomv[n_:  dim]\  := \ Table[Random[], {i, \ n}]\)], "Input",
  CellLabel->"In[60]:=",
  InitializationCell->True,
  FontFamily->"Times New Roman"]
}, Closed]],

Cell[CellGroupData[{

Cell["Wedge product: Definitions", "Subsubsection"],

Cell[BoxData[
    \(dim\)], "Input",
  CellLabel->"In[118]:="],

Cell[BoxData[
    \(\(wedge::usage\  = \ "\<wedge[a,b] is the alternate product of vectors \
a,b given by the matrix of its coordinates.\>";\)\)], "Input",
  CellLabel->"In[61]:=",
  InitializationCell->True],

Cell[BoxData[
    \(wedge[a_, b_] := \ 
      Module[{d = Length[a]}, 
        Table[a[\([i]\)]*b[\([j]\)] - a[\([j]\)]*b[\([i]\)], {i, 1, d}, {j, 
            1, d}]]\)], "Input",
  CellLabel->"In[62]:=",
  InitializationCell->True],

Cell[BoxData[
    \(wedge[vec[x], vec[y]] // MatrixForm\)], "Input",
  CellLabel->"In[121]:="],

Cell[BoxData[
    \(wedge[vec[x], t*vec[x]] // MatrixForm\)], "Input",
  CellLabel->"In[122]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Lists of vectors", "Subsubsection"],

Cell[BoxData[
    \(\(basis::usage = "\<basis[v] yields a sublist of v being a basis of the \
span of 
the vectors v[[i]]of v; v must be a matrix.\>";\)\)], "Input",
  CellLabel->"In[63]:=",
  InitializationCell->True],

Cell[BoxData[
    \(basis[v_] := 
      Module[{r, l, w, i, fdrop}, 
        If[Not[MatrixQ[v]], Print["\<Argument is not a matrix, no span!\>"]; 
          Return[]]; \[IndentingNewLine]r = rank[v]; \[IndentingNewLine]If[
          r \[Equal] 0, Print["\<The span is the null space!\>"]; 
          Return[]]; \[IndentingNewLine]w = 
          outzero[v, \ 10^\((\(-20\))\)]; \[IndentingNewLine]fdrop[i_] := 
          Drop[w, {i}]; \[IndentingNewLine]Label[new]; \[IndentingNewLine]l = 
          Length[w]; \[IndentingNewLine]If[r \[Equal] l, 
          Return[w]]; \[IndentingNewLine]For[i = 1, 
          i \[LessEqual] l, \(i++\), 
          If[rank[fdrop[i]] \[Equal] r, w = fdrop[i]; Goto[new]]]]\)], "Input",\

  CellLabel->"In[64]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(outzero::usage = "\<outzero[b_List,dd] is the sublist of the vectors \
b[[i]] with Abs[b[[i]].b[[i]]] > dd; the default value of dd is 10^(-20).\>";\
\)\)], "Input",
  CellLabel->"In[65]:=",
  InitializationCell->True],

Cell[BoxData[
    \(outzero[b_List, \ dd_:  10^\((\(-20\))\)] := 
      Module[{v = {}, l = Length[b]}, 
        For[i = 1, i \[LessEqual] l, \(i++\), 
          If[Not[Chop[b[\([i]\)] . b[\([i]\)], dd] \[Equal] 0], 
            AppendTo[v, b[\([i]\)]], Null[], AppendTo[v, b[\([i]\)]]]]; 
        Return[v]]\)], "Input",
  CellLabel->"In[66]:=",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["noprops: Definition", "Subsubsection"],

Cell[BoxData[
    \(Clear[c]\)], "Input",
  CellLabel->"In[152]:="],

Cell[BoxData[
    \(Clear[noprops]\)], "Input",
  CellLabel->"In[153]:="],

Cell[BoxData[
    \(\(noprops::usage\  = \ "\<noprops[c] deletes lines c[[j]] of the matrix \
c which are proportional to an element c[[i]] with i<j, and returns the \
matrix originated that way.\>";\)\)], "Input",
  CellLabel->"In[67]:=",
  InitializationCell->True],

Cell[BoxData[
    \(noprops[c_] := \ 
      Module[{m = 1, cc, \ d}, \[IndentingNewLine]If[
          Length[c] \[LessEqual] 1, Return[c]]; \[IndentingNewLine]If[
          Not[MatrixQ[c]], \ Print["\<incorrect argument!\>"]; 
          Return[]]; \[IndentingNewLine]d = 
          Length[c[\([1]\)]]; \[IndentingNewLine]cc = 
          outzero[c]; \[IndentingNewLine]If[Length[cc] \[LessEqual] 1, 
          Return[cc]]; \[IndentingNewLine]Label[ttt]; \[IndentingNewLine]For[
          j = m + 1, j \[LessEqual] \ Length[cc], \(j++\), \ 
          If[Chop[wedge[cc[\([m]\)], cc[\([j]\)]], 
                10^\((\(-20\))\)] \[Equal] \ nullmatrix[d], 
            cc[\([j]\)] = null[d]]]; \[IndentingNewLine]cc = outzero[cc]; 
        m\  = \ m + 1; \[IndentingNewLine]If[Length[cc]\  \[LessEqual] \ m, \ 
          Return[cc], Goto[ttt]]]\)], "Input",
  CellLabel->"In[68]:=",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Tests for noprops", "Subsubsection"],

Cell[BoxData[
    \(noprops[{null[], null[], null[]}]\)], "Input",
  CellLabel->"In[156]:="],

Cell[BoxData[
    \(noprops[{{0, 0, 2, 0}, {0, 0, 1, 0}, {0, 0, 3, 0}}]\)], "Input",
  CellLabel->"In[157]:="],

Cell[BoxData[
    \(noprops[{{0, 0, 2, 0}, {0, 0, 0, 0}, {0, 0, 3, 0}}]\)], "Input",
  CellLabel->"In[158]:="],

Cell[BoxData[
    \(noprops[{{0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 3, 0}}]\)], "Input",
  CellLabel->"In[159]:="],

Cell[BoxData[
    \(noprops[{{0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}}]\)], "Input",
  CellLabel->"In[160]:="],

Cell[BoxData[
    \(noprops[{{1, 0, 0, 0}, {0, 0, 0, 0}, {0, 2, 0, 0}}]\)], "Input",
  CellLabel->"In[161]:="],

Cell[BoxData[
    \(noprops[{{1, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}}]\)], "Input",
  CellLabel->"In[162]:="],

Cell[BoxData[
    \(noprops[{{1, 0, 0, 0}, {0, 0, 0, 0}, {a, 0, 0, 0}, {0, a, 0, 1}, \ 
        randomv[]}]\)], "Input",
  CellLabel->"In[163]:="],

Cell[BoxData[
    \(Clear[b]; 
    noprops[{{1, 0, 0, 0}, {0, 0, b, 0}, {Sin[t], 0, 0, 0}, {Cos[t], 0, 0, 
          0}}]\)], "Input",
  CellLabel->"In[164]:="],

Cell[BoxData[
    \(noprops[{}]\)], "Input",
  CellLabel->"In[165]:="],

Cell[BoxData[
    \(c2 = {2, vec[x]}\)], "Input",
  CellLabel->"In[166]:="],

Cell[BoxData[
    \(noprops[c2]\)], "Input",
  CellLabel->"In[167]:="],

Cell[BoxData[
    \(c2\  = {stb[1], \ Pi*stb[1]}\)], "Input",
  CellLabel->"In[168]:="],

Cell[BoxData[
    \(noprops[c2]\)], "Input",
  CellLabel->"In[169]:="],

Cell[BoxData[
    \(Clear[c2, a]\)], "Input",
  CellLabel->"In[170]:="],

Cell[BoxData[
    \(wedge[stb[1], \ N[Pi]*stb[1]] // MatrixForm\)], "Input",
  CellLabel->"In[171]:="],

Cell[BoxData[
    \(% \[Equal] \ nullmatrix[]\)], "Input",
  CellLabel->"In[172]:="]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["A.2.3 Matrix calculus", "Subsection"],

Cell[CellGroupData[{

Cell["General matrix. ", "Subsubsection"],

Cell[TextData[{
  "This is a ",
  StyleBox["general square matrix ",
    FontColor->RGBColor[0, 0, 1]],
  "with elements b[i,j]"
}], "Text"],

Cell[BoxData[
    \(matrix::usage\  = \ "\<matrix[b,n:dim] is a square matrix of order n \
with elements b[i,j].\>"\)], "Input",
  CellLabel->"In[69]:=",
  InitializationCell->True],

Cell[BoxData[
    \(matrix[b_, n_:  dim] := \ Array[b, {n, n}]\)], "Input",
  CellLabel->"In[70]:=",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["The nullmatrix", "Subsubsection"],

Cell[BoxData[
    \(\(nullmatrix::usage\  = \ "\<nullmatrix[d] is the square nullmatrix of \
order d; the default value of d is dim.\>";\)\)], "Input",
  CellLabel->"In[71]:=",
  InitializationCell->True],

Cell[BoxData[
    \(nullmatrix[d_:  dim] := \ Table[0, {i, 1, d}, {j, 1, d}]\)], "Input",
  CellLabel->"In[72]:=",
  InitializationCell->True],

Cell[BoxData[
    \(nullmatrix[] // MatrixForm\)], "Input",
  CellLabel->"In[177]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["General symmetric matrics blf[a]", "Subsubsection"],

Cell[BoxData[
    \(Clear[a, blf]\)], "Input",
  CellLabel->"In[204]:="],

Cell["\<\
For any symbol a we define the symmetric matrix blf[a] with \
coefficients a[i,j]:\
\>", "Text"],

Cell[BoxData[
    \(\(blf::usage\  = \ "\<blf[a, d:dim] is a symmetric matrix of order d \
with elements a[i,j].\>";\)\)], "Input",
  CellLabel->"In[73]:=",
  InitializationCell->True],

Cell[BoxData[
    \(blf[a_, d_:  dim] := \ 
      Module[{b\  = \ matrix[a]}, \[IndentingNewLine]For[i\  = \ 1, 
          i < d, \(i++\), 
          For[j = i + 1, j \[LessEqual] d, \(j++\), 
            a[j, i] = a[i, j]]]; \[IndentingNewLine]b]\)], "Input",
  CellLabel->"In[74]:=",
  InitializationCell->True],

Cell[BoxData[
    \(blf[a] // MatrixForm\)], "Input",
  CellLabel->"In[207]:="],

Cell[BoxData[
    \(blf[a]\  == \ Transpose[blf[a]]\)], "Input",
  CellLabel->"In[208]:="]
}, Closed]],

Cell[CellGroupData[{

Cell["Rank of a matrix", "Subsubsection",
  FontFamily->"Times New Roman"],

Cell["\<\
The rank of a matrix can be calculated by the following module, if \
the matrix is not too large:\
\>", "Text",
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(\(rank::usage = "\<rank[m] calculates the rank of the rectangular \
matrix m of a not
 too large order (< 10000).\>";\)\)], "Input",
  CellLabel->"In[75]:=",
  InitializationCell->True],

Cell[BoxData[
    \(rank[m_]\  := \ 
      Module[{\ di\ , \ nsp}, \[IndentingNewLine]If[Not[MatrixQ[m]], \ 
          Print["\<incorrect argument!\>"]; 
          Return[]]; \[IndentingNewLine]di\  = \ 
          Length[m[\([1]\)]]; \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ nsp\  = \ 
          NullSpace[m]; \n\ \ \ \ \ \ \ \ \ \ \ di\  - \ 
          Length[nsp]]\)], "Input",
  CellLabel->"In[76]:=",
  InitializationCell->True,
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(rank[{stb[1], \ stb[3], \ stb[4]}]\)], "Input",
  CellLabel->"In[180]:=",
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(rank[{stb[1], \ stb[1], \ stb[4]}]\)], "Input",
  CellLabel->"In[181]:=",
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(rank[{{Cos[t], Sin[t]}, {\(-Sin[t]\), Cos[t]}}]\)], "Input",
  CellLabel->"In[182]:=",
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(Timing[rank[IdentityMatrix[10^3]]]\)], "Input",
  CellLabel->"In[183]:=",
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(Timing[rank[matrix[b]]]\)], "Input",
  CellLabel->"In[184]:=",
  FontFamily->"Times New Roman"],

Cell["\<\
Of course, it makes not much sense to calculate the rank of a \
symbolic matrix the elements of which are not given explicitely; for such \
matrices of  high order time and resources are necedssary. For numerically \
given matrices the caculation runs fast:\
\>", "Text"],

Cell[BoxData[
    \(Timing[rank[randommatrix[300, 400]]]\)], "Input",
  CellLabel->"In[185]:=",
  FontFamily->"Times New Roman"]
}, Closed]],

Cell[CellGroupData[{

Cell["Random matrices", "Subsubsection"],

Cell[BoxData[
    \(\(randommatrix::usage\  = \ "\<randommatrix[k,l]is a matrix with k rows \
and l columns, with random elements out of the unit intervall.\>";\)\)], \
"Input",
  CellLabel->"In[77]:=",
  InitializationCell->True],

Cell[BoxData[
    \(randommatrix[k_, l_]\  := \ Table[randomv[l], \ {k}]\)], "Input",
  CellLabel->"In[78]:=",
  InitializationCell->True,
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(randommatrix[2, 3] // MatrixForm\)], "Input",
  CellLabel->"In[188]:=",
  FontFamily->"Times New Roman"],

Cell[BoxData[
    \(Timing[rank[randommatrix[300, 400]]]\)], "Input",
  CellLabel->"In[189]:=",
  FontFamily->"Times New Roman"]
}, Closed]],

Cell[CellGroupData[{

Cell["Transposition matrix. Reflection at hyperplanes", "Subsubsection"],

Cell[TextData[{
  StyleBox["Transposition:",
    FontColor->RGBColor[0, 0, 1]],
  " The following matrix may be used to transpose rows, columns, or \
coordinates:"
}], "Text"],

Cell[BoxData[
    \(\(\(trp::usage\)\(\ \)\(=\)\(\ \)\("\<trp[i,j,n:dim] is the unit matrix \
of order n where the i-th and the j-th row are transposed.\>"\)\(\ \)\)\)], \
"Input",
  CellLabel->"In[79]:=",
  InitializationCell->True],

Cell[BoxData[
    \(trp[i_, j_, n_:  dim] := 
      Module[{m = IdentityMatrix[n]}, \ 
        m[\([i]\)] = stb[j, n]; \[IndentingNewLine]m[\([j]\)] = 
          stb[i, n]; \[IndentingNewLine]m]\)], "Input",
  CellLabel->"In[80]:=",
  InitializationCell->True],

Cell[BoxData[
    \(trp[1, 3] // MatrixForm\)], "Input",
  CellLabel->"In[192]:="],

Cell[BoxData[
    \(trp[3, 3] // MatrixForm\)], "Input",
  CellLabel->"In[193]:="],

Cell[BoxData[
    \(trp[5, 6, 27] \[Equal] \ Transpose[trp[5, 6, 27]]\)], "Input",
  CellLabel->"In[194]:="],

Cell[BoxData[
    \(trp[5, 6, 27] \[Equal] \ Inverse[trp[5, 6, 27]]\)], "Input",
  CellLabel->"In[195]:="],

Cell["Left multiplication with trp transposes the rows:", "Text"],

Cell[BoxData[
    \(matrix[b] // MatrixForm\)], "Input",
  CellLabel->"In[196]:="],

Cell["Now we transpose two rows of the matrix:", "Text"],

Cell[BoxData[
    \(trp[1, 3] . % // MatrixForm\)], "Input",
  CellLabel->"In[197]:="],

Cell["Right multiplication transposes the columns:", "Text"],

Cell[BoxData[
    \(%% . trp[1, 3] // MatrixForm\)], "Input",
  CellLabel->"In[198]:="],

Cell["\<\
Left and right multiplication corresponds to the transposition of \
the i-th and j-th coordinates:\
\>", "Text"],

Cell[BoxData[
    \(trp[1, 3] . % // MatrixForm\)], "Input",
  CellLabel->"In[199]:="],

Cell[BoxData[""], "Input",
  CellLabel->"In[200]:="],

Cell[TextData[{
  StyleBox["Reflection",
    FontColor->RGBColor[0, 0, 1]],
  " at the hyperplane x[i]\[Equal] 0:"
}], "Text"],

Cell[BoxData[
    \(refl[i_, n_:  dim]\  := 
      Module[{mm = 
            IdentityMatrix[n]}, \[IndentingNewLine]mm[\([i, 
              i]\)] = \ \(-1\); \[IndentingNewLine]mm]\)], "Input",
  CellLabel->"In[81]:=",
  InitializationCell->True],

Cell[BoxData[
    \(refl[2] . vec[x]\)], "Input",
  CellLabel->"In[202]:="]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["References", "Subsubtitle",
  FontFamily->"Times"],

Cell["\<\
[Jag1] I. M. Jaglom.  Quadratic and skew-symmetric bilinear forms \
in a real symplectic space.  (Russian). Trudi Sem. Vect.-Tens. Analisu \
8:364-381. (1950).\
\>", "Text",
  CellTags->"Jag1"],

Cell["\<\
[Jag2] I. M. Jaglom. On the linear subspaces of a symplectic \
space.(Russian).Trudi Sem.Vect.-Tens. Analisu 9: 309-318. (1952).\
\>", "Text"],

Cell["\<\
[Jan] Pekka Janhunen. Declare.m. \
http://library.wolfram.com/infocenter/MathSource/4574/\
\>", "Text"],

Cell["\<\
[Mal] A. I. Maltzev. Grundlagen der linearen Algebra (Russ.) , \
Moskau 1956.\
\>", "Text",
  CellTags->"Mal"],

Cell["\<\
[OSIII] A.L. Onishchik,  Sulanke, R. Projektive und \
Cayley-Kleinsche Geometrien, \
http://www-irm.mathematik.hu-berlin.de/~sulanke\
\>", "Text",
  CellTags->"OSIII"]
}, Closed]],

Cell[CellGroupData[{

Cell["Address", "Subsubtitle",
  FontFamily->"Times"],

Cell["sulanke@mathematik.hu-berlin.de", "Text",
  FontFamily->"Times",
  CellTags->"address"],

Cell["Homepage:http://www-irm.mathematik.hu-berlin.de/~sulanke", "Text",
  CellLabel->"In[1]:=",
  FontFamily->"Times",
  CellTags->"hp"]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.2 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 944}},
AutoGeneratedPackage->Automatic,
WindowSize->{1008, 887},
WindowMargins->{{23, Automatic}, {Automatic, -1}},
ShowCellTags->True,
StyleDefinitions -> "sustyle.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "copyrigt"->{
    Cell[11647, 300, 55, 1, 54, "Subsection",
      CellTags->"copyrigt"]},
  "skewop"->{
    Cell[101957, 3467, 126, 3, 45, "Input",
      CellTags->"skewop"]},
  "torus"->{
    Cell[109676, 3767, 131, 3, 45, "Input",
      CellTags->"torus"],
    Cell[112132, 3866, 137, 3, 45, "Input",
      CellTags->"torus"]},
  "eq4.2.1"->{
    Cell[121131, 4203, 135, 3, 45, "Input",
      CellTags->"eq4.2.1"]},
  "diag"->{
    Cell[121783, 4224, 334, 7, 62, "Text",
      CellTags->"diag"]},
  "j2.1"->{
    Cell[122989, 4270, 155, 4, 45, "Input",
      InitializationCell->True,
      CellTags->"j2.1"],
    Cell[136443, 4734, 122, 3, 45, "Input",
      CellTags->"j2.1"]},
  "opj2"->{
    Cell[124935, 4346, 130, 3, 45, "Input",
      CellTags->"opj2"]},
  "B5"->{
    Cell[125431, 4362, 155, 4, 52, "Subsubsection",
      CellTags->"B5"]},
  "Jag1"->{
    Cell[162446, 5715, 203, 5, 42, "Text",
      CellTags->"Jag1"]},
  "Mal"->{
    Cell[162923, 5732, 120, 4, 42, "Text",
      CellTags->"Mal"]},
  "OSIII"->{
    Cell[163046, 5738, 177, 5, 42, "Text",
      CellTags->"OSIII"]},
  "address"->{
    Cell[163316, 5751, 93, 2, 42, "Text",
      CellTags->"address"]},
  "hp"->{
    Cell[163412, 5755, 137, 3, 42, "Text",
      CellTags->"hp"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"copyrigt", 164259, 5780},
  {"skewop", 164349, 5783},
  {"torus", 164434, 5786},
  {"eq4.2.1", 164590, 5791},
  {"diag", 164675, 5794},
  {"j2.1", 164756, 5797},
  {"opj2", 164939, 5803},
  {"B5", 165019, 5806},
  {"Jag1", 165107, 5809},
  {"Mal", 165187, 5812},
  {"OSIII", 165268, 5815},
  {"address", 165353, 5818},
  {"hp", 165434, 5821}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 72, 1, 73, "Title"],
Cell[1851, 56, 224, 8, 33, "Subsubtitle",
  Evaluatable->False],
Cell[2078, 66, 125, 4, 39, "Text",
  Evaluatable->False],
Cell[2206, 72, 249, 9, 60, "Text",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[2480, 85, 80, 2, 38, "Subsubsection"],
Cell[2563, 89, 791, 14, 88, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3391, 108, 34, 0, 46, "Subsection"],
Cell[3428, 110, 5869, 115, 680, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[9334, 230, 30, 0, 40, "Subsection"],
Cell[9367, 232, 455, 8, 68, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[9859, 245, 27, 0, 40, "Subsection"],
Cell[9889, 247, 226, 5, 28, "Text"],
Cell[10118, 254, 701, 16, 88, "Text"],
Cell[10822, 272, 208, 5, 28, "Text"],
Cell[11033, 279, 414, 10, 48, "Text"],
Cell[11450, 291, 160, 4, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[11647, 300, 55, 1, 54, "Subsection",
  CellTags->"copyrigt"],
Cell[11705, 303, 560, 9, 88, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[12302, 317, 33, 0, 60, "Section"],

Cell[CellGroupData[{
Cell[12360, 321, 38, 0, 46, "Subsection"],
Cell[12401, 323, 1025, 20, 204, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[13463, 348, 64, 0, 40, "Subsection"],
Cell[13530, 350, 328, 6, 48, "Text"],
Cell[13861, 358, 166, 3, 29, "Text"],
Cell[14030, 363, 55, 0, 28, "Text"],
Cell[14088, 365, 92, 3, 32, "Input"],
Cell[14183, 370, 324, 6, 48, "Text"],
Cell[14510, 378, 96, 2, 32, "Text"],
Cell[14609, 382, 215, 5, 32, "Input",
  InitializationCell->True],
Cell[14827, 389, 94, 2, 32, "Text"],
Cell[14924, 393, 152, 4, 32, "Input",
  Evaluatable->False],
Cell[15079, 399, 412, 7, 68, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15528, 411, 36, 0, 46, "Subsection"],
Cell[15567, 413, 96, 3, 32, "Text"],
Cell[15666, 418, 116, 3, 28, "Text"],
Cell[15785, 423, 186, 7, 31, "Text",
  Evaluatable->False],
Cell[15974, 432, 105, 2, 32, "Text"],
Cell[16082, 436, 241, 5, 48, "Text"],
Cell[16326, 443, 69, 2, 31, "Input"],
Cell[16398, 447, 710, 14, 123, "Text"],
Cell[17111, 463, 66, 2, 31, "Input"],
Cell[17180, 467, 71, 2, 31, "Input"],
Cell[17254, 471, 62, 2, 31, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[17365, 479, 78, 0, 92, "Section"],
Cell[17446, 481, 710, 11, 108, "Text"],

Cell[CellGroupData[{
Cell[18181, 496, 79, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[18285, 500, 72, 0, 38, "Subsubsection"],
Cell[18360, 502, 230, 5, 77, "Input",
  InitializationCell->True],
Cell[18593, 509, 583, 11, 146, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[19213, 525, 72, 0, 38, "Subsubsection"],
Cell[19288, 527, 70, 2, 31, "Input"],
Cell[19361, 531, 76, 0, 28, "Text"],
Cell[19440, 533, 65, 2, 31, "Input"],
Cell[19508, 537, 65, 2, 31, "Input"],
Cell[19576, 541, 80, 2, 31, "Input"],
Cell[19659, 545, 50, 0, 28, "Text"],
Cell[19712, 547, 80, 2, 31, "Input"],
Cell[19795, 551, 98, 2, 31, "Input"],
Cell[19896, 555, 78, 2, 31, "Input"],
Cell[19977, 559, 31, 0, 28, "Text"],
Cell[20011, 561, 176, 4, 31, "Input"],
Cell[20190, 567, 136, 4, 31, "Input"],
Cell[20329, 573, 74, 0, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[20440, 578, 75, 0, 32, "Subsubsection"],
Cell[20518, 580, 43, 0, 28, "Text"],
Cell[20564, 582, 66, 2, 31, "Input"],
Cell[20633, 586, 92, 3, 28, "Text"],
Cell[20728, 591, 92, 2, 31, "Input"],
Cell[20823, 595, 65, 2, 31, "Input"],
Cell[20891, 599, 65, 2, 31, "Input"],
Cell[20959, 603, 134, 3, 28, "Text"],
Cell[21096, 608, 251, 5, 77, "Input",
  InitializationCell->True],
Cell[21350, 615, 195, 5, 31, "Input",
  InitializationCell->True],
Cell[21548, 622, 205, 4, 77, "Input",
  InitializationCell->True],
Cell[21756, 628, 272, 6, 54, "Input",
  InitializationCell->True],
Cell[22031, 636, 311, 7, 60, "Text"],
Cell[22345, 645, 151, 3, 31, "Input"],
Cell[22499, 650, 70, 2, 31, "Input"],
Cell[22572, 654, 65, 2, 31, "Input"],
Cell[22640, 658, 70, 2, 31, "Input"],
Cell[22713, 662, 157, 4, 31, "Input"],
Cell[22873, 668, 237, 7, 28, "Text"],
Cell[23113, 677, 100, 2, 31, "Input"],
Cell[23216, 681, 41, 0, 28, "Text"],
Cell[23260, 683, 144, 3, 31, "Input"],
Cell[23407, 688, 101, 3, 28, "Text"],
Cell[23511, 693, 88, 2, 31, "Input"],
Cell[23602, 697, 66, 2, 31, "Input"],
Cell[23671, 701, 94, 3, 28, "Text"],
Cell[23768, 706, 82, 2, 31, "Input"],
Cell[23853, 710, 76, 2, 31, "Input"],
Cell[23932, 714, 79, 2, 31, "Input"],
Cell[24014, 718, 74, 2, 31, "Input"],
Cell[24091, 722, 53, 0, 28, "Text"],
Cell[24147, 724, 97, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[24281, 731, 52, 0, 38, "Subsubsection"],
Cell[24336, 733, 66, 2, 31, "Input"],
Cell[24405, 737, 79, 2, 31, "Input"],
Cell[24487, 741, 81, 2, 31, "Input"],
Cell[24571, 745, 85, 2, 31, "Input"],
Cell[24659, 749, 93, 2, 31, "Input"],
Cell[24755, 753, 88, 2, 31, "Input"],
Cell[24846, 757, 65, 2, 31, "Input"],
Cell[24914, 761, 69, 2, 31, "Input"],
Cell[24986, 765, 70, 2, 31, "Input"],
Cell[25059, 769, 79, 2, 31, "Input"],
Cell[25141, 773, 68, 0, 28, "Text"],
Cell[25212, 775, 65, 2, 31, "Input"],
Cell[25280, 779, 70, 2, 31, "Input"],
Cell[25353, 783, 79, 2, 31, "Input"],
Cell[25435, 787, 114, 3, 28, "Text"],
Cell[25552, 792, 66, 2, 31, "Input"],
Cell[25621, 796, 67, 2, 31, "Input"],
Cell[25691, 800, 71, 2, 31, "Input"],
Cell[25765, 804, 66, 2, 31, "Input"],
Cell[25834, 808, 71, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[25954, 816, 74, 0, 40, "Subsection"],

Cell[CellGroupData[{
Cell[26053, 820, 36, 0, 38, "Subsubsection"],
Cell[26092, 822, 295, 7, 28, "Text"],
Cell[26390, 831, 79, 2, 31, "Input"],
Cell[26472, 835, 107, 3, 28, "Text"],
Cell[26582, 840, 80, 2, 31, "Input"],
Cell[26665, 844, 707, 20, 100, "Text"],
Cell[27375, 866, 447, 10, 48, "Text"],
Cell[27825, 878, 578, 13, 48, "Text"],
Cell[28406, 893, 133, 3, 31, "Input",
  Evaluatable->False],
Cell[28542, 898, 153, 3, 31, "Input",
  Evaluatable->False],
Cell[28698, 903, 96, 3, 28, "Text"],
Cell[28797, 908, 461, 11, 48, "Text"],
Cell[29261, 921, 145, 3, 28, "Text"],
Cell[29409, 926, 333, 6, 100, "Input",
  InitializationCell->True],
Cell[29745, 934, 659, 13, 169, "Input",
  InitializationCell->True],
Cell[30407, 949, 63, 0, 28, "Text"],
Cell[30473, 951, 231, 5, 77, "Input",
  InitializationCell->True],
Cell[30707, 958, 225, 5, 54, "Input",
  InitializationCell->True],
Cell[30935, 965, 79, 1, 28, "Text"],
Cell[31017, 968, 79, 2, 31, "Input"],
Cell[31099, 972, 84, 2, 31, "Input"],
Cell[31186, 976, 85, 2, 31, "Input"],
Cell[31274, 980, 83, 2, 31, "Input"],
Cell[31360, 984, 35, 0, 28, "Text"],
Cell[31398, 986, 111, 2, 31, "Input"],
Cell[31512, 990, 613, 12, 68, "Text"],
Cell[32128, 1004, 138, 3, 31, "Input",
  Evaluatable->False],
Cell[32269, 1009, 153, 3, 31, "Input",
  Evaluatable->False],
Cell[32425, 1014, 187, 4, 28, "Text"],
Cell[32615, 1020, 355, 6, 123, "Input",
  InitializationCell->True],
Cell[32973, 1028, 652, 13, 169, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[33662, 1046, 49, 0, 38, "Subsubsection"],
Cell[33714, 1048, 194, 4, 28, "Text"],
Cell[33911, 1054, 83, 2, 31, "Input"],
Cell[33997, 1058, 146, 3, 31, "Input"],
Cell[34146, 1063, 87, 2, 31, "Input"],
Cell[34236, 1067, 146, 3, 31, "Input"],
Cell[34385, 1072, 70, 2, 31, "Input"],
Cell[34458, 1076, 79, 2, 31, "Input"],
Cell[34540, 1080, 96, 2, 31, "Input"],
Cell[34639, 1084, 75, 2, 31, "Input"],
Cell[34717, 1088, 146, 3, 31, "Input"],
Cell[34866, 1093, 68, 2, 31, "Input"],
Cell[34937, 1097, 146, 3, 31, "Input"],
Cell[35086, 1102, 158, 4, 28, "Text"],
Cell[35247, 1108, 86, 2, 31, "Input"],
Cell[35336, 1112, 146, 3, 31, "Input"],
Cell[35485, 1117, 73, 2, 31, "Input"],
Cell[35561, 1121, 146, 3, 31, "Input"],
Cell[35710, 1126, 73, 2, 31, "Input"],
Cell[35786, 1130, 146, 3, 31, "Input"],
Cell[35935, 1135, 73, 2, 31, "Input"],
Cell[36011, 1139, 146, 3, 31, "Input"],
Cell[36160, 1144, 64, 0, 28, "Text"],
Cell[36227, 1146, 95, 2, 31, "Input"],
Cell[36325, 1150, 89, 2, 31, "Input"],
Cell[36417, 1154, 109, 2, 31, "Input"],
Cell[36529, 1158, 89, 2, 31, "Input"],
Cell[36621, 1162, 111, 3, 28, "Text"],
Cell[36735, 1167, 88, 2, 31, "Input"],
Cell[36826, 1171, 146, 3, 31, "Input"],
Cell[36975, 1176, 135, 3, 28, "Text"],
Cell[37113, 1181, 81, 2, 31, "Input"],
Cell[37197, 1185, 146, 3, 31, "Input"],
Cell[37346, 1190, 86, 2, 31, "Input"],
Cell[37435, 1194, 146, 3, 31, "Input"],
Cell[37584, 1199, 80, 2, 31, "Input"],
Cell[37667, 1203, 70, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[37786, 1211, 65, 0, 40, "Subsection"],
Cell[37854, 1213, 488, 8, 68, "Text"],

Cell[CellGroupData[{
Cell[38367, 1225, 45, 0, 38, "Subsubsection"],
Cell[38415, 1227, 306, 6, 100, "Input",
  InitializationCell->True],
Cell[38724, 1235, 470, 8, 146, "Input",
  InitializationCell->True],
Cell[39197, 1245, 148, 4, 31, "Input",
  InitializationCell->True],
Cell[39348, 1251, 3622, 65, 1066, "Input",
  InitializationCell->True],
Cell[42973, 1318, 255, 5, 48, "Text"],
Cell[43231, 1325, 3687, 67, 1089, "Input",
  Evaluatable->False]
}, Closed]],

Cell[CellGroupData[{
Cell[46955, 1397, 40, 0, 38, "Subsubsection"],
Cell[46998, 1399, 70, 2, 31, "Input"],
Cell[47071, 1403, 121, 3, 31, "Input"],
Cell[47195, 1408, 71, 2, 31, "Input"],
Cell[47269, 1412, 73, 2, 31, "Input"],
Cell[47345, 1416, 70, 2, 31, "Input"],
Cell[47418, 1420, 104, 2, 31, "Input"],
Cell[47525, 1424, 70, 2, 31, "Input"],
Cell[47598, 1428, 70, 2, 31, "Input"],
Cell[47671, 1432, 95, 2, 31, "Input"],
Cell[47769, 1436, 93, 2, 31, "Input"],
Cell[47865, 1440, 93, 2, 31, "Input"],
Cell[47961, 1444, 95, 2, 31, "Input"],
Cell[48059, 1448, 88, 2, 31, "Input"],
Cell[48150, 1452, 88, 2, 31, "Input"],
Cell[48241, 1456, 99, 2, 31, "Input"],
Cell[48343, 1460, 102, 2, 31, "Input"],
Cell[48448, 1464, 99, 2, 31, "Input"],
Cell[48550, 1468, 110, 2, 31, "Input"],
Cell[48663, 1472, 113, 2, 31, "Input"],
Cell[48779, 1476, 73, 2, 31, "Input"],
Cell[48855, 1480, 153, 3, 31, "Input"],
Cell[49011, 1485, 134, 3, 31, "Input"],
Cell[49148, 1490, 107, 2, 31, "Input"],
Cell[49258, 1494, 90, 3, 28, "Text"],
Cell[49351, 1499, 85, 2, 31, "Input"],
Cell[49439, 1503, 136, 3, 31, "Input"],
Cell[49578, 1508, 170, 3, 54, "Input"],
Cell[49751, 1513, 144, 3, 31, "Input"],
Cell[49898, 1518, 185, 4, 54, "Input"],
Cell[50086, 1524, 136, 3, 31, "Input"],
Cell[50225, 1529, 127, 3, 31, "Input"],
Cell[50355, 1534, 134, 3, 31, "Input"],
Cell[50492, 1539, 86, 2, 31, "Input"],
Cell[50581, 1543, 86, 2, 31, "Input"],
Cell[50670, 1547, 86, 2, 31, "Input"],
Cell[50759, 1551, 129, 3, 31, "Input"],
Cell[50891, 1556, 146, 3, 31, "Input"],
Cell[51040, 1561, 75, 2, 31, "Input"],
Cell[51118, 1565, 111, 2, 31, "Input"],
Cell[51232, 1569, 146, 3, 31, "Input"],
Cell[51381, 1574, 95, 2, 31, "Input"],
Cell[51479, 1578, 67, 2, 31, "Input"],
Cell[51549, 1582, 77, 2, 31, "Input"],
Cell[51629, 1586, 161, 4, 31, "Input"],
Cell[51793, 1592, 77, 2, 31, "Input"],
Cell[51873, 1596, 112, 2, 31, "Input"],
Cell[51988, 1600, 161, 4, 31, "Input"],
Cell[52152, 1606, 98, 2, 31, "Input"],
Cell[52253, 1610, 85, 2, 31, "Input"],
Cell[52341, 1614, 161, 4, 31, "Input"],
Cell[52505, 1620, 121, 3, 31, "Input"],
Cell[52629, 1625, 161, 4, 31, "Input"],
Cell[52793, 1631, 114, 2, 31, "Input"],
Cell[52910, 1635, 161, 4, 31, "Input"],
Cell[53074, 1641, 68, 2, 31, "Input"],
Cell[53145, 1645, 70, 2, 31, "Input"],
Cell[53218, 1649, 159, 3, 31, "Input"],
Cell[53380, 1654, 77, 2, 31, "Input"],
Cell[53460, 1658, 82, 2, 31, "Input"],
Cell[53545, 1662, 161, 4, 31, "Input"],
Cell[53709, 1668, 68, 2, 31, "Input"],
Cell[53780, 1672, 81, 2, 31, "Input"],
Cell[53864, 1676, 66, 2, 31, "Input"],
Cell[53933, 1680, 80, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[54062, 1688, 40, 0, 40, "Subsection"],
Cell[54105, 1690, 476, 9, 68, "Text"],
Cell[54584, 1701, 73, 2, 31, "Input"],
Cell[54660, 1705, 116, 2, 31, "Input"],
Cell[54779, 1709, 155, 3, 28, "Text"],
Cell[54937, 1714, 121, 3, 31, "Input"],
Cell[55061, 1719, 91, 2, 31, "Input"],
Cell[55155, 1723, 92, 2, 31, "Input"],
Cell[55250, 1727, 107, 2, 31, "Input"],
Cell[55360, 1731, 82, 2, 31, "Input"],
Cell[55445, 1735, 83, 2, 31, "Input"],
Cell[55531, 1739, 102, 2, 31, "Input"],
Cell[55636, 1743, 74, 2, 31, "Input"],
Cell[55713, 1747, 103, 3, 28, "Text"],
Cell[55819, 1752, 228, 5, 77, "Input",
  InitializationCell->True],
Cell[56050, 1759, 195, 5, 31, "Input",
  InitializationCell->True],
Cell[56248, 1766, 87, 2, 31, "Input"],
Cell[56338, 1770, 134, 3, 31, "Input"],
Cell[56475, 1775, 75, 2, 31, "Input"],
Cell[56553, 1779, 69, 2, 31, "Input"],
Cell[56625, 1783, 69, 2, 31, "Input"],
Cell[56697, 1787, 75, 2, 31, "Input"],
Cell[56775, 1791, 126, 3, 31, "Input"],
Cell[56904, 1796, 64, 2, 31, "Input"],
Cell[56971, 1800, 78, 2, 31, "Input"],
Cell[57052, 1804, 78, 2, 31, "Input"],
Cell[57133, 1808, 78, 2, 31, "Input"],
Cell[57214, 1812, 78, 2, 31, "Input"],
Cell[57295, 1816, 167, 4, 28, "Text"],
Cell[57465, 1822, 66, 2, 31, "Input"],
Cell[57534, 1826, 75, 2, 31, "Input"],
Cell[57612, 1830, 98, 3, 28, "Text"],
Cell[57713, 1835, 92, 2, 31, "Input"],
Cell[57808, 1839, 109, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[57954, 1846, 115, 3, 40, "Subsection"],

Cell[CellGroupData[{
Cell[58094, 1853, 42, 0, 38, "Subsubsection"],
Cell[58139, 1855, 249, 5, 48, "Text"],
Cell[58391, 1862, 290, 5, 100, "Input",
  InitializationCell->True],
Cell[58684, 1869, 417, 9, 100, "Input",
  InitializationCell->True],
Cell[59104, 1880, 73, 0, 28, "Text"],
Cell[59180, 1882, 87, 2, 31, "Input"],
Cell[59270, 1886, 137, 3, 28, "Text"],
Cell[59410, 1891, 75, 2, 31, "Input"],
Cell[59488, 1895, 101, 2, 31, "Input"],
Cell[59592, 1899, 73, 2, 31, "Input"],
Cell[59668, 1903, 73, 2, 31, "Input"],
Cell[59744, 1907, 67, 0, 28, "Text"],
Cell[59814, 1909, 229, 5, 77, "Input",
  InitializationCell->True],
Cell[60046, 1916, 122, 3, 31, "Input",
  InitializationCell->True],
Cell[60171, 1921, 80, 2, 31, "Input"],
Cell[60254, 1925, 68, 2, 31, "Input"],
Cell[60325, 1929, 143, 3, 28, "Text"],
Cell[60471, 1934, 90, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[60598, 1941, 30, 0, 32, "Subsubsection"],
Cell[60631, 1943, 73, 2, 31, "Input"],
Cell[60707, 1947, 73, 2, 31, "Input"],
Cell[60783, 1951, 73, 2, 31, "Input"],
Cell[60859, 1955, 71, 2, 31, "Input"],
Cell[60933, 1959, 77, 2, 31, "Input"],
Cell[61013, 1963, 77, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[61127, 1970, 41, 0, 32, "Subsubsection"],
Cell[61171, 1972, 127, 3, 28, "Text"],
Cell[61301, 1977, 72, 2, 31, "Input"],
Cell[61376, 1981, 56, 0, 28, "Text"],
Cell[61435, 1983, 75, 2, 31, "Input"],
Cell[61513, 1987, 79, 2, 31, "Input"],
Cell[61595, 1991, 64, 2, 31, "Input"],
Cell[61662, 1995, 133, 3, 28, "Text"],
Cell[61798, 2000, 72, 2, 31, "Input"],
Cell[61873, 2004, 75, 2, 31, "Input"],
Cell[61951, 2008, 75, 2, 31, "Input"],
Cell[62029, 2012, 80, 2, 31, "Input"],
Cell[62112, 2016, 68, 2, 31, "Input"],
Cell[62183, 2020, 80, 2, 31, "Input"],
Cell[62266, 2024, 23, 0, 28, "Text"],
Cell[62292, 2026, 106, 2, 31, "Input"],
Cell[62401, 2030, 87, 3, 28, "Text"],
Cell[62491, 2035, 169, 4, 31, "Input"],
Cell[62663, 2041, 88, 2, 31, "Input"],
Cell[62754, 2045, 89, 2, 31, "Input"],
Cell[62846, 2049, 66, 2, 31, "Input"],
Cell[62915, 2053, 66, 2, 31, "Input"],
Cell[62984, 2057, 76, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[63097, 2064, 61, 0, 38, "Subsubsection"],
Cell[63161, 2066, 59, 0, 28, "Text"],
Cell[63223, 2068, 249, 5, 77, "Input",
  InitializationCell->True],
Cell[63475, 2075, 122, 3, 31, "Input",
  InitializationCell->True],
Cell[63600, 2080, 23, 0, 28, "Text"],
Cell[63626, 2082, 106, 2, 31, "Input"],
Cell[63735, 2086, 84, 2, 31, "Input"],
Cell[63822, 2090, 86, 2, 31, "Input"],
Cell[63911, 2094, 129, 3, 31, "Input"],
Cell[64043, 2099, 223, 5, 60, "Text"],
Cell[64269, 2106, 135, 3, 31, "Input",
  InitializationCell->True],
Cell[64407, 2111, 275, 5, 77, "Input",
  InitializationCell->True],
Cell[64685, 2118, 87, 2, 31, "Input"],
Cell[64775, 2122, 97, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[64933, 2131, 57, 0, 92, "Section"],
Cell[64993, 2133, 212, 4, 48, "Text"],

Cell[CellGroupData[{
Cell[65230, 2141, 36, 0, 38, "Subsubsection"],
Cell[65269, 2143, 232, 6, 28, "Text"],
Cell[65504, 2151, 159, 4, 28, "Text"],
Cell[65666, 2157, 113, 2, 31, "Input",
  Evaluatable->False],
Cell[65782, 2161, 1314, 27, 160, "Text"],
Cell[67099, 2190, 89, 3, 28, "Text"],
Cell[67191, 2195, 223, 5, 77, "Input",
  InitializationCell->True],
Cell[67417, 2202, 359, 7, 77, "Input",
  InitializationCell->True],
Cell[67779, 2211, 33, 0, 28, "Text"],
Cell[67815, 2213, 70, 2, 31, "Input"],
Cell[67888, 2217, 94, 2, 31, "Input"],
Cell[67985, 2221, 93, 2, 31, "Input"],
Cell[68081, 2225, 96, 2, 31, "Input"],
Cell[68180, 2229, 70, 2, 31, "Input"],
Cell[68253, 2233, 70, 0, 28, "Text"],
Cell[68326, 2235, 73, 2, 31, "Input"],
Cell[68402, 2239, 70, 2, 31, "Input"],
Cell[68475, 2243, 96, 3, 28, "Text"],
Cell[68574, 2248, 167, 4, 31, "Input",
  InitializationCell->True],
Cell[68744, 2254, 173, 4, 31, "Input",
  InitializationCell->True],
Cell[68920, 2260, 76, 2, 31, "Input"],
Cell[68999, 2264, 74, 2, 31, "Input"],
Cell[69076, 2268, 141, 3, 31, "Input"],
Cell[69220, 2273, 70, 2, 31, "Input"],
Cell[69293, 2277, 81, 2, 31, "Input"],
Cell[69377, 2281, 75, 0, 28, "Text"],
Cell[69455, 2283, 128, 3, 31, "Input"],
Cell[69586, 2288, 182, 4, 28, "Text"],
Cell[69771, 2294, 159, 4, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[69967, 2303, 43, 0, 32, "Subsubsection"],
Cell[70013, 2305, 97, 2, 31, "Input"],
Cell[70113, 2309, 100, 2, 31, "Input"],
Cell[70216, 2313, 68, 2, 31, "Input"],
Cell[70287, 2317, 94, 2, 31, "Input"],
Cell[70384, 2321, 104, 2, 31, "Input"],
Cell[70491, 2325, 64, 2, 31, "Input"],
Cell[70558, 2329, 109, 2, 31, "Input"],
Cell[70670, 2333, 70, 2, 31, "Input"],
Cell[70743, 2337, 109, 3, 28, "Text"],
Cell[70855, 2342, 68, 2, 31, "Input"],
Cell[70926, 2346, 68, 0, 28, "Text"],
Cell[70997, 2348, 138, 3, 31, "Input"],
Cell[71138, 2353, 59, 0, 28, "Text"],
Cell[71200, 2355, 106, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[71343, 2362, 99, 3, 32, "Subsubsection"],
Cell[71445, 2367, 165, 4, 31, "Input"],
Cell[71613, 2373, 77, 2, 31, "Input"],
Cell[71693, 2377, 65, 2, 31, "Input"],
Cell[71761, 2381, 90, 2, 31, "Input"],
Cell[71854, 2385, 78, 2, 31, "Input"],
Cell[71935, 2389, 96, 2, 31, "Input"],
Cell[72034, 2393, 77, 2, 31, "Input"],
Cell[72114, 2397, 68, 2, 31, "Input"],
Cell[72185, 2401, 69, 2, 31, "Input"],
Cell[72257, 2405, 131, 3, 28, "Text"],
Cell[72391, 2410, 102, 2, 31, "Input"],
Cell[72496, 2414, 50, 0, 28, "Text"],
Cell[72549, 2416, 76, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[72662, 2423, 86, 1, 32, "Subsubsection"],
Cell[72751, 2426, 135, 3, 46, "Text"],
Cell[72889, 2431, 70, 2, 30, "Input"],
Cell[72962, 2435, 95, 2, 30, "Input"],
Cell[73060, 2439, 95, 2, 30, "Input"],
Cell[73158, 2443, 100, 2, 30, "Input"],
Cell[73261, 2447, 78, 2, 30, "Input"],
Cell[73342, 2451, 71, 2, 30, "Input"],
Cell[73416, 2455, 112, 2, 30, "Input"],
Cell[73531, 2459, 70, 2, 30, "Input"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[73650, 2467, 48, 0, 92, "Section"],
Cell[73701, 2469, 142, 3, 28, "Text"],

Cell[CellGroupData[{
Cell[73868, 2476, 59, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[73952, 2480, 36, 0, 38, "Subsubsection"],
Cell[73991, 2482, 369, 6, 48, "Text"],
Cell[74363, 2490, 111, 2, 31, "Input"],
Cell[74477, 2494, 173, 4, 54, "Input"],
Cell[74653, 2500, 68, 2, 31, "Input"],
Cell[74724, 2504, 70, 2, 31, "Input"],
Cell[74797, 2508, 834, 19, 88, "Text"],
Cell[75634, 2529, 263, 7, 29, "Text"],
Cell[75900, 2538, 264, 5, 48, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[76201, 2548, 39, 0, 38, "Subsubsection"],
Cell[76243, 2550, 522, 8, 68, "Text"],
Cell[76768, 2560, 121, 3, 31, "Input"],
Cell[76892, 2565, 104, 3, 28, "Text"],
Cell[76999, 2570, 143, 3, 31, "Input"],
Cell[77145, 2575, 87, 2, 31, "Input"],
Cell[77235, 2579, 80, 2, 31, "Input"],
Cell[77318, 2583, 576, 11, 80, "Text"],
Cell[77897, 2596, 615, 12, 68, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[78561, 2614, 83, 1, 40, "Subsection"],
Cell[78647, 2617, 172, 4, 28, "Text"],
Cell[78822, 2623, 88, 2, 31, "Input"],
Cell[78913, 2627, 306, 6, 100, "Input",
  InitializationCell->True],
Cell[79222, 2635, 206, 5, 31, "Input",
  InitializationCell->True],
Cell[79431, 2642, 233, 4, 48, "Text"],
Cell[79667, 2648, 208, 5, 54, "Input"],
Cell[79878, 2655, 181, 4, 28, "Text"],
Cell[80062, 2661, 171, 4, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[80270, 2670, 51, 0, 40, "Subsection"],
Cell[80324, 2672, 682, 26, 50, "Text"],

Cell[CellGroupData[{
Cell[81031, 2702, 53, 0, 38, "Subsubsection"],
Cell[81087, 2704, 551, 21, 49, "Text"],
Cell[81641, 2727, 671, 10, 88, "Text"],
Cell[82315, 2739, 103, 2, 31, "Input",
  Evaluatable->False],
Cell[82421, 2743, 56, 0, 28, "Text"],
Cell[82480, 2745, 93, 2, 31, "Input"],
Cell[82576, 2749, 93, 2, 31, "Input"],
Cell[82672, 2753, 45, 0, 28, "Text"],
Cell[82720, 2755, 93, 2, 31, "Input"],
Cell[82816, 2759, 247, 8, 29, "Text"],
Cell[83066, 2769, 114, 2, 31, "Input"],
Cell[83183, 2773, 85, 2, 31, "Input"],
Cell[83271, 2777, 226, 7, 29, "Text"],
Cell[83500, 2786, 388, 14, 29, "Text"],
Cell[83891, 2802, 230, 6, 29, "Text"],
Cell[84124, 2810, 90, 2, 31, "Input"],
Cell[84217, 2814, 244, 7, 28, "Text"],
Cell[84464, 2823, 99, 2, 31, "Input"],
Cell[84566, 2827, 78, 2, 31, "Input"],
Cell[84647, 2831, 105, 3, 28, "Text"],
Cell[84755, 2836, 685, 25, 50, "Text"],
Cell[85443, 2863, 90, 2, 31, "Input"],
Cell[85536, 2867, 33, 0, 28, "Text"],
Cell[85572, 2869, 110, 2, 31, "Input"],
Cell[85685, 2873, 349, 9, 49, "Text"],
Cell[86037, 2884, 110, 2, 31, "Input"],
Cell[86150, 2888, 306, 11, 29, "Text"],
Cell[86459, 2901, 106, 2, 31, "Input"],
Cell[86568, 2905, 362, 12, 29, "Text"],
Cell[86933, 2919, 110, 2, 31, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[87080, 2926, 47, 0, 38, "Subsubsection"],
Cell[87130, 2928, 609, 20, 49, "Text"],
Cell[87742, 2950, 167, 4, 31, "Input"],
Cell[87912, 2956, 438, 11, 50, "Text"],
Cell[88353, 2969, 113, 2, 31, "Input"],
Cell[88469, 2973, 42, 0, 28, "Text"],
Cell[88514, 2975, 106, 2, 31, "Input"],
Cell[88623, 2979, 69, 2, 31, "Input"],
Cell[88695, 2983, 47, 0, 28, "Text"],
Cell[88745, 2985, 2932, 113, 105, "Text"],
Cell[91680, 3100, 411, 10, 50, "Text"],
Cell[92094, 3112, 81, 2, 31, "Input"],
Cell[92178, 3116, 341, 17, 29, "Text"],
Cell[92522, 3135, 102, 2, 31, "Input"],
Cell[92627, 3139, 82, 2, 31, "Input"],
Cell[92712, 3143, 839, 27, 70, "Text"],
Cell[93554, 3172, 101, 2, 31, "Input"],
Cell[93658, 3176, 449, 13, 53, "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[94156, 3195, 83, 1, 40, "Subsection"],
Cell[94242, 3198, 235, 5, 48, "Text"],
Cell[94480, 3205, 131, 3, 28, "Text"],
Cell[94614, 3210, 137, 3, 31, "Input"],
Cell[94754, 3215, 97, 2, 31, "Input"],
Cell[94854, 3219, 69, 2, 31, "Input"],
Cell[94926, 3223, 114, 2, 31, "Input"],
Cell[95043, 3227, 152, 3, 31, "Input"],
Cell[95198, 3232, 76, 0, 28, "Text"],
Cell[95277, 3234, 106, 2, 31, "Input"],
Cell[95386, 3238, 511, 13, 48, "Text"],

Cell[CellGroupData[{
Cell[95922, 3255, 148, 3, 38, "Subsubsection"],
Cell[96073, 3260, 70, 2, 31, "Input"],
Cell[96146, 3264, 116, 2, 31, "Input"],
Cell[96265, 3268, 132, 3, 31, "Input"],
Cell[96400, 3273, 132, 3, 28, "Text"],
Cell[96535, 3278, 117, 2, 31, "Input"],
Cell[96655, 3282, 203, 4, 48, "Text"],
Cell[96861, 3288, 70, 2, 31, "Input"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[96992, 3297, 72, 0, 92, "Section"],
Cell[97067, 3299, 1221, 27, 128, "Text"],

Cell[CellGroupData[{
Cell[98313, 3330, 54, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[98392, 3334, 71, 0, 38, "Subsubsection"],
Cell[98466, 3336, 222, 6, 28, "Text"],
Cell[98691, 3344, 125, 3, 31, "Input",
  InitializationCell->True],
Cell[98819, 3349, 91, 2, 31, "Input"],
Cell[98913, 3353, 101, 3, 28, "Text"],
Cell[99017, 3358, 141, 3, 31, "Input"],
Cell[99161, 3363, 69, 2, 31, "Input"],
Cell[99233, 3367, 186, 6, 28, "Text"],
Cell[99422, 3375, 130, 3, 31, "Input",
  InitializationCell->True],
Cell[99555, 3380, 112, 3, 28, "Text"],
Cell[99670, 3385, 114, 3, 31, "Input",
  InitializationCell->True],
Cell[99787, 3390, 21, 0, 28, "Text"],
Cell[99811, 3392, 102, 2, 31, "Input"],
Cell[99916, 3396, 168, 4, 31, "Input"],
Cell[100087, 3402, 69, 0, 28, "Text"],
Cell[100159, 3404, 82, 2, 31, "Input"],
Cell[100244, 3408, 73, 2, 31, "Input"],
Cell[100320, 3412, 107, 3, 28, "Text"],
Cell[100430, 3417, 155, 4, 31, "Input"],
Cell[100588, 3423, 58, 0, 28, "Text"],
Cell[100649, 3425, 100, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[100786, 3432, 131, 3, 32, "Subsubsection"],
Cell[100920, 3437, 246, 5, 48, "Text"],
Cell[101169, 3444, 288, 6, 100, "Input"],
Cell[101460, 3452, 44, 0, 28, "Text"],
Cell[101507, 3454, 425, 9, 77, "Input",
  InitializationCell->True],
Cell[101935, 3465, 19, 0, 28, "Text"],
Cell[101957, 3467, 126, 3, 45, "Input",
  CellTags->"skewop"],
Cell[102086, 3472, 73, 2, 31, "Input"],
Cell[102162, 3476, 95, 3, 28, "Text"],
Cell[102260, 3481, 98, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[102395, 3488, 56, 0, 32, "Subsubsection"],
Cell[102454, 3490, 147, 3, 28, "Text"],
Cell[102604, 3495, 142, 3, 31, "Input",
  InitializationCell->True],
Cell[102749, 3500, 82, 2, 31, "Input"],
Cell[102834, 3504, 73, 2, 31, "Input"],
Cell[102910, 3508, 102, 2, 31, "Input"],
Cell[103015, 3512, 179, 4, 28, "Text"],
Cell[103197, 3518, 107, 3, 31, "Input",
  InitializationCell->True],
Cell[103307, 3523, 80, 2, 31, "Input"],
Cell[103390, 3527, 68, 2, 31, "Input"],
Cell[103461, 3531, 72, 2, 31, "Input"],
Cell[103536, 3535, 70, 2, 31, "Input"],
Cell[103609, 3539, 73, 2, 31, "Input"],
Cell[103685, 3543, 70, 2, 31, "Input"],
Cell[103758, 3547, 150, 3, 28, "Text"],
Cell[103911, 3552, 82, 2, 31, "Input"],
Cell[103996, 3556, 87, 2, 31, "Input"],
Cell[104086, 3560, 244, 5, 60, "Text"],
Cell[104333, 3567, 600, 13, 169, "Input"],
Cell[104936, 3582, 69, 2, 31, "Input"],
Cell[105008, 3586, 151, 3, 28, "Text"],
Cell[105162, 3591, 84, 2, 31, "Input"],
Cell[105249, 3595, 448, 9, 146, "Input"],
Cell[105700, 3606, 92, 3, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[105829, 3614, 120, 3, 32, "Subsubsection"],
Cell[105952, 3619, 167, 4, 28, "Text"],
Cell[106122, 3625, 84, 2, 31, "Input"],
Cell[106209, 3629, 314, 6, 100, "Input"],
Cell[106526, 3637, 88, 3, 28, "Text"],
Cell[106617, 3642, 94, 2, 31, "Input"],
Cell[106714, 3646, 110, 2, 31, "Input"],
Cell[106827, 3650, 93, 3, 28, "Text"],
Cell[106923, 3655, 85, 2, 31, "Input"],
Cell[107011, 3659, 30, 0, 28, "Text"],
Cell[107044, 3661, 113, 2, 31, "Input"],
Cell[107160, 3665, 137, 4, 60, "Text"],
Cell[107300, 3671, 168, 4, 31, "Input"],
Cell[107471, 3677, 86, 2, 31, "Input"],
Cell[107560, 3681, 73, 2, 31, "Input"],
Cell[107636, 3685, 85, 2, 31, "Input"],
Cell[107724, 3689, 99, 2, 31, "Input"],
Cell[107826, 3693, 74, 2, 31, "Input"],
Cell[107903, 3697, 246, 5, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[108186, 3707, 29, 0, 32, "Subsubsection"],
Cell[108218, 3709, 133, 3, 28, "Text"],
Cell[108354, 3714, 98, 2, 31, "Input"],
Cell[108455, 3718, 95, 2, 31, "Input"],
Cell[108553, 3722, 57, 0, 28, "Text"],
Cell[108613, 3724, 69, 2, 31, "Input"],
Cell[108685, 3728, 329, 7, 48, "Text"],
Cell[109017, 3737, 135, 3, 31, "Input"],
Cell[109155, 3742, 101, 3, 28, "Text"],
Cell[109259, 3747, 104, 2, 31, "Input"],
Cell[109366, 3751, 70, 2, 31, "Input"],
Cell[109439, 3755, 72, 2, 31, "Input"],
Cell[109514, 3759, 86, 2, 31, "Input"],
Cell[109603, 3763, 70, 2, 31, "Input"],
Cell[109676, 3767, 131, 3, 45, "Input",
  CellTags->"torus"],
Cell[109810, 3772, 100, 2, 31, "Input"],
Cell[109913, 3776, 201, 4, 54, "Input"],
Cell[110117, 3782, 71, 2, 31, "Input"],
Cell[110191, 3786, 90, 3, 28, "Text"],
Cell[110284, 3791, 104, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[110425, 3798, 47, 0, 32, "Subsubsection"],
Cell[110475, 3800, 129, 3, 28, "Text"],
Cell[110607, 3805, 129, 3, 31, "Input"],
Cell[110739, 3810, 104, 2, 31, "Input"],
Cell[110846, 3814, 57, 0, 28, "Text"],
Cell[110906, 3816, 69, 2, 31, "Input"],
Cell[110978, 3820, 219, 4, 48, "Text"],
Cell[111200, 3826, 114, 2, 31, "Input"],
Cell[111317, 3830, 86, 2, 31, "Input"],
Cell[111406, 3834, 61, 0, 28, "Text"],
Cell[111470, 3836, 134, 3, 31, "Input"],
Cell[111607, 3841, 101, 3, 28, "Text"],
Cell[111711, 3846, 112, 2, 31, "Input"],
Cell[111826, 3850, 70, 2, 31, "Input"],
Cell[111899, 3854, 72, 2, 31, "Input"],
Cell[111974, 3858, 82, 2, 31, "Input"],
Cell[112059, 3862, 70, 2, 31, "Input"],
Cell[112132, 3866, 137, 3, 45, "Input",
  CellTags->"torus"],
Cell[112272, 3871, 92, 2, 31, "Input"],
Cell[112367, 3875, 188, 3, 54, "Input"],
Cell[112558, 3880, 71, 2, 31, "Input"],
Cell[112632, 3884, 90, 3, 28, "Text"],
Cell[112725, 3889, 96, 2, 31, "Input"],
Cell[112824, 3893, 70, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[112931, 3900, 88, 3, 32, "Subsubsection"],
Cell[113022, 3905, 379, 8, 80, "Text"],
Cell[113404, 3915, 114, 2, 31, "Input"],
Cell[113521, 3919, 132, 3, 28, "Text"],
Cell[113656, 3924, 91, 2, 31, "Input"],
Cell[113750, 3928, 70, 2, 31, "Input"],
Cell[113823, 3932, 76, 2, 31, "Input"],
Cell[113902, 3936, 50, 0, 28, "Text"],
Cell[113955, 3938, 150, 4, 31, "Input"],
Cell[114108, 3944, 82, 2, 31, "Input"],
Cell[114193, 3948, 70, 2, 31, "Input"],
Cell[114266, 3952, 78, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[114393, 3960, 125, 3, 40, "Subsection"],
Cell[114521, 3965, 552, 11, 68, "Text"],

Cell[CellGroupData[{
Cell[115098, 3980, 63, 0, 38, "Subsubsection"],
Cell[115164, 3982, 582, 10, 100, "Text"],
Cell[115749, 3994, 157, 4, 31, "Input",
  InitializationCell->True],
Cell[115909, 4000, 139, 3, 28, "Text"],
Cell[116051, 4005, 97, 2, 31, "Input"],
Cell[116151, 4009, 74, 2, 31, "Input"],
Cell[116228, 4013, 75, 2, 31, "Input"],
Cell[116306, 4017, 78, 2, 31, "Input"],
Cell[116387, 4021, 140, 3, 31, "Input"],
Cell[116530, 4026, 25, 0, 28, "Text"],
Cell[116558, 4028, 254, 6, 120, "Input"],
Cell[116815, 4036, 78, 2, 31, "Input"],
Cell[116896, 4040, 142, 3, 31, "Input"],
Cell[117041, 4045, 95, 2, 31, "Input"],
Cell[117139, 4049, 390, 7, 100, "Text"],
Cell[117532, 4058, 89, 2, 31, "Input"],
Cell[117624, 4062, 144, 3, 31, "Input"],
Cell[117771, 4067, 96, 2, 31, "Input"],
Cell[117870, 4071, 110, 2, 31, "Input"],
Cell[117983, 4075, 78, 2, 31, "Input"],
Cell[118064, 4079, 84, 2, 31, "Input"],
Cell[118151, 4083, 81, 2, 31, "Input"],
Cell[118235, 4087, 86, 2, 31, "Input"],
Cell[118324, 4091, 66, 2, 31, "Input"],
Cell[118393, 4095, 69, 0, 28, "Text"],
Cell[118465, 4097, 395, 8, 112, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[118897, 4110, 54, 0, 32, "Subsubsection"],
Cell[118954, 4112, 89, 2, 31, "Input"],
Cell[119046, 4116, 94, 2, 31, "Input"],
Cell[119143, 4120, 77, 2, 31, "Input"],
Cell[119223, 4124, 76, 2, 31, "Input"],
Cell[119302, 4128, 84, 2, 31, "Input"],
Cell[119389, 4132, 83, 2, 31, "Input"],
Cell[119475, 4136, 98, 2, 31, "Input"],
Cell[119576, 4140, 144, 3, 28, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[119769, 4149, 49, 0, 40, "Subsection"],
Cell[119821, 4151, 131, 3, 28, "Text"],

Cell[CellGroupData[{
Cell[119977, 4158, 65, 0, 38, "Subsubsection"],
Cell[120045, 4160, 251, 7, 28, "Text"],
Cell[120299, 4169, 135, 3, 31, "Input",
  InitializationCell->True],
Cell[120437, 4174, 71, 2, 31, "Input"],
Cell[120511, 4178, 34, 0, 28, "Text"],
Cell[120548, 4180, 84, 2, 31, "Input"],
Cell[120635, 4184, 142, 3, 28, "Text"],
Cell[120780, 4189, 127, 3, 31, "Input",
  InitializationCell->True],
Cell[120910, 4194, 83, 2, 31, "Input"],
Cell[120996, 4198, 132, 3, 28, "Text"],
Cell[121131, 4203, 135, 3, 45, "Input",
  CellTags->"eq4.2.1"],
Cell[121269, 4208, 302, 5, 48, "Text"],
Cell[121574, 4215, 135, 3, 31, "Input",
  InitializationCell->True],
Cell[121712, 4220, 68, 2, 31, "Input"],
Cell[121783, 4224, 334, 7, 62, "Text",
  CellTags->"diag"]
}, Closed]],

Cell[CellGroupData[{
Cell[122154, 4236, 69, 0, 38, "Subsubsection"],
Cell[122226, 4238, 74, 0, 28, "Text"],
Cell[122303, 4240, 194, 5, 31, "Input",
  InitializationCell->True],
Cell[122500, 4247, 80, 2, 31, "Input"],
Cell[122583, 4251, 41, 0, 28, "Text"],
Cell[122627, 4253, 71, 2, 31, "Input"],
Cell[122701, 4257, 79, 2, 31, "Input"],
Cell[122783, 4261, 120, 3, 31, "Input",
  InitializationCell->True],
Cell[122906, 4266, 80, 2, 31, "Input"],
Cell[122989, 4270, 155, 4, 45, "Input",
  InitializationCell->True,
  CellTags->"j2.1"],
Cell[123147, 4276, 421, 8, 80, "Text"],
Cell[123571, 4286, 102, 3, 28, "Text"],
Cell[123676, 4291, 104, 2, 31, "Input"],
Cell[123783, 4295, 87, 2, 31, "Input"],
Cell[123873, 4299, 89, 2, 31, "Input"],
Cell[123965, 4303, 145, 3, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[124147, 4311, 127, 3, 38, "Subsubsection"],
Cell[124277, 4316, 93, 3, 28, "Text"],
Cell[124373, 4321, 121, 3, 31, "Input"],
Cell[124497, 4326, 81, 2, 31, "Input"],
Cell[124581, 4330, 88, 2, 31, "Input"],
Cell[124672, 4334, 83, 2, 31, "Input"],
Cell[124758, 4338, 80, 2, 31, "Input"],
Cell[124841, 4342, 91, 2, 31, "Input"],
Cell[124935, 4346, 130, 3, 45, "Input",
  CellTags->"opj2"],
Cell[125068, 4351, 326, 6, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[125431, 4362, 155, 4, 52, "Subsubsection",
  CellTags->"B5"],
Cell[125589, 4368, 89, 3, 28, "Text"],
Cell[125681, 4373, 120, 3, 31, "Input"],
Cell[125804, 4378, 74, 2, 31, "Input"],
Cell[125881, 4382, 84, 2, 31, "Input"],
Cell[125968, 4386, 83, 2, 31, "Input"],
Cell[126054, 4390, 107, 2, 31, "Input"],
Cell[126164, 4394, 87, 2, 31, "Input"],
Cell[126254, 4398, 91, 2, 31, "Input"],
Cell[126348, 4402, 851, 15, 132, "Text"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[127260, 4424, 110, 3, 92, "Section"],

Cell[CellGroupData[{
Cell[127395, 4431, 117, 3, 46, "Subsection"],
Cell[127515, 4436, 859, 16, 196, "Text"],

Cell[CellGroupData[{
Cell[128399, 4456, 122, 3, 38, "Subsubsection"],
Cell[128524, 4461, 202, 4, 28, "Text"],
Cell[128729, 4467, 68, 2, 31, "Input"],
Cell[128800, 4471, 334, 6, 60, "Text"],
Cell[129137, 4479, 93, 2, 31, "Input"],
Cell[129233, 4483, 158, 3, 31, "Input"],
Cell[129394, 4488, 228, 4, 54, "Input"],
Cell[129625, 4494, 406, 9, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[130068, 4508, 124, 3, 38, "Subsubsection"],
Cell[130195, 4513, 94, 3, 28, "Text"],
Cell[130292, 4518, 165, 4, 31, "Input"],
Cell[130460, 4524, 83, 2, 31, "Input"],
Cell[130546, 4528, 82, 2, 31, "Input"],
Cell[130631, 4532, 93, 2, 31, "Input"],
Cell[130727, 4536, 93, 2, 31, "Input"],
Cell[130823, 4540, 594, 9, 88, "Text"],
Cell[131420, 4551, 109, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[131566, 4558, 146, 3, 38, "Subsubsection"],
Cell[131715, 4563, 181, 4, 31, "Input"],
Cell[131899, 4569, 83, 2, 31, "Input"],
Cell[131985, 4573, 82, 2, 31, "Input"],
Cell[132070, 4577, 79, 2, 31, "Input"],
Cell[132152, 4581, 72, 2, 31, "Input"],
Cell[132227, 4585, 91, 2, 31, "Input"],
Cell[132321, 4589, 344, 8, 48, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[132702, 4602, 231, 8, 38, "Subsubsection"],
Cell[132936, 4612, 179, 4, 28, "Text"],
Cell[133118, 4618, 173, 4, 31, "Input"],
Cell[133294, 4624, 83, 2, 31, "Input"],
Cell[133380, 4628, 82, 2, 31, "Input"],
Cell[133465, 4632, 82, 2, 31, "Input"],
Cell[133550, 4636, 93, 2, 31, "Input"],
Cell[133646, 4640, 110, 2, 31, "Input"],
Cell[133759, 4644, 70, 2, 31, "Input"],
Cell[133832, 4648, 225, 4, 48, "Text"],
Cell[134060, 4654, 103, 2, 31, "Input"],
Cell[134166, 4658, 98, 3, 28, "Text"],
Cell[134267, 4663, 106, 2, 31, "Input"],
Cell[134376, 4667, 183, 4, 31, "Input"],
Cell[134562, 4673, 252, 5, 48, "Text"],
Cell[134817, 4680, 181, 4, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[135047, 4690, 38, 0, 40, "Subsection"],
Cell[135088, 4692, 673, 10, 120, "Text"],

Cell[CellGroupData[{
Cell[135786, 4706, 74, 0, 38, "Subsubsection"],
Cell[135863, 4708, 166, 4, 31, "Input"],
Cell[136032, 4714, 80, 2, 31, "Input"],
Cell[136115, 4718, 79, 2, 31, "Input"],
Cell[136197, 4722, 92, 2, 31, "Input"],
Cell[136292, 4726, 80, 2, 31, "Input"],
Cell[136375, 4730, 65, 2, 31, "Input"],
Cell[136443, 4734, 122, 3, 45, "Input",
  CellTags->"j2.1"],
Cell[136568, 4739, 184, 4, 28, "Text"],
Cell[136755, 4745, 105, 2, 31, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[136897, 4752, 101, 3, 38, "Subsubsection"],
Cell[137001, 4757, 94, 3, 28, "Text"],
Cell[137098, 4762, 164, 4, 31, "Input"],
Cell[137265, 4768, 81, 2, 31, "Input"],
Cell[137349, 4772, 37, 0, 28, "Text"],
Cell[137389, 4774, 111, 2, 31, "Input"],
Cell[137503, 4778, 80, 2, 31, "Input"],
Cell[137586, 4782, 88, 2, 31, "Input"],
Cell[137677, 4786, 83, 2, 31, "Input"],
Cell[137763, 4790, 77, 2, 31, "Input"],
Cell[137843, 4794, 74, 2, 31, "Input"],
Cell[137920, 4798, 60, 0, 28, "Text"],
Cell[137983, 4800, 102, 2, 31, "Input"],
Cell[138088, 4804, 66, 2, 31, "Input"],
Cell[138157, 4808, 164, 4, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[138358, 4817, 67, 0, 38, "Subsubsection"],
Cell[138428, 4819, 142, 3, 28, "Text"],
Cell[138573, 4824, 95, 2, 31, "Input"],
Cell[138671, 4828, 82, 2, 31, "Input"],
Cell[138756, 4832, 96, 2, 31, "Input"],
Cell[138855, 4836, 83, 2, 31, "Input"],
Cell[138941, 4840, 82, 2, 31, "Input"],
Cell[139026, 4844, 82, 2, 31, "Input"],
Cell[139111, 4848, 90, 2, 31, "Input"],
Cell[139204, 4852, 83, 2, 31, "Input"],
Cell[139290, 4856, 80, 2, 31, "Input"],
Cell[139373, 4860, 87, 2, 31, "Input"],
Cell[139463, 4864, 70, 2, 31, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[139582, 4872, 53, 0, 40, "Subsection"],
Cell[139638, 4874, 235, 5, 48, "Text"],

Cell[CellGroupData[{
Cell[139898, 4883, 93, 3, 38, "Subsubsection"],
Cell[139994, 4888, 96, 2, 31, "Input"],
Cell[140093, 4892, 91, 2, 31, "Input"],
Cell[140187, 4896, 81, 2, 31, "Input"],
Cell[140271, 4900, 93, 2, 31, "Input"],
Cell[140367, 4904, 259, 7, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[140663, 4916, 113, 3, 38, "Subsubsection"],
Cell[140779, 4921, 142, 3, 31, "Input"],
Cell[140924, 4926, 79, 2, 31, "Input"],
Cell[141006, 4930, 86, 2, 31, "Input"],
Cell[141095, 4934, 83, 2, 31, "Input"],
Cell[141181, 4938, 89, 2, 31, "Input"],
Cell[141273, 4942, 93, 2, 31, "Input"],
Cell[141369, 4946, 127, 3, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[141533, 4954, 134, 3, 38, "Subsubsection"],
Cell[141670, 4959, 137, 3, 31, "Input"],
Cell[141810, 4964, 80, 2, 31, "Input"],
Cell[141893, 4968, 87, 2, 31, "Input"],
Cell[141983, 4972, 83, 2, 31, "Input"],
Cell[142069, 4976, 90, 2, 31, "Input"],
Cell[142162, 4980, 91, 2, 31, "Input"],
Cell[142256, 4984, 119, 3, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[142412, 4992, 73, 0, 38, "Subsubsection"],
Cell[142488, 4994, 138, 3, 31, "Input"],
Cell[142629, 4999, 77, 2, 31, "Input"],
Cell[142709, 5003, 76, 2, 31, "Input"],
Cell[142788, 5007, 84, 2, 31, "Input"],
Cell[142875, 5011, 83, 2, 31, "Input"],
Cell[142961, 5015, 87, 2, 31, "Input"],
Cell[143051, 5019, 93, 2, 31, "Input"],
Cell[143147, 5023, 256, 7, 28, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[143440, 5035, 77, 2, 38, "Subsubsection"],
Cell[143520, 5039, 179, 6, 28, "Text"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[143760, 5052, 110, 2, 92, "Section"],

Cell[CellGroupData[{
Cell[143895, 5058, 140, 3, 46, "Subsection"],
Cell[144038, 5063, 157, 4, 28, "Text"],
Cell[144198, 5069, 66, 2, 31, "Input"],
Cell[144267, 5073, 64, 2, 31, "Input"],
Cell[144334, 5077, 64, 2, 31, "Input"],
Cell[144401, 5081, 209, 6, 28, "Text"],
Cell[144613, 5089, 123, 3, 28, "Text"],
Cell[144739, 5094, 385, 6, 77, "Input"],
Cell[145127, 5102, 124, 3, 28, "Text"],
Cell[145254, 5107, 173, 3, 31, "Input"],
Cell[145430, 5112, 206, 4, 48, "Text"],
Cell[145639, 5118, 310, 5, 54, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[145986, 5128, 76, 1, 40, "Subsection"],

Cell[CellGroupData[{
Cell[146087, 5133, 37, 0, 38, "Subsubsection"],
Cell[146127, 5135, 214, 4, 72, "Input",
  InitializationCell->True],
Cell[146344, 5141, 173, 4, 30, "Input",
  InitializationCell->True],
Cell[146520, 5147, 144, 3, 30, "Input",
  InitializationCell->True],
Cell[146667, 5152, 165, 4, 30, "Input",
  InitializationCell->True],
Cell[146835, 5158, 177, 5, 30, "Input",
  InitializationCell->True],
Cell[147015, 5165, 160, 4, 30, "Input",
  InitializationCell->True],
Cell[147178, 5171, 119, 3, 30, "Input",
  InitializationCell->True],
Cell[147300, 5176, 208, 5, 51, "Input",
  InitializationCell->True],
Cell[147511, 5183, 115, 3, 30, "Input",
  InitializationCell->True],
Cell[147629, 5188, 318, 8, 72, "Input",
  InitializationCell->True],
Cell[147950, 5198, 574, 12, 156, "Input",
  InitializationCell->True],
Cell[148527, 5212, 207, 5, 51, "Input",
  InitializationCell->True],
Cell[148737, 5219, 253, 6, 72, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[149027, 5230, 39, 0, 38, "Subsubsection"],
Cell[149069, 5232, 193, 4, 51, "Input",
  InitializationCell->True],
Cell[149265, 5238, 169, 4, 33, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[149471, 5247, 51, 0, 38, "Subsubsection"],
Cell[149525, 5249, 62, 2, 30, "Input"],
Cell[149590, 5253, 208, 4, 72, "Input",
  InitializationCell->True],
Cell[149801, 5259, 233, 6, 51, "Input",
  InitializationCell->True],
Cell[150037, 5267, 94, 2, 30, "Input"],
Cell[150134, 5271, 96, 2, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[150267, 5278, 41, 0, 38, "Subsubsection"],
Cell[150311, 5280, 218, 5, 51, "Input",
  InitializationCell->True],
Cell[150532, 5287, 772, 15, 219, "Input",
  InitializationCell->True],
Cell[151307, 5304, 240, 5, 72, "Input",
  InitializationCell->True],
Cell[151550, 5311, 374, 8, 93, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[151961, 5324, 44, 0, 38, "Subsubsection"],
Cell[152008, 5326, 67, 2, 30, "Input"],
Cell[152078, 5330, 73, 2, 30, "Input"],
Cell[152154, 5334, 267, 5, 72, "Input",
  InitializationCell->True],
Cell[152424, 5341, 905, 16, 240, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[153366, 5362, 42, 0, 38, "Subsubsection"],
Cell[153411, 5364, 92, 2, 30, "Input"],
Cell[153506, 5368, 110, 2, 30, "Input"],
Cell[153619, 5372, 110, 2, 30, "Input"],
Cell[153732, 5376, 110, 2, 30, "Input"],
Cell[153845, 5380, 110, 2, 30, "Input"],
Cell[153958, 5384, 110, 2, 30, "Input"],
Cell[154071, 5388, 110, 2, 30, "Input"],
Cell[154184, 5392, 146, 3, 30, "Input"],
Cell[154333, 5397, 160, 4, 30, "Input"],
Cell[154496, 5403, 70, 2, 30, "Input"],
Cell[154569, 5407, 75, 2, 30, "Input"],
Cell[154647, 5411, 70, 2, 30, "Input"],
Cell[154720, 5415, 87, 2, 30, "Input"],
Cell[154810, 5419, 70, 2, 30, "Input"],
Cell[154883, 5423, 71, 2, 30, "Input"],
Cell[154957, 5427, 102, 2, 30, "Input"],
Cell[155062, 5431, 84, 2, 30, "Input"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[155195, 5439, 43, 0, 40, "Subsection"],

Cell[CellGroupData[{
Cell[155263, 5443, 41, 0, 38, "Subsubsection"],
Cell[155307, 5445, 140, 5, 27, "Text"],
Cell[155450, 5452, 181, 4, 30, "Input",
  InitializationCell->True],
Cell[155634, 5458, 128, 3, 30, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[155799, 5466, 39, 0, 38, "Subsubsection"],
Cell[155841, 5468, 204, 4, 51, "Input",
  InitializationCell->True],
Cell[156048, 5474, 142, 3, 30, "Input",
  InitializationCell->True],
Cell[156193, 5479, 85, 2, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[156315, 5486, 57, 0, 38, "Subsubsection"],
Cell[156375, 5488, 72, 2, 30, "Input"],
Cell[156450, 5492, 106, 3, 27, "Text"],
Cell[156559, 5497, 184, 4, 30, "Input",
  InitializationCell->True],
Cell[156746, 5503, 313, 7, 72, "Input",
  InitializationCell->True],
Cell[157062, 5512, 79, 2, 30, "Input"],
Cell[157144, 5516, 90, 2, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[157271, 5523, 74, 1, 39, "Subsubsection"],
Cell[157348, 5526, 154, 4, 29, "Text"],
Cell[157505, 5532, 205, 5, 51, "Input",
  InitializationCell->True],
Cell[157713, 5539, 454, 10, 133, "Input",
  InitializationCell->True],
Cell[158170, 5551, 126, 3, 33, "Input"],
Cell[158299, 5556, 126, 3, 33, "Input"],
Cell[158428, 5561, 139, 3, 33, "Input"],
Cell[158570, 5566, 126, 3, 33, "Input"],
Cell[158699, 5571, 115, 3, 33, "Input"],
Cell[158817, 5576, 281, 5, 46, "Text"],
Cell[159101, 5583, 128, 3, 33, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[159266, 5591, 40, 0, 38, "Subsubsection"],
Cell[159309, 5593, 230, 5, 72, "Input",
  InitializationCell->True],
Cell[159542, 5600, 171, 4, 33, "Input",
  InitializationCell->True],
Cell[159716, 5606, 124, 3, 33, "Input"],
Cell[159843, 5611, 128, 3, 33, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[160008, 5619, 72, 0, 38, "Subsubsection"],
Cell[160083, 5621, 175, 5, 27, "Text"],
Cell[160261, 5628, 233, 5, 72, "Input",
  InitializationCell->True],
Cell[160497, 5635, 260, 6, 72, "Input",
  InitializationCell->True],
Cell[160760, 5643, 82, 2, 30, "Input"],
Cell[160845, 5647, 82, 2, 30, "Input"],
Cell[160930, 5651, 108, 2, 30, "Input"],
Cell[161041, 5655, 106, 2, 30, "Input"],
Cell[161150, 5659, 65, 0, 27, "Text"],
Cell[161218, 5661, 82, 2, 30, "Input"],
Cell[161303, 5665, 56, 0, 27, "Text"],
Cell[161362, 5667, 86, 2, 30, "Input"],
Cell[161451, 5671, 60, 0, 27, "Text"],
Cell[161514, 5673, 87, 2, 30, "Input"],
Cell[161604, 5677, 122, 3, 27, "Text"],
Cell[161729, 5682, 86, 2, 30, "Input"],
Cell[161818, 5686, 52, 1, 30, "Input"],
Cell[161873, 5689, 126, 4, 27, "Text"],
Cell[162002, 5695, 246, 6, 72, "Input",
  InitializationCell->True],
Cell[162251, 5703, 75, 2, 30, "Input"]
}, Closed]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[162387, 5712, 56, 1, 32, "Subsubtitle"],
Cell[162446, 5715, 203, 5, 42, "Text",
  CellTags->"Jag1"],
Cell[162652, 5722, 152, 3, 28, "Text"],
Cell[162807, 5727, 113, 3, 28, "Text"],
Cell[162923, 5732, 120, 4, 42, "Text",
  CellTags->"Mal"],
Cell[163046, 5738, 177, 5, 42, "Text",
  CellTags->"OSIII"]
}, Closed]],

Cell[CellGroupData[{
Cell[163260, 5748, 53, 1, 24, "Subsubtitle"],
Cell[163316, 5751, 93, 2, 42, "Text",
  CellTags->"address"],
Cell[163412, 5755, 137, 3, 42, "Text",
  CellTags->"hp"]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

